﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert ein Messresultat in Bezug auf eine Datenmenge.
    /// </summary>
    public class CwlbMeasurementDetail 
    {
        /// <summary>
        /// Liefert eine Zeitdifferenz mit Wert 0.
        /// </summary>
        private static TimeSpan TimeSpanZero = TimeSpan.FromTicks(0);

        /// <summary>
        /// Liefert oder definiert die Anzahl Bytes für die Messreihe aufgebaut wird.
        /// </summary>
        public bool ExcludeFirst { get; set; }

        /// <summary>
        /// Liefert oder definiert die Datengrösse für dieses Detail.
        /// </summary>
        public int DataLength { get; set; }

        #region Eigenschaften der Messreihe
        /// <summary>
        /// Liste für die Aufnahme der Messwerte 1 Domäne in process.
        /// </summary>
        public List<TimeSpan> TimeSpanCollection { get; private set; }

        /// <summary>
        /// Gets or sets the instance counter of the test objects for this test.
        /// </summary>
        public int InstanceCounter { get; set; }

        /// <summary>
        /// Durchschnittliche Zwit der Messreihe.
        /// </summary>
        public TimeSpan TimeSpanAvarage
        {
            get
            {
                if (TimeSpanCollection.Count == 0)
                {
                    return TimeSpanZero;
                }

                if (ExcludeFirst)
                {
                    double dTicksAverage = TimeSpanCollection.Skip(1).Average(item => item.Ticks);
                    return TimeSpan.FromTicks((long)dTicksAverage);
                }
                else
                {
                    double dTicksAverage = TimeSpanCollection.Average(item => item.Ticks);
                    return TimeSpan.FromTicks((long)dTicksAverage);
                }
            }
        }

        /// <summary>
        /// Erster Wert der Messreihe.
        /// </summary>
        public TimeSpan TimeSpanFirst
        {
            get
            {
                if (TimeSpanCollection.Count == 0)
                {
                    return TimeSpanZero;
                }
                return TimeSpanCollection[0];
            }
        }

        /// <summary>
        /// Minimaler Wert der Messreiehe.
        /// </summary>
        public TimeSpan TimeSpanMin
        {
            get
            {
                if (TimeSpanCollection.Count == 0)
                {
                    return TimeSpanZero;
                }

                if (ExcludeFirst)
                {
                    return TimeSpanCollection.Skip(1).Min();
                }
                else
                {
                    return TimeSpanCollection.Min();
                }
            }
        }

        /// <summary>
        /// Maximaler Wert der Messreiehe.
        /// </summary>
        public TimeSpan TimeSpanMax
        {
            get
            {
                if (TimeSpanCollection.Count == 0)
                {
                    return TimeSpanZero;
                }

                if (ExcludeFirst)
                {
                    return TimeSpanCollection.Skip(1).Max();
                }
                else
                {
                    return TimeSpanCollection.Max();
                }
            }
        }
        #endregion

        /// <summary>
        /// Liefert oder definiert das Datenobjekt für die Messungen dieses Details.
        /// </summary>
        public CwlbDemoData DataObejct { get; set; }

        /// <summary>
        /// Initialisiert ein Detail für den Aufbau einer Messreihe.
        /// </summary>
        /// <param name="bExcludeFirst">Definiert ob der erste Wert in den Resultaten ausgeschlossen werden soll.</param>
        public CwlbMeasurementDetail(bool bExcludeFirst, int iDataLength, CwlbDemoData objData)
        {
            ExcludeFirst = bExcludeFirst;
            DataLength = iDataLength;
            DataObejct = objData;
            TimeSpanCollection = new List<TimeSpan>();
        }
    }
}
