﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Repräsentiert ein View-Element für die Zusammenfassung der Messungen.
    /// </summary>
    public class CwlbOverviewDetailView: CwlbViewBase
    {
        /// <summary>
        /// Hält den Durchschnittswert für die Messung in 1 Domäne in einem Prozessraum.
        /// </summary>
        private TimeSpan _tsInProcessAvarage;

        /// <summary>
        /// Hält den Durchschnittswert für die Messung in 2 Domänen in einem Prozessraum.
        /// </summary>
        private TimeSpan _tsDomainInProcessAvarage;

        /// <summary>
        /// Hält den Durchschnittswert für die Messung in 2 Prozessen auf einer Maschine.
        /// </summary>
        private TimeSpan _tsProcessLocalAvarage;

        /// <summary>
        /// Hält den Durchschnittswert für die Messung in 2 Prozessen auf zwei Maschinen.
        /// </summary>
        private TimeSpan _tsProcessRemoteAvarage;

        /// <summary>
        /// Liefert oder definiert die Datengrösse der Messreihe.
        /// </summary>
        public int DataLength { get; set; }

        /// <summary>
        /// Liefert oder definiert den durchschnittlichen Messwert für die Messung mit 1 Domäne in einem Prozess.
        /// </summary>
        public TimeSpan TimeSpanInProcessAvarage
        {
            get { return _tsInProcessAvarage; }
            set
            {
                _tsInProcessAvarage = value;
                OnPropertyChanged("TimeSpanInProcessAvarage");
            }
        }

        /// <summary>
        /// Liefert oder definiert den durchschnittlichen Messwert für die Messung mit 2 Domänen in einem Prozessraum.
        /// </summary>
        public TimeSpan TimeSpanDomainInProcessAvarage
        {
            get { return _tsDomainInProcessAvarage; }
            set
            {
                _tsDomainInProcessAvarage = value;
                OnPropertyChanged("TimeSpanDomainInProcessAvarage");
            }
        }

        /// <summary>
        /// Liefert oder definiert den durchschnittlichen Messwert für die Messung mit 2 Prozessen auf 1 Maschinen.
        /// </summary>
        public TimeSpan TimeSpanProcessLocalAvarage
        {
            get { return _tsProcessLocalAvarage; }
            set
            {
                _tsProcessLocalAvarage = value;
                OnPropertyChanged("TimeSpanProcessLocalAvarage");
            }
        }

        /// <summary>
        /// Liefert oder definiert den durchschnittlichen Messwert für die Messung mit 2 Prozessen auf 2 Maschinen.
        /// </summary>
        public TimeSpan TimeSpanProcessRemoteAvarage
        {
            get { return _tsProcessRemoteAvarage; }
            set
            {
                _tsProcessRemoteAvarage = value;
                OnPropertyChanged("TimeSpanProcessRemoteAvarage");
            }
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbOverviewDetailView.
        /// </summary>
        /// <param name="iDataLength">Grösse der Daten für die das View-Element verwendung findet.</param>
        public CwlbOverviewDetailView(int iDataLength)
        {
            DataLength = iDataLength;
            OnPropertyChanged("DataLength");
        }
    }
}
