﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Stellt eine Klasse für die Verwendung des Servers zur Verfügung.
    /// </summary>
    public class CwlbPerformanceService : IwlbPerformanceService
    {
        /// <summary>
        /// Liefert den Namen der Kommunikationskonfiguration für die Verwendung von named pipes.
        /// </summary>
        public static string NAMED_PIPE = "named_pipe";

        /// <summary>
        /// Definiert den Namen der Kommunikationskonfiguration für die Vewendung von TCP/IP.
        /// </summary>
        public static string TCP_IP = "tcp_ip";

        /// <summary>
        /// Definiert den Namen des Automationsmutex für den Serverbetrieb.
        /// </summary>
        public static string AUTOMATION_MUTEX_NAME = "WeroSoft.Samples.LearningWcf.PerformanceTests";

        /// <summary>
        /// Defininiert die domänenspezifische Stoppuhr.
        /// </summary>
        private static Stopwatch _objStopwatch = Stopwatch.StartNew();

        /// <summary>
        /// Routing Methode für die Verwendung für der Verarbeitung der Daten.
        /// </summary>
        /// <param name="objData">Datenobjekt, das für die Verarbeitung verwendet wird.</param>
        /// <returns>Datenobjekt, das für die Verarbeitung verwendet wurde.</returns>
        public CwlbDemoData Modify(CwlbDemoData objData)
        {
                return CwlbDemoData.Modify(objData);
        }

        /// <summary>
        /// Gets the amount of instances created in the domain.
        /// </summary>
        /// <returns>Count of test instances within the domain.</returns>
        public int GetInstanceCounter()
        {
            return CwlbDemoData.InstanceCounter;
        }

        /// <summary>
        /// Resets the instance counter in the domain.
        /// </summary>
        public void ResetInstanceCounter()
        {
            CwlbDemoData.InstanceCounter = 0;
        }

        /// <summary>
        /// Returns the machine name of the server.
        /// </summary>
        /// <returns></returns>
        public string Ping()
        {
            return Environment.MachineName;
        }
    }
}
