﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.Threading;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert den Server für die 
    /// </summary>
    public class Program
    {
        static void Main(string[] args)
        {
            ServiceHost objHost = null;

            try
            {
                objHost = new ServiceHost(typeof(CwlbPerformanceService));

                if (args.Length < 1)
                {
                    MakeInfo();
                    ConfigureNamedPipe(objHost);
                }
                else if (args[0].ToLower() == CwlbPerformanceService.NAMED_PIPE)
                {
                    ConfigureNamedPipe(objHost);
                }
                else if (args[0].ToLower() == CwlbPerformanceService.TCP_IP)
                {
                    ConfigureTCP(objHost);
                }
                else
                {
                    ConfigureNamedPipe(objHost);
                }

                // Host starten
                objHost.Open();

                // Prüfen ob der Host programmatisch wieder beendet werden soll
                try
                {
                    Mutex objMutex = Mutex.OpenExisting(CwlbPerformanceService.AUTOMATION_MUTEX_NAME);
                    // Der Mutex war erstellt und kann benutzt werden
                    objMutex.WaitOne(1000 * 60 * 3);
                    // Die Beendigung wurde signalisert.
                }
                catch (WaitHandleCannotBeOpenedException)
                {
                    // Der Mutex konnte nicht erreicht werden, es wird eine händische 
                    // Serverkontrolle vorgenommen.
                    Console.WriteLine(string.Format("Maschinenname: {0}", Environment.MachineName));
                    Console.WriteLine("Host gestartet ...");
                    Console.ReadLine();
                }
            }
            catch (Exception excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            finally
            {
                if (objHost != null && objHost.State == CommunicationState.Opened)
                {
                    objHost.Close();
                }
            }
        }

        private static void ConfigureNamedPipe(ServiceHost objHost)
        {
            NetNamedPipeBinding objBinding = new NetNamedPipeBinding(NetNamedPipeSecurityMode.None);
            objBinding.ReaderQuotas.MaxArrayLength = int.MaxValue;
            objBinding.MaxReceivedMessageSize = int.MaxValue;
            objHost.AddServiceEndpoint(typeof(IwlbPerformanceService), objBinding, "net.pipe://localhost/LearningWcf");
        }

        private static void ConfigureTCP(ServiceHost objHost)
        {
            NetTcpBinding objBinding = new NetTcpBinding(SecurityMode.None);
            objBinding.ReaderQuotas.MaxArrayLength = int.MaxValue;
            objBinding.MaxReceivedMessageSize = int.MaxValue;
            objHost.AddServiceEndpoint(typeof(IwlbPerformanceService), objBinding, "net.tcp://localhost:5678/LearningWcf");
        }

        private static void MakeInfo()
        {
            ConsoleColor clrCurrent = Console.ForegroundColor;
            Console.ForegroundColor = ConsoleColor.Red;
            Console.WriteLine("Der Server muss mit mit einem Parameter gestartet werden:");
            Console.WriteLine("   named_pipe -> Start für Nutzung von NamedPipe auf der Maschine");
            Console.WriteLine("   tcp_ip     -> Start als Server für Nutzung über TCP/IP");
            Console.WriteLine();
            Console.WriteLine("Es wird der Standardwert named_pipe konfiguriert.");
            Console.ForegroundColor = clrCurrent;
        }
    }
}
