﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.ComponentModel;
using System.ServiceModel;
using System.Diagnostics;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Modelltest
    /// </summary>
    [TestClass]
    public class CwlbModelTest
    {
        /// <summary>
        /// Liefert oder definiert den Testkontext-
        /// </summary>
        public TestContext TestContext {get; set; }

        /// <summary>
        /// Test die Erstellung des Testmodells.
        /// </summary>
        [TestMethod]
        public void CreateModel()
        {
            // Lokale Daten
            BackgroundWorker objWorker = new BackgroundWorker();

            // Erstellen des Modells
            CwlbMeasurementModel objModel = new CwlbMeasurementModel();
            Assert.IsNotNull(objModel, "Das Modell darf nicht der Instanzierung nicht null sein.");
            Assert.IsNotNull(objModel.LocalMachineName, "Der Maschinenname darf nach der Erstellung nicht null sein.");
            Assert.IsNotNull(objModel.IpAddressLocalMachine, "Die IP Adresse darf nach der Erstellung nicht null sein.");

            // Initialisieren des Modells
            objModel.InitializeMeasurement(objWorker, 16777216, true);
            Assert.IsNotNull(objModel.DataLengthCollection, "Die DataLengthColelction ist nicht initialisert.");
            Assert.AreEqual<int>(objModel.DataLengthCollection.Count, 19, "Es sind zuwenige Dateneinträge generiert worden.");
            Assert.AreEqual<int>(objModel.MeasurementCaseCollection.Count,4, "Dei Anzahl der Messfälle stimmt nicht.");
            foreach (int iValue in objModel.DataLengthCollection)
            {
                Assert.AreNotEqual<int>(0, iValue, "Eine Datenlänge ist nicht korrekt initialisiert.");
            }
        }

        /// <summary>
        /// Perfromancetest für manuelle Geschwindigkeitsmessung.
        /// </summary>
        [TestMethod]
        public void PerformanceTest()
        {
            IwlbPerformanceService objProxy = null;
            try
            {
                Stopwatch objStopwatch = Stopwatch.StartNew();
                NetTcpBinding objBinding = new NetTcpBinding(SecurityMode.None);
                objBinding.ReaderQuotas.MaxArrayLength = int.MaxValue;
                objBinding.MaxReceivedMessageSize = int.MaxValue;
                objProxy = ChannelFactory<IwlbPerformanceService>.CreateChannel(objBinding, new EndpointAddress("net.tcp://localhost:5678/LearningWcf"));
                CwlbDemoData objData = new CwlbDemoData();
                objData.BinaryData = new byte[1024 * 1024 * 16];
                new Random().NextBytes(objData.BinaryData);
                objStopwatch.Restart();
                objData = objProxy.Modify(objData);
                TestContext.WriteLine( "Gemessene Zeit: {0}", objStopwatch.Elapsed);
            }
            catch (Exception excObject)
            {
            }
            finally
            {
            }
        }
    }
}
