﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;

namespace WeroSoft.Samples.Data
{
    /// <summary>
    /// Datenvertrag für das Beispiel
    /// </summary>
    [DataContract]
    public class CwlbDemoData
    {
        /// <summary>
        /// Ganzzahliger Wert für die Demo.
        /// </summary>
        [DataMember]
        public int IntegerValue;

        /// <summary>
        /// Zeichnfolge für die Demo.
        /// </summary>
        [DataMember]
        public string StringValue { get; set; }

        /// <summary>
        /// Datum-Zeit-Wert für die Demo.
        /// </summary>
        [DataMember]
        public DateTime DateTimeValue { get; set; }

        /// <summary>
        /// Eigener Auflistungswert für die Demo.
        /// </summary>
        [DataMember]
        public EwlbWorkerStatus Status { get; set; }
    }

    /// <summary>
    /// Definition der eigenen Auflistung.
    /// </summary>
    public enum EwlbWorkerStatus
    {
        Pending,
        Running,
        Ended
    }

    /// <summary>
    /// Fehlervertrag für dei Demo.
    /// </summary>
    [DataContract]
    public class CwlbFault
    {
        /// <summary>
        /// Codedefinition.
        /// </summary>
        [DataMember]
        public int Code;

        /// <summary>
        /// Nähere Bezeichnung des Fehlers.
        /// </summary>
        [DataMember]
        public string Message { get; set; }

        /// <summary>
        /// Zeitpuntk wann der Fehler erzeugt wurde.
        /// </summary>
        /// <remarks>
        /// Das sind frei erfundene Demodaten für einen Fehlervertrag.
        /// </remarks>
        [DataMember]
        public DateTime Created { get; set; }

        /// <summary>
        /// Status der Informationsverarbeitung.
        /// </summary>
        /// <remarks>
        /// Das sind frei erfundene Demodaten für einen Fehlervertrag.
        /// </remarks>
        [DataMember]
        public EwlbWorkerStatus Status { get; set; }
    }
}
