﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;
using WeroSoft.Samples.Data;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Klasse für die Ausführungskontrolle der Anwendung.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Einstiegspunkt in das Programm.
        /// </summary>
        static void Main()
        {
            // Deklaration der Proxy-Objekte
            IwlbDemoWebService objProxy = null;
            IwlbDemoWebService objProxyMtom = null;

            try
            {
                // Definieren Sie hier die Adresse des Servers, wenn Sie das Beispiel
                // verteilt über zwei Machinen benutzen. Für eine lokale Benutzung von
                // Server und Client auf der gleichen Maschine verwenden Sie "localhost";
                // string strHostAddress = "172.16.172.2";
                string strHostAddress = "localhost";

                // Normalen Client erstellen
                objProxy = ChannelFactory<IwlbDemoWebService>.CreateChannel(new BasicHttpBinding(),
                    new EndpointAddress(string.Format("http://{0}:4711/WebServiceDemo", strHostAddress)));

                // Client mit Mtom verwenden
                BasicHttpBinding objBinding = new BasicHttpBinding();
                objBinding.MessageEncoding = WSMessageEncoding.Mtom;
                objProxyMtom = ChannelFactory<IwlbDemoWebService>.CreateChannel(objBinding,
                    new EndpointAddress(string.Format("http://{0}:4711/WebServiceDemoStreaming", strHostAddress)));

                // Aufruf ohne Fehler
                CwlbDemoData objData = new CwlbDemoData();
                CwlbDemoData[] aobjData = objProxy.GetDemoData(objData, 2, "Testdaten");
                CwlbDemoData[] aobjDataMtom = objProxyMtom.GetDemoData(objData, 2, "Testdaten");


                // Aufruf mit Fehler
                aobjData = objProxy.GetDemoData(objData, -2, "Testdaten");
            }
            catch (FaultException<CwlbFault> excObject)
            {
                Console.WriteLine("Eigener Fehler gefangen:");
                Console.WriteLine(string.Format("   Fehlertext:          {0}", excObject.Message));
                Console.WriteLine(string.Format("   Fehlergrund:         {0}", excObject.Reason));
                Console.WriteLine(string.Format("   Fehlercode:          {0}", excObject.Code.Name));
                Console.WriteLine(string.Format("   Text Fehlerdetail:   {0}", excObject.Detail.Message));
                Console.WriteLine(string.Format("   Code Fehlerdetail:   {0}", excObject.Detail.Code));
                Console.WriteLine(string.Format("   Datum Fehlerdetail:  {0}", excObject.Detail.Created));
                Console.WriteLine(string.Format("   Status Fehlerdetail: {0}", excObject.Detail.Status));
            }
            catch (Exception excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            finally
            {
                if (objProxy != null && ((IChannel)objProxy).State == CommunicationState.Opened)
                {
                    ((IChannel)objProxy).Close();
                }
            }
        }
    }
}
