﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert den Client.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Einsprungpunkt in das Programm.
        /// </summary>
        static void Main()
        {
            IwlbDemoService objProxy = null;

            try
            {
                // Definieren Sie hier die Adresse des Servers, wenn Sie das Beispiel
                // verteilt über zwei Machinen benutzen. Für eine lokale Benutzung von
                // Server und Client auf der gleichen Maschine verwenden Sie "localhost";
                // string strHostAddress = "172.16.172.2";
                string strHostIPAddress = "localhost";

                // Normalen Client erstellen
                objProxy = ChannelFactory<IwlbDemoService>.CreateChannel(
                    new NetTcpBinding(SecurityMode.None),
                    new EndpointAddress(
                        string.Format("net.tcp://{0}:4711/{1}",
                            strHostIPAddress,
                            GwlbLearningWcf.WcfDemoServiceName)));

                // Aufruf ohne Fehler
                Console.WriteLine("Aufruf ohne Fehler ...");
                CwlbDemoData objData = new CwlbDemoData { DateTimeValue = DateTime.Now, StringValue = "Anfrage" };
                CwlbDemoData[] aobjData = objProxy.GetDemoData(objData, 20);
                foreach (CwlbDemoData objItem in aobjData)
                {
                    Console.WriteLine(objItem);
                }

                // Aufruf mit Fehler
                Console.WriteLine();
                Console.WriteLine("Aufruf mit serverseitigem Fehler ...");
                aobjData = objProxy.GetDemoData(objData, -2);
            }
            catch (FaultException<CwlbFault> excObject)
            {
                // Verarbeitung des Serverfehlers
                Console.WriteLine("Eigener Fehler gefangen:");
                Console.WriteLine(string.Format("   Fehlertext:          {0}", excObject.Message));
                Console.WriteLine(string.Format("   Fehlergrund:         {0}", excObject.Reason));
                Console.WriteLine(string.Format("   Fehlercode:          {0}", excObject.Code.Name));
                Console.WriteLine(string.Format("   Text Fehlerdetail:   {0}", excObject.Detail.Message));
                Console.WriteLine(string.Format("   Code Fehlerdetail:   {0}", excObject.Detail.Code));
                Console.WriteLine(string.Format("   Datum Fehlerdetail:  {0}", excObject.Detail.Created));
            }
            catch (CommunicationException excObject)
            {
                // Verarbeitung einer allgmeinen Kommunikationsausnahme
                Console.WriteLine(excObject.Message);
            }
            catch (Exception excObject)
            {
                // Verarbeitung aller andern Ausnahmen
                Console.WriteLine(excObject.Message);
            }
            finally
            {
                // Beenden des Clients im Normalfall
                if (objProxy != null && ((IChannel)objProxy).State == CommunicationState.Opened)
                {
                    ((IChannel)objProxy).Close();
                }
                // Beenden des Clients im Ausnahmefall
                if (objProxy != null && ((IChannel)objProxy).State == CommunicationState.Faulted)
                {
                    ((IChannel)objProxy).Abort();
                }
            }
        }
    }
}
