﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert den Service Host.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Einsprungpunkt in das Programm.
        /// </summary>
        static void Main()
        {
            // Definiert den Service Host.
            ServiceHost objHost = null;

            try
            {
                // Erstellen des Diensthosts für den Dienst CwlbDemoService
                objHost = new ServiceHost(typeof(CwlbDemoService));
                // Defineren eines Kommunikationsendpunktes
                objHost.AddServiceEndpoint(
                    typeof(IwlbDemoService),
                    new NetTcpBinding(SecurityMode.None), 
                    string.Format("net.tcp://localhost:4711/{0}", GwlbLearningWcf.WcfDemoServiceName));

                // Mex Dienstkonfiguration herstellen
                ServiceMetadataBehavior objMexBehavior = objHost.Description.Behaviors.Find<ServiceMetadataBehavior>();
                if (objMexBehavior == null)
                {
                    // Die Mex Dienstkonfiguration steht noch nicht zur Verfügung, 
                    // ensprechend ergänzen wir sie imperativ
                    objMexBehavior = new ServiceMetadataBehavior();
                    objHost.Description.Behaviors.Add(objMexBehavior);
                }

                objHost.AddServiceEndpoint(
                    typeof(IMetadataExchange),
                    new NetTcpBinding(SecurityMode.None),
                    string.Format("net.tcp://localhost:4711/mex"));


                // Diensthost starten und verhindern, dass der Prozess autoamtisch beendet wird
                objHost.Open();
                Console.WriteLine("Service Host ist gestartet. Mit einem Tastendruck kann er beendet werden ...");
                Console.ReadKey(true);
            }
            catch (Exception excObject)
            {
                // Ausnahmebehandlung
                Console.WriteLine(excObject.Message);
            }
            finally
            {
                // Beenden des Dienstes im Normalfall
                if (objHost != null && objHost.State == CommunicationState.Opened)
                {
                    objHost.Close();
                }
                // Beenden des Dienstes im Ausnahmefall
                if (objHost != null && objHost.State == CommunicationState.Faulted)
                {
                    objHost.Abort();
                }
            }
        }
    }
}
