﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert die Schnittstelle des Dienstes.
    /// </summary>
    [ServiceContract(Namespace="http://WeroSoft.net/Samples/LearningWcf", Name="DemoService")]
    [DataContractFormat(Style=OperationFormatStyle.Document)]
    public interface IwlbDemoService
    {
        /// <summary>
        /// Liefert eine Identifikation des angesprochenen Dienstes.
        /// </summary>
        /// <returns></returns>
        [OperationContract(Action="ApplicationPing", ReplyAction="ApplicationPingResponse", Name="ApplicationPing")]
        string Ping();

        /// <summary>
        /// Liefert einen zufälligen Wert zwischen den definierten Grenzen.
        /// </summary>
        /// <param name="iLowerBoundary">Untere Grenze des Wertebereichs aus dem der zufällige Wert ausgewählt wird.</param>
        /// <param name="iUpperBoundary">Obere Grenze des Wertebereichs aus dem der zufällige Wert ausgewählt wird.</param>
        /// <returns>Zufälliger Wert.</returns>
        [OperationContract]
        [return: MessageParameter(Name="RandomValue")]
        int GetRandomValue(
            [MessageParameter(Name = "lowerBoundary")]int iLowerBoundary,
            [MessageParameter(Name = "upperBoundary")]int iUpperBoundary);

        /// <summary>
        /// Liefert ein Array von Daten.
        /// </summary>
        /// <param name="demoData">Datenobjekt das als Vorlage vom Client an den Server gesendet wird.</param>
        /// <param name="iParam">Anzahl der zu generierenden Elemente in der Rückgabe. Dieser WErt darf nicht kleiner als 1 sein.</param>
        /// <returns>Array von serverseitig erzeugten Elementen.</returns>
        [OperationContract]
        [FaultContract(typeof(CwlbFault), Name = "DefaultFault", Namespace = "http://WeroSoft.net/Samples/Fault", Action = "FaultAction")]
        [FaultContract(typeof(int), Name = "IntFault", Namespace = "http://WeroSoft.net/Samples/Fault", Action = "FaultIntAction")]
        [FaultContract(typeof(string))]
        [FaultContract(typeof(double))]
        CwlbDemoData[] GetDemoData(CwlbDemoData demoData, int iParam);

        /// <summary>
        /// Liefert ein definierte Anzahl von Einträgen in einem Verzeichnis.
        /// </summary>
        /// <param name="iParam">Anzahl der Einträge die generiert werden sollen.</param>
        /// <returns>Verzeicnis mit Demodaten.</returns>
        [OperationContract]
        CwlbDemoDirectory<string, CwlbDemoData> GetDictionary(int iParam);

        /// <summary>
        /// Demomethode mit einem Nachrichtenvertrag.
        /// </summary>
        /// <param name="objMessage">Nachrichtenvertrag.</param>
        [OperationContract]
        void UseMessageData(CwlbMessageData objMessage);
    }
}
