﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Threading;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert die Anwendungsklasse.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Definiert den Einstiegspunkt in die Anwendung
        /// </summary>
        static void Main()
        {
            // Proxyobjekt deklarieren
            IwlbOrganizationManager objSimpleProxy = null;

            try
            {
                // ------------------------------------------------------------
                // Einfachen Proxy instanziieren und verwenden
                // ------------------------------------------------------------
                objSimpleProxy = ChannelFactory<IwlbOrganizationManager>.CreateChannel(
                    new NetTcpBinding(),
                    new EndpointAddress("net.tcp://localhost:4711/LearningWcf"));

                // Daten vom Server holen
                CwlbCompany objCompany = objSimpleProxy.GetOrganizationStructure(null);

                // Kontrollausgabe durchführen
                Console.WriteLine("Benutzung des einfachen Clients (generiert ChannelFactory<T>):");
                Console.WriteLine("Name der Firma:. . . . . . . {0}", objCompany.Name);
                Console.WriteLine("Name der CEO:  . . . . . . . {0}", objCompany.CEO.Name);
                Console.WriteLine("Name der CFO:  . . . . . . . {0}", objCompany.CFO.Name);
                Console.WriteLine("Name der Division: . . . . . {0}", objCompany.Divisions[0].Name);
                Console.WriteLine("Name des Managers: . . . . . {0}", objCompany.Divisions[0].Manager.Name);
                Console.WriteLine("Name der Vertragsabteilung:  {0}", objCompany.Divisions[0].Divisions[0].Name);
                Console.WriteLine("Name des Managers: . . . . . {0}", objCompany.Divisions[0].Divisions[0].Manager.Name);
                Console.WriteLine("Mitarbeiter der Division {0}:", objCompany.Divisions[0].Divisions[0].Name);
                foreach (var objPerson in objCompany.Divisions[0].Divisions[0].Employees)
                {
                    Console.WriteLine("   - {0}", objPerson.Name);
                }
                Console.WriteLine(new string('-', 60));
                Console.WriteLine("Typ der Liste von 'Divisions' ist: {0}", objCompany.Divisions.GetType().Name);
                Console.WriteLine(new string('-', 60));

                // ------------------------------------------------------------
                // Funktionellen Proxy instanziieren und verwenden
                // ------------------------------------------------------------
                Console.WriteLine();
                Console.WriteLine("Benutzung des funktionellen Clients (generiert mit Visual Studio):");
                using (OrganizationManagerClient objFunctionalProxy = new OrganizationManagerClient("DefaultBinding_OrganizationManager_OrganizationManager"))
                {
                    werosoft.net.Samples.Data.Company objFunctionalClientCompany = objFunctionalProxy.GetOrganizationStructure(null);
                    // Kontrollausgabe durchführen
                    Console.WriteLine("Name der Firma:. . . . . . . {0}", objFunctionalClientCompany.Name);
                    Console.WriteLine("Name der CEO:  . . . . . . . {0}", objFunctionalClientCompany.CEO.Name);
                    Console.WriteLine("Name der CFO:  . . . . . . . {0}", objFunctionalClientCompany.CFO.Name);
                    Console.WriteLine("Name der Division: . . . . . {0}", objFunctionalClientCompany.Divisions[0].Name);
                    Console.WriteLine("Name des Managers: . . . . . {0}", objFunctionalClientCompany.Divisions[0].Manager.Name);
                    Console.WriteLine("Name der Vertragsabteilung:  {0}", objFunctionalClientCompany.Divisions[0].Divisions[0].Name);
                    Console.WriteLine("Name des Managers: . . . . . {0}", objFunctionalClientCompany.Divisions[0].Divisions[0].Manager.Name);
                    Console.WriteLine("Mitarbeiter der Division {0}:", objFunctionalClientCompany.Divisions[0].Divisions[0].Name);
                    foreach (var objPerson in objFunctionalClientCompany.Divisions[0].Divisions[0].Employees)
                    {
                        Console.WriteLine("   - {0}", objPerson.Name);
                    }
                    Console.WriteLine(new string('-', 60));
                    Console.WriteLine("Typ der Liste von 'Divisions' ist: {0}", objFunctionalClientCompany.Divisions.GetType().Name);
                    Console.WriteLine(new string('-', 60));
                }
            }
            catch (Exception excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            finally
            {
                // Proxyobjekt ordentlich schliessen
                if (objSimpleProxy != null && ((IChannel)objSimpleProxy).State == CommunicationState.Opened)
                {
                    ((IChannel)objSimpleProxy).Close();
                }
                if (objSimpleProxy != null && ((IChannel)objSimpleProxy).State == CommunicationState.Faulted)
                {
                    ((IChannel)objSimpleProxy).Abort();
                }
            }
        }
    }
}
