﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Threading;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert die Anwendungsklasse.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Definiert den Einstiegspunkt in die Anwendung
        /// </summary>
        static void Main()
        {
            // Proxyobjekt deklarieren
            IwlbOrganizationManager objSmartProxy = null;
            OrganizationManagerClient objRichProxy = null;

            try
            {
                // ------------------------------------------------------------
                // Smart-Client instanziieren und verwenden
                // ------------------------------------------------------------
                objSmartProxy = ChannelFactory<IwlbOrganizationManager>.CreateChannel(
                    new NetTcpBinding(),
                    new EndpointAddress("net.tcp://localhost:4711/LearningWcf"));

                // Daten vom Server holen
                CwlbCompany objSimpleClientCompany = objSmartProxy.GetOrganizationStructure(null);

                // Kontrollausgabe durchführen
                Console.WriteLine("Benutzung des einfachen Clients (generiert ChannelFactory<T>):");
                Console.WriteLine("Name der Firma:. . . . . . . {0}", objSimpleClientCompany.Name);
                Console.WriteLine("Name der CEO:  . . . . . . . {0}", objSimpleClientCompany.CEO.Name);
                Console.WriteLine("Name der CFO:  . . . . . . . {0}", objSimpleClientCompany.CFO.Name);

                // Lesen der ersten Unterorganisation
                CwlbOrganization objSimpleOrganization = objSimpleClientCompany.Content.OfType<CwlbOrganization>().FirstOrDefault();
                Console.WriteLine("Name der Division: . . . . . {0}", objSimpleOrganization.Name);
                Console.WriteLine("Name des Managers: . . . . . {0}", objSimpleOrganization.Manager.Name);

                // Lesen der nächsten Unterorganisation 
                objSimpleOrganization = objSimpleOrganization.Content[0] as CwlbOrganization;
                Console.WriteLine("Name der Vertragsabteilung:  {0}", objSimpleOrganization.Name);
                Console.WriteLine("Name des Managers: . . . . . {0}", objSimpleOrganization.Manager.Name);
                Console.WriteLine("Mitarbeiter der Division {0}:", objSimpleOrganization.Name);
                foreach (var objPerson in objSimpleOrganization.Content)
                {
                    Console.WriteLine("   - {0}", objPerson.Name);
                }

                // ------------------------------------------------------------
                // Rich-Client instanziieren und verwenden
                // ------------------------------------------------------------
                Console.WriteLine();
                Console.WriteLine("Benutzung des funktionellen Clients (generiert mit Visual Studio):");
                objRichProxy = new OrganizationManagerClient("OrganizationManager");
                
                werosoft.net.Samples.Data.Company objCompany = objRichProxy.GetOrganizationStructure(null);
                // Kontrollausgabe durchführen
                Console.WriteLine("Name der Firma:. . . . . . . {0}", objCompany.Name);
                Console.WriteLine("Name der CEO:  . . . . . . . {0}", objCompany.CEO.Name);
                Console.WriteLine("Name der CFO:  . . . . . . . {0}", objCompany.CFO.Name);

                // Lesen der ersten Unterorganisation
                werosoft.net.Samples.Data.Organization objOrganization = objCompany.Content
                    .OfType<werosoft.net.Samples.Data.Organization>().FirstOrDefault();
                Console.WriteLine("Name der Division: . . . . . {0}", objOrganization.Name);
                Console.WriteLine("Name des Managers: . . . . . {0}", objOrganization.Manager.Name);

                // Lesen der nächsten Unterorganisation 
                objOrganization = objOrganization.Content[0] as werosoft.net.Samples.Data.Organization;
                Console.WriteLine("Name der Division: . . . . . {0}", objOrganization.Name);
                Console.WriteLine("Name des Managers: . . . . . {0}", objOrganization.Manager.Name);
                Console.WriteLine("Mitarbeiter der Division {0}:", objOrganization.Name);
                foreach (var objPerson in objOrganization.Content)
                {
                    Console.WriteLine("   - {0}", objPerson.Name);
                }
            }
            catch (Exception excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            finally
            {
                // Proxyobjekt ordentlich schliessen
                if (objSmartProxy != null && ((IChannel)objSmartProxy).State == CommunicationState.Opened)
                {
                    ((IChannel)objSmartProxy).Close();
                }
                if (objSmartProxy != null && ((IChannel)objSmartProxy).State == CommunicationState.Faulted)
                {
                    ((IChannel)objSmartProxy).Abort();
                }
                if (objRichProxy != null && objRichProxy.State == CommunicationState.Opened)
                {
                    objRichProxy.Close();
                }
                if (objRichProxy != null && objRichProxy.State == CommunicationState.Faulted)
                {
                    objRichProxy.Abort();
                }
                if (objRichProxy != null)
                {
                    objRichProxy.ChannelFactory.Close();
                }
            }
        }
    }
}
