﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert den Organisationsdienst.
    /// </summary>
    public class CwlbOrganizationManager : IwlbOrganizationManager
    {
        /// <summary>
        /// Liefert eine Organisationsstruktur.
        /// </summary>
        /// <param name="strCompanyId">Identifikation des zu liefernden Strukturelements.</param>
        /// <returns>Eine Struktur einer Organisation.</returns>
        public CwlbCompany GetOrganizationStructure(string strCompanyId)
        {
            // Daten fest codiert zusammenstellen
            CwlbCompany objCompany = CreateOrganizationPart<CwlbCompany>("WCF Company") as CwlbCompany;
            objCompany.CEO = AddPersonToOrganization(objCompany, "Master", "Peter");
            objCompany.CFO = AddPersonToOrganization(objCompany, "Monait", "Annarosa");

            CwlbOrganization objDivision = AddStructureElementToOrganisation(objCompany, "Verträge") as CwlbOrganization;
            objDivision.Manager = objCompany.CEO;
            objDivision = AddStructureElementToOrganisation(objDivision, "Datenverträge") as CwlbOrganization;
            objDivision.Manager = objCompany.CFO;

            // Mitarbeiter ergänzen
            AddPersonToOrganization(objDivision, "Ambrunnen", "Fritz");
            AddPersonToOrganization(objDivision, "Aufderalp", "Gertrud");
            AddPersonToOrganization(objDivision, "Unterdorf", "Suzanna");
            AddPersonToOrganization(objDivision, "Manner", "Therese");
            AddPersonToOrganization(objDivision, "Blauer", "Jochen");

            // Daten ausliefern
            return objCompany;
        }

        /// <summary>
        /// Erstellt einen Teil in der Organisationsstruktur
        /// </summary>
        /// <typeparam name="T">Typ des Elements, das erstellt werden soll.</typeparam>
        /// <param name="strName">Definiert den Namen des Elements.</param>
        /// <returns>Liefert das erstelle Element.</returns>
        private CwlbOrganizationPart CreateOrganizationPart<T>(string strName) where T : CwlbOrganizationPart, new()
        {
            T objPart = new T();
            objPart.Name = strName;
            objPart.Id = Guid.NewGuid().ToString();
            if (objPart is CwlbStructureElement)
            {
                ((CwlbStructureElement)(object)objPart).Content = new List<CwlbOrganizationPart>();
            }
            return objPart;
        }

        /// <summary>
        /// Fügt ein Element zu der Organisationstruktur hinzu.
        /// </summary>
        /// <param name="objElement">Element zu dem das neue Element hinzugefügt werden soll.</param>
        /// <param name="strName">Name des Elements das hinzugefügt werden soll.</param>
        /// <returns>Liefert das erstelle Element.</returns>
        private CwlbStructureElement AddStructureElementToOrganisation(CwlbStructureElement objElement, string strName)
        {
            CwlbOrganization objDivision = CreateOrganizationPart<CwlbOrganization>(strName) as CwlbOrganization;

            objElement.Content.Add(objDivision);
            objDivision.ParentOrganization = objElement;
            return objDivision;
        }

        /// <summary>
        /// Fügt eine Person zu einer Struktur hinzu.
        /// </summary>
        /// <param name="objElement">Element zu dem das neue Element hinzugefügt werden soll.</param>
        /// <param name="strName">Name der Person, die hinzugefügt werden soll.</param>
        /// <param name="strFirstName">Vorname der Person, die hinzugefügt werden soll.</param>
        /// <returns>Liefert das neu erstellte Personenobjekt.</returns>
        private CwlbPerson AddPersonToOrganization(CwlbStructureElement objElement, string strName, string strFirstName)
        {
            CwlbPerson objPerson = CreateOrganizationPart<CwlbPerson>(strName) as CwlbPerson;
            objPerson.FirstName = strFirstName;

            objElement.Content.Add(objPerson);
            objPerson.ParentOrganization = objElement;
            return objPerson;
        }
    }
}
