﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;

namespace WeroSoft.Samples.LearningWcfContract
{
    /// <summary>
    /// Implementiert den Rückrufvertrag.
    /// </summary>
    [CallbackBehavior(UseSynchronizationContext=false)]
    public class CwlbClientCallback : IwlbClientCallback
    {
        /// <summary>
        /// Wird ausgelöst wenn ein blockierender Aufruf vom Server eintrifft.
        /// </summary>
        public event EventHandler<CwlbInfoEventArgs> BlockingCallback = delegate { };

        /// <summary>
        /// Wird ausgelöst wenn ein nicht blockierender Aufruf vom Server eintrifft.
        /// </summary>
        public event EventHandler<CwlbInfoEventArgs> NonBlockingCallback = delegate { };

        /// <summary>
        /// Wird ausgelöst wenn ein zyklischer Rückruf vom Server eintrifft.
        /// </summary>
        public event EventHandler<CwlbInfoEventArgs> CyclicCallback = delegate { };

        /// <summary>
        /// Rückrufmethode für einen blockierenden Rückruf.
        /// </summary>
        /// <param name="objEventArgs">Rückrufdaten.</param>
        public void BlockingClientCallback(CwlbInfoEventArgs objEventArgs)
        {
            BlockingCallback(this, objEventArgs);
        }

        /// <summary>
        /// Rückrufmethode für einen nicht blockierenden Rückruf.
        /// </summary>
        /// <param name="objEventArgs">Rückrufdaten.</param>
        public void NonBlockingClientCallback(CwlbInfoEventArgs objEventArgs)
        {
            NonBlockingCallback(this, objEventArgs);
        }

        /// <summary>
        /// Rückrufmethode für einen zyklischen Aufruf.
        /// </summary>
        /// <param name="senderName">Name des Absenders dieses Aufrufs</param>
        /// <param name="objEventArgs">Rückrufdaten.</param>
        public void CyclicClientCallback(string senderName, CwlbInfoEventArgs objEventArgs)
        {
            CyclicCallback(senderName, objEventArgs);
        }
    }
}
