﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;

namespace WeroSoft.Samples.LearningWcfContract
{
    /// <summary>
    /// Definiert den Demodienst für die Fehlerbehandlung des Beispiels.
    /// </summary>
    [ServiceContract(Name="FaultDemoService", Namespace="http://WeroSoft.net/Samples/Contracts")]
    public interface IwlbFaultDemoService
    {
        /// <summary>
        /// Löst eine Ausnahme ohne eigene Fehlerbehandlung im Dienst aus.
        /// </summary>
        /// <param name="iDividend">Dividend für die Division.</param>
        /// <param name="iDivisor">Divisor für die Division.</param>
        /// <returns>Resultat der Division.</returns>
        [OperationContract(Name = "Divide_NoFaultHandling")]
        int Divide_NoFaultHandling(int iDividend, int iDivisor);

        /// <summary>
        /// Löst eine Ausnahme mit eigener minimalen Fehlerbehandlung aus.
        /// </summary>
        /// <param name="iDividend">Dividend für die Division.</param>
        /// <param name="iDivisor">Divisor für die Division.</param>
        /// <returns>Resultat der Division.</returns>
        [OperationContract(Name = "Divide_WithSimpleFaultHandling")]
        int Divide_WithSimpleFaultHandling(int iDividend, int iDivisor);

        /// <summary>
        /// Löst eine Ausnahme mit eigener Fehlerbehandlung und dem Typ CwlbFaultDetail aus.
        /// </summary>
        /// <param name="iDividend">Dividend für die Division.</param>
        /// <param name="iDivisor">Divisor für die Division.</param>
        /// <returns>Resultat der Division.</returns>
        [FaultContract(typeof(CwlbFaultDetail))]
        [OperationContract(Name = "Divide_WithFullFeaturedFaultHandling")]
        int Divide_WithFullFeaturedFaultHandling(int iDividend, int iDivisor);
    }
}
