﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WeroSoft.Samples.LearningWcfContract;
using System.ServiceModel;
using System.Reflection;

namespace WeroSoft.Samples.LearningWcfImplementation
{
    /// <summary>
    /// Implementierung des Singleton-Dienstes.
    /// </summary>
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
    //[CwlbErrorHandler(typeof(CwlbServiceErrorHandler), FaultDetailConverter=typeof(CwlbFaultConverter), KeepSessionAlive=true)]
    public class CwlbDemoServiceSingle : CwlbDemoServiceBase, IwlbFaultDemoService
    {
        /// <summary>
        /// Hält eine Referenz auf den verwendeten Instanzkontext
        /// </summary>
        private static InstanceContext _objContext;

        /// <summary>
        /// Hält eine Referenz auf den verwendeten Instanzkontext
        /// </summary>
        protected override InstanceContext LastInstanceContext 
        {
            get { return _objContext; }
            set { _objContext = value; }
        }

        /// <summary>
        /// Definiert eine einfache Methode für die Demonstration der Polymorphie in Diensten.
        /// </summary>
        /// <param name="strClientData">Daten, die der Client für den Ping an den Service übermittelt.</param>
        /// <returns>Formatierte Antwort des Dienstes.</returns>
        [OperationBehavior(ReleaseInstanceMode = ReleaseInstanceMode.BeforeAndAfterCall)]
        public override CwlbInfoEventArgs DemoCall(string strClientData)
        {
            return DemoCallPrepare(strClientData);
        }

        /// <summary>
        /// Löst eine Ausnahme ohne eigene Fehlerbehandlung im Dienst aus.
        /// </summary>
        /// <param name="iDividend">Dividend für die Division.</param>
        /// <param name="iDivisor">Divisor für die Division.</param>
        /// <returns>Resultat der Division.</returns>
        public virtual int Divide_NoFaultHandling(int iDividend, int iDivisor)
        {
            return iDividend / iDivisor;
        }

        /// <summary>
        /// Löst eine Ausnahme mit eigener minimalen Fehlerbahndlung aus.
        /// </summary>
        /// <param name="iDividend">Dividend für die Division.</param>
        /// <param name="iDivisor">Divisor für die Division.</param>
        /// <returns>Resultat der Division.</returns>
        public virtual int Divide_WithSimpleFaultHandling(int iDividend, int iDivisor)
        {
            try
            {
                return iDividend / iDivisor;
            }
            catch (Exception excObject)
            {
                throw new FaultException(excObject.Message);
            }
        }

        /// <summary>
        /// Löst eine Ausnahme mit eigener Fehlerbehandlung und dem Typ CwlbFaultDetail.
        /// </summary>
        /// <param name="iDividend">Dividend für die Division.</param>
        /// <param name="iDivisor">Divisor für die Division.</param>
        /// <returns>Resultat der Division.</returns>
        public virtual int Divide_WithFullFeaturedFaultHandling(int iDividend, int iDivisor)
        {
            try
            {
                return iDividend / iDivisor;
            }
            catch (Exception excObject)
            {
                CwlbFaultDetail objFaultDetail = new CwlbFaultDetail();
                objFaultDetail.MachineName = Environment.MachineName;
                objFaultDetail.ServiceName = this.GetType().Name;
                objFaultDetail.MethodName = MethodInfo.GetCurrentMethod().Name;
                throw new FaultException<CwlbFaultDetail>(objFaultDetail, new FaultReason(excObject.Message), new FaultCode("MATH025", new FaultCode("MATH42")));
            }
        }
    }
}
