﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ServiceModel;
using WeroSoft.Samples.LearningWcfContract;
using System.ServiceModel.Channels;
using System.Collections.ObjectModel;
using System.Threading;

namespace WeroSoft.Samples.LearningWcfClient
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private const string ProxyNetTcpPerCall = "TcpPerCall";
        private const string ProxyHttpPerCall = "HttpPerCall";
        private const string ProxyNetTcpPerSession = "TcpPerSession";
        private const string ProxyHttpPerSession = "HttpPerSession";
        private const string ProxyNetTcpSingle = "TcpSingle";
        private const string ProxyHttpSingle = "HttpSingle";
        private const string ProxyTcpCallback = "TcpCallback";
        private const string ProxyTcpFaultWithoutDetails = "TcpFaultWithoutDetails";
        private const string ProxyTcpFaultWithDetails = "TcpFaultWithDetails";
        private const string FormatLocalHostTcp = "net.tcp://localhost:4711/{0}";
        private const string FormatLocalHostHttp = "http://localhost:4712/{0}";
        private const int SendTimeOut = 17;

        // Stellt ein Synchronisationobjekt für die Anwendung zur Verfügung
        private object _objSyncRoot = new object();
        // Hält ein Verzeichnis mit den erstellten Proxy objekten
        private static Dictionary<string, IwlbDemoService> _cobjProxy;
        // Hält eine Liste von Proxies für die Prüfung der Verbindungsmenge
        private List<IwlbDemoService> _cobjMassProxyTest = new List<IwlbDemoService>();
        // Hält das Proxyobjekt für zyklische Rückrufe
        private IwlbCallbackDemoService _objCyclicCallbackProxy = null;
        // Hält das Rückrufobjekt für zyklische Rückrufe
        private CwlbClientCallback _objCyclicCallback = null;
        // Hält das Proxyobjekt für die Demo von Fehlerhandhabung.
        private IwlbFaultDemoService _objFaultProxy = null;
        // Hält eine Liste für die Kontrolle der Expander in der Benutzerschnittstelle
        private List<Expander> _cobjExpander = null;

        /// <summary>
        /// Leifert eine Liste der informationsereigniss, die im Rahmen des Betriebs der Lösung enstanden sind.
        /// </summary>
        public ObservableCollection<CwlbInfoEventArgs> MessageCollection { get; private set; }

        /// <summary>
        /// Initialisiert eine Instanz der Klasse MainWindow.
        /// </summary>
        public MainWindow()
        {
            // Initialisieren der Instanzdaten
            _cobjProxy = new Dictionary<string, IwlbDemoService>();
            MessageCollection = new ObservableCollection<CwlbInfoEventArgs>();
            this.DataContext = this;
            _cobjExpander = new List<Expander>();

            // Intialisieren der Steuerelemente
            InitializeComponent();
        }

        /// <summary>
        /// Ereignisbehandlung beim expandieren eines Expanders.
        /// </summary>
        private void Expander_Expanded(object sender, RoutedEventArgs e)
        {
            foreach (Expander objExpander in _cobjExpander)
            {
                if (!object.ReferenceEquals(sender, objExpander))
                {
                    objExpander.IsExpanded = false;
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Laden eines Expander Steuerelements
        /// </summary>
        private void Expander_Loaded(object sender, RoutedEventArgs e)
        {
            _cobjExpander.Add(sender as Expander);
        }

        /// <summary>
        /// Ereignisbehandlung wenn die Schaltfläche Erstellen eines Proxy betätigt wurde.
        /// </summary>
        private void ButtonCreateProxy_Clicked(object sender, RoutedEventArgs e)
        {
            // Erstellen des Proxy
            CreateProxy(sender);
        }

        /// <summary>
        /// Erstellen eines Proxy objekts
        /// </summary>
        /// <param name="strType"></param>
        private void CreateProxy(object sender)
        {
            // Prüfen der Parameter
            Button objButton = sender as Button;
            if (objButton == null)
            {
                MessageBox.Show("Ungültiger sender für die Verarbeitung Erstellen Proxy");
                return;
            }

            string strType = objButton.CommandParameter as string;
            if (strType == null)
            {
                MessageBox.Show("Ungültiger Befehlsparameter empfangen.");
                return;
            }

            // Eventuell vorhandener Proxy des Typs löschen
            DeleteProxy(strType);

            // Erstellen der neuen Daten für den Proxy
            try
            {
                EndpointAddress objEndpointAddress;
                System.ServiceModel.Channels.Binding objBinding;
                if (strType.Contains("Tcp"))
                {
                    objBinding = new NetTcpBinding();
                    objEndpointAddress = new EndpointAddress(string.Format(FormatLocalHostTcp, strType));
                }
                else if (strType.Contains("Http"))
                {
                    objBinding = new BasicHttpBinding();
                    objEndpointAddress = new EndpointAddress(string.Format(FormatLocalHostHttp, strType));
                }
                else
                {
                    objBinding = new NetTcpBinding();
                    ((NetTcpBinding)objBinding).ReliableSession.Enabled = true;
                    objEndpointAddress = new EndpointAddress(string.Format(FormatLocalHostTcp, "Shared"));
                }

                // Proxy herstellen und registrieren
                IwlbDemoService objProxy = ChannelFactory<IwlbDemoService>.CreateChannel(objBinding, objEndpointAddress);
                _cobjProxy.Add(strType, objProxy);

                // Mitteilung in das Log schreiben
                WriteInfo("Erstellen Proxy nach: {0}", objEndpointAddress.ToString());
            }
            catch (Exception excObject)
            {
                // Mitteilung in das Log schreiben
                WriteInfo("Erstellen Proxy: {0}", excObject.Message);
            }
        }

        /// <summary>
        /// Ereignisbehandlung wenn die Schaltfläche Löschen eines Proxy betätigt wurde.
        /// </summary>
        private void ButtonDeleteProxy_Clicked(object sender, RoutedEventArgs e)
        {
            // Prüfen der Parameter
            Button objButton = sender as Button;
            if (objButton == null)
            {
                MessageBox.Show("Ungültiger Sender für die Verarbeitung Erstellen Proxy");
            }

            string strCommandParameter = objButton.CommandParameter as string;
            if (strCommandParameter == null)
            {
                MessageBox.Show("Ungültiger Befehlsparameter empfangen.");
            }

            // Erstellen des Proxy
            DeleteProxy(strCommandParameter);
        }

        /// <summary>
        /// Löscht einen definiert Proxy aus dem Verzeichnis.
        /// </summary>
        /// <param name="strType">Typ des Proxys der zu löschen ist.</param>
        private void DeleteProxy(string strType)
        {
            try
            {
                // Prüfen ob der Proxy vorhanden ist
                if (_cobjProxy.ContainsKey(strType))
                {
                    // Prüfen ob Proxy einen Fehler aufweis und diesen neu erstellen wenn ja
                    IChannel objChannel = _cobjProxy[strType] as IChannel;
                    if (objChannel.State == CommunicationState.Faulted)
                    {
                        objChannel.Abort();
                        // Mitteilung in das Log schreiben
                        WriteInfo(this, new CwlbInfoEventArgs
                        {
                            Sender = strType,
                            Text = string.Format("Abbrechen Proxy")
                        });
                    }
                    else if (objChannel.State == CommunicationState.Opened)
                    {
                        objChannel.Close();
                        // Mitteilung in das Log schreiben
                        WriteInfo(this, new CwlbInfoEventArgs
                        {
                            Sender = strType,
                            Text = string.Format("Schliessen Proxy")
                        });
                    }
                    else
                    {
                        // Mitteilung in das Log schreiben
                        WriteInfo(this, new CwlbInfoEventArgs
                        {
                            Sender = strType,
                            Text = string.Format("Proxy gelöscht")
                        });
                    }

                    // Proxy löschen
                    _cobjProxy.Remove(strType);
                }
            }
            catch (Exception excObject)
            {
                // Mitteilung in das Log schreiben
                WriteInfo("Löschen Proxy: {0}", excObject.Message);
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Ausführen des Methodenaufrufs auf einem definierten Proxy.
        /// </summary>
        private void ButtonGo_Clicked(object sender, RoutedEventArgs e)
        {
            Button objButton = sender as Button;
            if (objButton == null)
            {
                MessageBox.Show("Ungültiger Sender für die Verarbeitung 'Erstellen Proxy'.");
                return;
            }

            string strCommandParameter = objButton.CommandParameter as string;
            if (strCommandParameter == null)
            {
                MessageBox.Show("Ungültiger Befehlsparameter empfangen.");
                return;
            }
            ExecuteCall(strCommandParameter, (IwlbDemoService objProxy, string strInput) => { return objProxy.Ping(strInput); });
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonGoPerCall_Clicked(object sender, RoutedEventArgs e)
        {
            Button objButton = sender as Button;
            if (objButton == null)
            {
                MessageBox.Show("Ungültiger Sender für die Verarbeitung 'Erstellen Proxy'.");
                return;
            }

            string strCommandParameter = objButton.CommandParameter as string;
            if (strCommandParameter == null)
            {
                MessageBox.Show("Ungültiger Befehlsparameter empfangen.");
                return;
            }
            ExecuteCall(strCommandParameter, (IwlbDemoService objProxy, string strInput) => { return objProxy.DemoCall(strInput); });
        }


        /// <summary>
        /// Ausführen des Methodenaufrufs zum Server.
        /// </summary>
        /// <param name="strType"></param>
        /// <param name="function"></param>
        private void ExecuteCall(string strType, Func<IwlbDemoService, string, CwlbInfoEventArgs> function)
        {
            // Prüfe ob ein geteilter Zugriff vorliegt
            string strInstanceId = null;
            strType = CheckSharedAccess(strType, ref strInstanceId);

            // Prüfen ob der Proxy vorhanden ist
            if (!_cobjProxy.ContainsKey(strType))
            {
                MessageBox.Show("Der Proxy ist nicht vorhanden, Sie müssen diesen zuerst erstellen.");
                return;
            }

            try
            {
                // Proxy aus dem Verzeichnis holen
                IwlbDemoService objProxy = _cobjProxy[strType];

                if (strInstanceId == null)
                {
                    // Normaler Aufruf
                    CwlbInfoEventArgs objPing = function(objProxy, Environment.MachineName);
                    WriteInfo(this, objPing);
                }
                else
                {
                    // Aufruf für den Geteilten Zugriff
                    using (new OperationContextScope((IContextChannel)objProxy))
                    {
                        MessageHeader header = MessageHeader.CreateHeader(GwlbContractGlobals.SharedHeaderName, GwlbContractGlobals.SharedHeaderNamespace, strInstanceId);
                        OperationContext.Current.OutgoingMessageHeaders.Add(header);
                        CwlbInfoEventArgs objPing = function(objProxy, Environment.MachineName);
                        WriteInfo(this, objPing);
                    }
                }
            }
            catch (CommunicationException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Communication);
            }
            catch (TimeoutException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Timeout);
            }
            catch (Exception excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.GeneralFault);
            }
        }

        /// <summary>
        /// Prüft ob ein Typ eine Instanzidentifikation enthält.
        /// </summary>
        /// <param name="strType">Typ der untersucht werden soll.</param>
        /// <param name="strInstanceId">Instanz die in einer Typdefinition enthalten war.</param>
        /// <returns></returns>
        private string CheckSharedAccess(string strType, ref string strInstanceId)
        {
            string[] astrParameters = strType.Split(',');
            if (astrParameters.Length == 1)
            {
                return strType;
            }
            else
            {
                strInstanceId = astrParameters[1];
                return astrParameters[0];
            }
        }

        /// <summary>
        /// Speichert einen Text als Information in den Ausgaben im Fenster.
        /// </summary>
        private void WriteInfo(string strText, params object[] aobjParams)
        {
            WriteInfo((object)"Self", new CwlbInfoEventArgs
            {
                Sender = "Self",
                Text = string.Format(strText, aobjParams)
            });
        }

        /// <summary>
        /// Speichert eine empfangene Information eines Dienstobjekts oder einer andern Quelle in die 
        /// Liste der Nachrichten zur Visualisierung.
        /// </summary>
        private void WriteInfo(object sender, CwlbInfoEventArgs eventArgs)
        {
            if (!Dispatcher.CheckAccess())
            {
                // Durchführen des Threadwechsels wenn notwendig.
                EventHandler<CwlbInfoEventArgs> objWriteInfoDelegate = WriteInfo;
                Dispatcher.BeginInvoke(objWriteInfoDelegate, new object[] { sender, eventArgs });
            }
            else
            {
                MessageCollection.Add(eventArgs);
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Verändern der Grösse des Listenelements
        /// </summary>
        private void LogView_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            ListView lvLog = sender as ListView;
            if (lvLog == null || e.PreviousSize.Width == 0.0)
            {
                return;
            }

            // Size the columns
            double factor = e.NewSize.Width / e.PreviousSize.Width;
            foreach (var column in ((GridView)lvLog.View).Columns)
            {
                column.Width = column.ActualWidth * factor;
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Löschen der Logliste.
        /// </summary>
        private void ButtonDeleteLog_Clicked(object sender, RoutedEventArgs e)
        {
            MessageCollection.Clear();
        }

        /// <summary>
        /// Ereignisbahndlung beim Betätigen der Schaltfläche Massentest.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonMaxConnection_Clicked(object sender, RoutedEventArgs e)
        {
            ThreadPool.QueueUserWorkItem(ExecutePerSessionMassTest);
        }

        /// <summary>
        /// Steuert die Ausführung der Massentests für das sitzungsorientierte Verhaltn von Proxys
        /// </summary>
        /// <param name="state">Unbenutztes Statusobjekt</param>
        private void ExecutePerSessionMassTest(object state)
        {
            ManualResetEvent objSync = new ManualResetEvent(false);

            // Erstelle maximal 1200 Client objekte
            for (int iCount = 1; iCount <= 1200; iCount++)
            {
                objSync.Reset();
                ThreadPool.QueueUserWorkItem(CallPerSession, new Tuple<int, ManualResetEvent>(iCount, objSync));
                if (!objSync.WaitOne(2000))
                {
                    break;
                }
            }

            // Stoppen aller erzeugten Proxys
            foreach (IChannel objChannel in _cobjMassProxyTest)
            {
                ReleaseProxy(objChannel);
            }
        }

        /// <summary>
        /// Schliesst einen Proxy ordungsgemäss.
        /// </summary>
        /// <param name="objChannel">Zu schliessender Kanal.</param>
        private static void ReleaseProxy(IChannel objChannel)
        {
            if (objChannel != null && objChannel.State == CommunicationState.Opened)
            {
                objChannel.Close();
            }
            else if (objChannel != null && objChannel.State == CommunicationState.Faulted)
            {
                objChannel.Abort();
            }
        }

        /// <summary>
        /// Steuer die Erstellung eines Proxy's und die Ausführung der Methode Ping zum Server.
        /// </summary>
        /// <param name="objState">Spezifsche Daten des Proxyobjekts.</param>
        private void CallPerSession(object objState)
        {
            try
            {
                // Übernehmen der Daten
                Tuple<int, ManualResetEvent> objControlData = objState as Tuple<int, ManualResetEvent>;
                // Erstellen des Proxy, Hinzufügen in die Liste
                IwlbDemoService objProxy = ChannelFactory<IwlbDemoService>.CreateChannel(new NetTcpBinding(), new EndpointAddress(string.Format(FormatLocalHostTcp, ProxyNetTcpPerSession)));
                _cobjMassProxyTest.Add(objProxy);
                // Zeit für den Aufruf messen
                DateTime dtWait = DateTime.UtcNow;
                CwlbInfoEventArgs objData = objProxy.Ping("TEST");
                objData.Sender = objControlData.Item1.ToString();
                // Wenn die Zeit gröser als eine Sekunde war, wurde offenbar
                // die Sitzung vom Server nicht mehr anerkannt.
                if (DateTime.UtcNow - dtWait < TimeSpan.FromSeconds(1))
                {
                    WriteInfo(this, objData);
                }

                // Synchronisation der Steuermethode.
                objControlData.Item2.Set();
            }
            catch (Exception excObject)
            {
                // Mitteilung in das Log schreiben
                WriteInfo("Erstellen Massenproxy: {0}", excObject.Message);
            }
        }

        #region Ereignisbehandlung für Rückrufe
        /// <summary>
        /// Wird aufgerufen um einen blockierenden Rückruf zu demonstrieren.
        /// </summary>
        private void ButtonBlockingCallback_Clicked(object sender, RoutedEventArgs e)
        {
            // Vorspann für die Benutzerführung
            if (MessageBox.Show(Properties.Resources.BlockingCall, Title, MessageBoxButton.YesNo) == MessageBoxResult.No)
            {
                return;
            }

            // Definieren der lokalen Daten und einleiten der Vararbeitung
            IwlbCallbackDemoService objProxy = null;
            CwlbClientCallback objCallback = null;
            try
            {
                // Erstelle das Rückrufobjekts und anschliessend damit den Proxy
                objCallback = new CwlbClientCallback();
                objCallback.BlockingCallback += new EventHandler<CwlbInfoEventArgs>(OnBlockingCallback);
                objProxy = DuplexChannelFactory<IwlbCallbackDemoService>.CreateChannel(
                    objCallback,
                    new NetTcpBinding { SendTimeout = TimeSpan.FromSeconds(SendTimeOut) },
                    new EndpointAddress(string.Format(FormatLocalHostTcp, ProxyTcpCallback)));

                // Durchführen des blockierenden Aufrufs
                objProxy.BlockingCallback(Environment.MachineName);
                WriteInfo("Der blockierende Rückruf ist nicht korrekt zu Ende gegangen.");
            }
            catch (CommunicationException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Communication);
            }
            catch (TimeoutException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Timeout);
            }
            catch (Exception excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.GeneralFault);
            }
            finally
            {
                // Freigeben der lokalen Elemente
                ReleaseProxy(objProxy as IChannel);
                objCallback.BlockingCallback -= new EventHandler<CwlbInfoEventArgs>(OnBlockingCallback);
            }
        }

        /// <summary>
        /// Ereignisbehandlung für den blockierenden Aufruf.
        /// </summary>
        void OnBlockingCallback(object sender, CwlbInfoEventArgs e)
        {
            // Prüfen ob ein Threadwechsel notwendig ist
            if (!Dispatcher.CheckAccess())
            {
                // Threadwchsel durchführen
                EventHandler<CwlbInfoEventArgs> delegateRecall = OnBlockingCallback;
                Dispatcher.Invoke(delegateRecall, new object[] { sender, e });
            }
            else
            {
                MessageCollection.Add(e);
            }
        }

        /// <summary>
        /// Wird aufgerufen um einen korrekten Rückruf zu demonstrieren
        /// </summary>
        private void ButtonNonBlockingCallback_Clicked(object sender, RoutedEventArgs e)
        {
            // Definieren der lokalen Daten und einleiten der Vararbeitung
            IwlbCallbackDemoService objProxy = null;
            CwlbClientCallback objCallback = null;
            try
            {
                // Erstelle das Callback client object und anschliessend 
                // damit den Proxy
                objCallback = new CwlbClientCallback();
                objCallback.NonBlockingCallback += new EventHandler<CwlbInfoEventArgs>(OnNonBlockingCallback);
                objProxy = DuplexChannelFactory<IwlbCallbackDemoService>.CreateChannel(
                    objCallback,
                    new NetTcpBinding { SendTimeout = TimeSpan.FromSeconds(SendTimeOut) },
                    new EndpointAddress(string.Format(FormatLocalHostTcp, ProxyTcpCallback)));

                // Durchführen des nicht blockierenden Aufrufs
                objProxy.NonBlockingCallback(Environment.MachineName);
            }
            catch (CommunicationException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Communication);
            }
            catch (TimeoutException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Timeout);
            }
            catch (Exception excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.GeneralFault);
            }
            finally
            {
                // Freigeben der lokalen Elemente
                ReleaseProxy(objProxy as IChannel);
                objCallback.NonBlockingCallback -= new EventHandler<CwlbInfoEventArgs>(OnNonBlockingCallback);
            }
        }

        /// <summary>
        /// Ereignisbehandlung für den blockierenden Aufruf.
        /// </summary>
        void OnNonBlockingCallback(object sender, CwlbInfoEventArgs e)
        {
            // Prüfen ob ein Threadwechsel notwendig ist
            if (!Dispatcher.CheckAccess())
            {
                // Threadwchsel durchführen
                EventHandler<CwlbInfoEventArgs> delegateRecall = OnNonBlockingCallback;
                Dispatcher.BeginInvoke(delegateRecall, new object[] { sender, e });
            }
            else
            {
                MessageCollection.Add(e);
            }
        }

        /// <summary>
        /// Wird aufgerufen um einen zyklischen Rückruf auszulösen.
        /// </summary>
        private void ButtonStartCyclicCallback_Clicked(object sender, RoutedEventArgs e)
        {
            try
            {
                // Erstelle das Callback client object und anschliessend damit den Proxy
                _objCyclicCallback = new CwlbClientCallback();
                _objCyclicCallback.CyclicCallback += new EventHandler<CwlbInfoEventArgs>(OnCyclicCallback);
                _objCyclicCallbackProxy = DuplexChannelFactory<IwlbCallbackDemoService>.CreateChannel(
                    _objCyclicCallback,
                    new NetTcpBinding { SendTimeout = TimeSpan.FromSeconds(SendTimeOut) },
                    new EndpointAddress(string.Format(FormatLocalHostTcp, ProxyTcpCallback)));

                // Durchführen Registrierung
                _objCyclicCallbackProxy.RegisterClient();

                // Benutzeroberfläche steuern
                ButtonStart.IsEnabled = false;
                ButtonStop.IsEnabled = true;
                ComboBoxChangeInterval.IsEnabled = true;
            }
            catch (CommunicationException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Communication);
            }
            catch (TimeoutException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Timeout);
            }
            catch (Exception excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.GeneralFault);
            }
        }

        /// <summary>
        /// Ereignisbehandlung für den zyklischen Rückruf vom Server.
        /// </summary>
        private void OnCyclicCallback(object sender, CwlbInfoEventArgs e)
        {
            if (!Dispatcher.CheckAccess())
            {
                EventHandler<CwlbInfoEventArgs> delegateRecall = OnNonBlockingCallback;
                Dispatcher.BeginInvoke(delegateRecall, new object[] { sender, e });
            }
            else
            {
                MessageCollection.Add(e);
            }
        }

        /// <summary>
        /// Wird ausgelöst um den zyklischen Rückruf zu stoppen.
        /// </summary>
        private void ButtonStopCyclicCallback_Clicked(object sender, RoutedEventArgs e)
        {
            try
            {
                // Durchführen des Blockierenden Aufrufs
                _objCyclicCallbackProxy.UnregisterClient();

                // Benutzeroberfläche steuern
                ButtonStart.IsEnabled = true;
                ButtonStop.IsEnabled = false;
                ComboBoxChangeInterval.IsEnabled = false;
            }
            catch (CommunicationException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Communication);
            }
            catch (TimeoutException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Timeout);
            }
            catch (Exception excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.GeneralFault);
            }
            finally
            {
                ReleaseProxy(_objCyclicCallbackProxy as IChannel);
                _objCyclicCallbackProxy = null;
                _objCyclicCallback.CyclicCallback -= new EventHandler<CwlbInfoEventArgs>(OnCyclicCallback);
                _objCyclicCallback = null;
            }
        }

        /// <summary>
        /// Wird aufgerufen wenn die Intervallzeit des zyklischen Rückrufs ändern soll.
        /// </summary>
        private void CyclicCallbackInterval_Changed(object sender, SelectionChangedEventArgs e)
        {
            if (_objCyclicCallbackProxy == null)
            {
                return;
            }

            try
            {
                if (e.AddedItems.Count > 0 && e.AddedItems[0] is ComboBoxItem)
                {
                    _objCyclicCallbackProxy.SetInterval(Convert.ToDouble(((ComboBoxItem)e.AddedItems[0]).Tag));
                }
            }
            catch (CommunicationException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Communication);
            }
            catch (TimeoutException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Timeout);
            }
            catch (Exception excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.GeneralFault);
            }
        }
        #endregion

        #region Ereignisbehandlungen für Fehlerbehandlung
        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Proxy erstellen 'ohne ExceptionDetailsinFaults'.
        /// </summary>
        private void ButtonCreateFaultProxyWithoutDetails_Clicked(object sender, RoutedEventArgs e)
        {
            try
            {
                if (_objFaultProxy != null)
                {
                    ReleaseProxy(_objFaultProxy as IChannel);
                }

                // Erstellen das Proxy für die Demo der Fehlerbehandlung
                _objFaultProxy = ChannelFactory<IwlbFaultDemoService>.CreateChannel(
                    new NetTcpBinding { SendTimeout = TimeSpan.FromSeconds(SendTimeOut) },
                    new EndpointAddress(string.Format(FormatLocalHostTcp, ProxyTcpFaultWithoutDetails)));
            }
            catch (CommunicationException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Communication);
            }
            catch (TimeoutException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Timeout);
            }
            catch (Exception excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.GeneralFault);
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Proxy erstellen 'mit ExceptionDetailsinFaults'.
        /// </summary>
        private void ButtonCreateFaultProxyWithDetails_Clicked(object sender, RoutedEventArgs e)
        {
            try
            {
                if (_objFaultProxy != null)
                {
                    ReleaseProxy(_objFaultProxy as IChannel);
                }

                // Erstellen das Proxy für die Demo der Fehlerbehandlung
                _objFaultProxy = ChannelFactory<IwlbFaultDemoService>.CreateChannel(
                    new NetTcpBinding { SendTimeout = TimeSpan.FromSeconds(SendTimeOut) },
                    new EndpointAddress(string.Format(FormatLocalHostTcp, ProxyTcpFaultWithDetails)));
            }
            catch (CommunicationException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Communication);
            }
            catch (TimeoutException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Timeout);
            }
            catch (Exception excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.GeneralFault);
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche 'Auslösen' eines Fehlers ohne Fehlerbehandlung.
        /// </summary>
        private void ButtonCallFaultWithoutHandling_Clicked(object sender, RoutedEventArgs e)
        {
            try
            {
                if (IsProxyFaulted())
                {
                    return;
                }

                // Fehlerbehaftete Methode aufrufen
                _objFaultProxy.Divide_NoFaultHandling(1, 0);
                WriteInfo("Nicht erwartet Ausführung der Methode Divide_NoFaultHandling()");
            }
            catch (FaultException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Communication);
            }
            catch (CommunicationException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Communication);
            }
            catch (TimeoutException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Timeout);
            }
            catch (Exception excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.GeneralFault);
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche 'Auslösen' eines Fehlers mit einfacher Fehlerbehandlung.
        /// </summary>
        private void ButtonCallFaultSimpleHandling_Clicked(object sender, RoutedEventArgs e)
        {
            try
            {
                if (IsProxyFaulted())
                {
                    return;
                }

                // Fehlerbehaftete Methode aufrufen
                _objFaultProxy.Divide_WithSimpleFaultHandling(1, 0);
                WriteInfo("Nicht erwartet Ausführung der Methode Divide_WithSimpleFaultHandling()");
            }
            catch (FaultException excObject)
            {
                string strOut = string.Format("Message: {1}{0}Reason:{2}{0}Codename: {3}{0}Subcode: {4}", Environment.NewLine, excObject.Message, excObject.Reason, excObject.Code.Name, excObject.Code.SubCode);
                MessageBox.Show(strOut, Properties.Resources.Fault);
            }
            catch (CommunicationException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Fault);
            }
            catch (TimeoutException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Timeout);
            }
            catch (Exception excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.GeneralFault);
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche 'Auslösen' eines Fehlers mit vollständiger Fehlerbehandlung.
        /// </summary>
        private void ButtonCallFaultFullHandling_Clicked(object sender, RoutedEventArgs e)
        {
            try
            {
                if (IsProxyFaulted())
                {
                    return;
                }

                // Fehlerbehaftete Methode aufrufen
                _objFaultProxy.Divide_WithFullFeaturedFaultHandling(1, 0);
                WriteInfo("Nicht erwartet Ausführung der Methode Divide_WithFullFeaturedFaultHandling()");
            }
            catch (FaultException<CwlbFaultDetail> excObject)
            {
                string strOut = string.Format("Message: {1}{0}Reason:{2}{0}Codename: {3}{0}Subcode: {4}{0}Maschine: {5}{0}Service: {6}{0}Methode: {7}",
                    Environment.NewLine,             // 0
                    excObject.Message,               // 1
                    excObject.Reason,                // 2
                    excObject.Code.Name,             // 3
                    excObject.Code.SubCode.Name,     // 4
                    excObject.Detail.MachineName,    // 5
                    excObject.Detail.ServiceName,    // 6
                    excObject.Detail.MethodName);    // 7
                MessageBox.Show(strOut, Properties.Resources.Fault);
            }
            catch (FaultException excObject)
            {
                string strOut = string.Format("Message: {1}{0}Reason:{2}{0}Codename: {3}{0}Subcode: {4}",
                    Environment.NewLine,
                    excObject.Message, 
                    excObject.Reason,
                    excObject.Code.Name,
                    excObject.Code.SubCode.Name);
                MessageBox.Show(strOut, Properties.Resources.Fault);
            }
            catch (CommunicationException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Communication);
            }
            catch (TimeoutException excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.Timeout);
            }
            catch (Exception excObject)
            {
                MessageBox.Show(excObject.Message, Properties.Resources.GeneralFault);
            }
        }

        /// <summary>
        /// Prüft ob ein Proxy vorhanden oder fehlerhaft ist.
        /// </summary>
        /// <returns>True, wenn der Proxy erneuert werden muss. False wenn der Proxy verwendet werden kann.</returns>
        private bool IsProxyFaulted()
        {
            if (_objFaultProxy == null)
            {
                MessageBox.Show("Sie müssen zuerst einen Proxy erzeugen.", Title);
                return true;
            }
            if (((IChannel)_objFaultProxy).State == CommunicationState.Faulted)
            {
                MessageBox.Show("Der Proxy ist in fehlerhaften Zustand. Sie müssen den Proxy zuerst neu erstellen.", Title);
                return true;
            }

            return false;
        }
        #endregion
    }
}
