﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using WeroSoft.Samples.LearningWcfImplementation;
using System.ServiceModel;
using System.Collections.ObjectModel;
using WeroSoft.Samples.LearningWcfContract;
using System.ServiceModel.Description;

namespace WeroSoft.Samples.LearningWcfHost
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        // Hält die Liste der registrieren Hosts
        private List<ServiceHost> _cobjHost;
        // Stellt ein Synchronisationobjekt für die Anwendung zur Verfügung
        private object _objSyncRoot = new object();

        /// <summary>
        /// Leifert eine Liste der informationsereigniss, die im Rahmen des Betriebs der Lösung enstanden sind.
        /// </summary>
        public ObservableCollection<CwlbInfoEventArgs> InfoEventArgsCollection { get; private set; }

        /// <summary>
        /// Intialisiert eine neue Instanz der Klasse MainWindow.
        /// </summary>
        public MainWindow()
        {
            InfoEventArgsCollection = new ObservableCollection<CwlbInfoEventArgs>();
            this.DataContext = this;
            CwlbDemoServiceBase.ServiceObjectAction += new EventHandler<CwlbInfoEventArgs>(ServiceObjectAction);

            Application.Current.DispatcherUnhandledException += new System.Windows.Threading.DispatcherUnhandledExceptionEventHandler(UnhandledException);
            InitializeComponent();
        }

        /// <summary>
        /// Ereignisbehandlung beim Eintreffen eines unbehandleten Fehlers in der Hauptanwendung.
        /// </summary>
        private void UnhandledException(object sender, System.Windows.Threading.DispatcherUnhandledExceptionEventArgs e)
        {
            WriteInfo(sender, new CwlbInfoEventArgs { Text = e.Exception.Message });
        }

        /// <summary>
        /// Ereignisbehandlung wenn das Hauptfenster fertig geladen ist.
        /// </summary>
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            _cobjHost = new List<ServiceHost>();
            OpenHosts();
        }

        /// <summary>
        /// Ereignisbehandlung beim Beenden des Hauptfensters.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
        }

        /// <summary>
        /// Ereignisbehandlung für das Betätigen der Schaltflöche 'Starten der Diensthosts'
        /// </summary>
        private void ButtonOpenHosts_Clicked(object sender, RoutedEventArgs e)
        {
            OpenHosts();
        }

        /// <summary>
        /// Ereignisbehandlung für das Betätigen der Schaltflöche 'Beenden der Diensthosts'
        /// </summary>
        private void ButtonCloseHosts_Clicked(object sender, RoutedEventArgs e)
        {
            CloseHosts();
        }


        /// <summary>
        /// Öffnet alle registrierten Hosts für die Verarbeitung.
        /// </summary>
        private void OpenHosts()
        {
            try
            {
                if (_cobjHost.Count == 0)
                {
                    RegisterHost(new ServiceHost(typeof(CwlbDemoServicePerCall)));
                    RegisterHost(new ServiceHost(typeof(CwlbDemoServicePerSession)));
                    RegisterHost(new ServiceHost(typeof(CwlbDemoServiceSingle)));
                    RegisterHost(new ServiceHost(typeof(CwlbDemoServiceShared)));
                    RegisterHost(new ServiceHost(typeof(CwlbCallbackDemoServiceSingle)));
                    RegisterHost(new ServiceHost(typeof(CwlbDemoServiceIncludeExceptionInFaultsSingle)));
                }
                foreach (ServiceHost objHost in _cobjHost)
                {
                    objHost.Open();
                    
                }
            }
            catch (Exception excObject)
            {
                // Mitteilung in das Log schreiben
                WriteInfo(this, new CwlbInfoEventArgs
                {
                    Sender = "Host öffnen",
                    Text = excObject.Message
                });
            }
        }

        /// <summary>
        /// Öffnet alle registrierten Hosts für die Verarbeitung.
        /// </summary>
        private void CloseHosts()
        {
            try
            {
                foreach (ServiceHost objHost in _cobjHost)
                {
                    if (objHost.State == CommunicationState.Faulted)
                    {
                        objHost.Abort();
                    }
                    if (objHost.State == CommunicationState.Opened)
                    {
                        objHost.Close();
                    }
                    objHost.Opened -= new EventHandler(Host_Opened);
                    objHost.Faulted -= new EventHandler(Host_Faulted);
                    objHost.Closed -= new EventHandler(Host_Closed);
                    objHost.UnknownMessageReceived -= new EventHandler<UnknownMessageReceivedEventArgs>(Host_UnknownMessageReceived);
                }
                _cobjHost.Clear();
            }
            catch (Exception excObject)
            {
                // Mitteilung in das Log schreiben
                WriteInfo(this, new CwlbInfoEventArgs
                {
                    Sender = "Host schliessen",
                    Text = excObject.Message
                });
            }
        }

        /// <summary>
        /// Registriert einen Host.
        /// </summary>
        /// <param name="objHost"></param>
        private void RegisterHost(ServiceHost objHost)
        {
            objHost.Opened += new EventHandler(Host_Opened);
            objHost.Faulted += new EventHandler(Host_Faulted);
            objHost.Closed += new EventHandler(Host_Closed);
            objHost.UnknownMessageReceived += new EventHandler<UnknownMessageReceivedEventArgs>(Host_UnknownMessageReceived);
            _cobjHost.Add(objHost);
        }

        /// <summary>
        /// Ereignisbehandlung beim Öffnen eines Host.
        /// </summary>
        private void Host_Opened(object sender, EventArgs e)
        {
            WriteInfo(sender, new CwlbInfoEventArgs { Text = "Diensthost gestartet", Sender = ((ServiceHost)sender).Description.Name });
            foreach (ServiceEndpoint endpoint in ((ServiceHost)sender).Description.Endpoints)
            {
                CwlbInfoEventArgs objEvent = new CwlbInfoEventArgs();
                objEvent.Sender = string.Format("Endpunkt: {0}", endpoint.Name);
                objEvent.Text = string.Format("Adresse: {0}", endpoint.Address);
                WriteInfo(sender, objEvent);
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim SChliessen eines Host.
        /// </summary>
        private void Host_Closed(object sender, EventArgs e)
        {
            WriteInfo(sender, new CwlbInfoEventArgs { Text = "Diensthost geschlossen", Sender = ((ServiceHost)sender).Description.Name });
        }

        /// <summary>
        /// Ereignisbehandlung bei fehlerhaftem Verhalten eines Host.
        /// </summary>
        private void Host_Faulted(object sender, EventArgs e)
        {
            WriteInfo(sender, new CwlbInfoEventArgs { Text = "Diensthost fehlerhaft", Sender = ((ServiceHost)sender).Description.Name });
        }

        /// <summary>
        /// Ereignisbehandlung beim Erhalt einer unbekannten Mitteilung.
        /// </summary>
        private void Host_UnknownMessageReceived(object sender, UnknownMessageReceivedEventArgs e)
        {
            WriteInfo(sender, new CwlbInfoEventArgs { Text = "Unbekannte Nachricht erhalten", Sender = ((ServiceHost)sender).Description.Name });
        }

        /// <summary>
        /// Ereignisbehandlung beim Erhalt eines Informationsobjektes der Dienstobjekte.
        /// </summary>
        private void ServiceObjectAction(object sender, CwlbInfoEventArgs objEventArgs)
        {
            if (string.IsNullOrWhiteSpace(objEventArgs.Sender))
            {
                objEventArgs.Sender = sender.GetType().Name;
            }

            WriteInfo(sender, objEventArgs);
        }

        /// <summary>
        /// Speichert eine empfangene Information eines Dienstobjekts oder einer andern Quelle in die 
        /// Liste der Nachrichten zur Visualisierung.
        /// </summary>
        private void WriteInfo(object sender, CwlbInfoEventArgs eventArgs)
        {
            if (!Dispatcher.CheckAccess())
            {
                // Durchführen des Threadwechsels wenn notwendig.
                EventHandler<CwlbInfoEventArgs> objWriteInfoDelegate = WriteInfo;
                Dispatcher.BeginInvoke(objWriteInfoDelegate, new object[] { sender, eventArgs });
            }
            else
            {
                InfoEventArgsCollection.Add(eventArgs);
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Verändern der Grösse des Listenelements
        /// </summary>
        private void LogView_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            ListView lvLog = sender as ListView;
            if (lvLog == null || e.PreviousSize.Width == 0.0)
            {
                return;
            }

            // Size the columns
            double factor = e.NewSize.Width / e.PreviousSize.Width;
            foreach (var column in ((GridView)lvLog.View).Columns)
            {
                column.Width = column.ActualWidth * factor;
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Löschen der Logliste.
        /// </summary>
        private void ButtonDeleteLog_Clicked(object sender, RoutedEventArgs e)
        {
            InfoEventArgsCollection.Clear();
        }
    }
}
