﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WeroSoft.Samples.LearningWcfContract;
using System.ServiceModel;
using System.Reflection;

namespace WeroSoft.Samples.LearningWcfImplementation
{
    /// <summary>
    /// Implementierung des Singleton Dienstes mit konfigurierten Ausnahmedaten.
    /// </summary>
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single, IncludeExceptionDetailInFaults=true)]
    public class CwlbDemoServiceIncludeExceptionInFaultsSingle : CwlbDemoServiceSingle 
    {
        /// <summary>
        /// Löst eine Ausnahme ohne eigene Fehlerbehandlung im Dienst aus.
        /// </summary>
        /// <param name="iDividend">Dividend für die Division.</param>
        /// <param name="iDivisor">Divisor für die Division.</param>
        /// <returns>Resultat der Division.</returns>
        public override int Divide_NoFaultHandling(int iDividend, int iDivisor)
        {
            return iDividend / iDivisor;
        }

        /// <summary>
        /// Löst eine Ausnahme mit eigener minimalen Fehlerbahndlung aus.
        /// </summary>
        /// <param name="iDividend">Dividend für die Division.</param>
        /// <param name="iDivisor">Divisor für die Division.</param>
        /// <returns>Resultat der Division.</returns>
        public override int Divide_WithSimpleFaultHandling(int iDividend, int iDivisor)
        {
            try
            {
                return iDividend / iDivisor;
            }
            catch (Exception excObject)
            {
                throw new FaultException(excObject.Message);
            }
        }

        /// <summary>
        /// Löst eine Ausnahme mit eigener Fehlerbehandlung und dem Typ CwlbFaultDetail.
        /// </summary>
        /// <param name="iDividend">Dividend für die Division.</param>
        /// <param name="iDivisor">Divisor für die Division.</param>
        /// <returns>Resultat der Division.</returns>
        public override int Divide_WithFullFeaturedFaultHandling(int iDividend, int iDivisor)
        {
            try
            {
                return iDividend / iDivisor;
            }
            catch (Exception excObject)
            {
                CwlbFaultDetail objFaultDetail = new CwlbFaultDetail();
                objFaultDetail.MachineName = Environment.MachineName;
                objFaultDetail.ServiceName = this.GetType().Name;
                objFaultDetail.MethodName = MethodInfo.GetCurrentMethod().Name;
                throw new FaultException<CwlbFaultDetail>(objFaultDetail, new FaultReason(excObject.Message), new FaultCode("MATH026", new FaultCode("MATH42")));
            }
        }
    }
}
