﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel.Description;
using System.ServiceModel;
using System.Collections.ObjectModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Dispatcher;

namespace WeroSoft.Samples.LearningWcfImplementation
{
    /// <summary>
    /// Definiert das notwendige Attribut für die Dekoration 
    /// einer Dienstimplementierung für geteilte Instanziierung.
    /// </summary>
    public class CwlbShareableServiceAttribute : Attribute, IServiceBehavior
    {
        /// <summary>
        /// Bietet die Möglichkeit, benutzerdefinierte Daten an Bindungselemente zu
        /// übergeben, um die Vertragsimplementierung zu unterstützen.
        /// </summary>
        /// <param name="objDescription">Beschreibung des Deinstes.</param>
        /// <param name="objServiceHostBase">Host des Dienstes.</param>
        /// <param name="objEndpoints">Liste der Dienstendpunkte.</param>
        /// <param name="objParameters">Benutzerdefinierte Objekte, die auf die Bindungselemente Zugrif haben.</param>
        public void AddBindingParameters(
            ServiceDescription objDescription, 
            ServiceHostBase objServiceHostBase, 
            Collection<ServiceEndpoint> objEndpoints, 
            BindingParameterCollection objParameters)
        {
        }

        /// <summary>
        /// Stellt die Funktion zum Ändern von Eigenschaftenwerten zur Laufzeit oder zum Einfügen 
        /// von benutzerdefinierten Erweiterungsobjekten, wie Fehlerhandler, Nachrichten- oder 
        /// Parameterinterceptoren, Sicherheitserweiterungen und anderen benutzerdefinierten 
        /// Erweiterungsobjekten, bereit. 
        /// </summary>
        /// <param name="objDescription">Beschreibung des Dienstes.</param>
        /// <param name="objServiceHostBase">Host des Dienstes.</param>
        public void ApplyDispatchBehavior(ServiceDescription objDescription, ServiceHostBase objServiceHostBase)
        {
            // Erstellen des eigenen Kontextproviders.
            CwlbContextProvider objExtension = new CwlbContextProvider();

            // Konfiguration des eigenen Dispatcherproviders für jeden Endpunktdispatcher
            // eines jeden Channeldispatchers des Service Hosts.
            foreach (ChannelDispatcher objChannelDispatcher in objServiceHostBase.ChannelDispatchers)
            {
                foreach (EndpointDispatcher objEndpointDispatcher in objChannelDispatcher.Endpoints)
                {
                    objEndpointDispatcher.DispatchRuntime.InstanceContextProvider = objExtension;
                }
            }
        }

        /// <summary>
        /// Stellt die Funktion zur Verfügung, mit der der Diensthost und die Dienstbeschreibung 
        /// geprüft werden können, um sicherzustellen, dass der Dienst erfolgreich ausgeführt wird. 
        /// </summary>
        /// <param name="objDescription">Beschreibung des Dienstes.</param>
        /// <param name="objServiceHostBase">Host des Dienstes.</param>
        public void Validate(ServiceDescription objDescription, ServiceHostBase objServiceHostBase)
        {
        }
    }
}
