﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.MsmqIntegration;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert den Service Host.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Einsprungpunkt in das Programm.
        /// </summary>
        static void Main(string[] astrArguments)
        {
            // Definiert den Service Host.
            ServiceHost objHost = null;
            CwlbArgumentHelper objArgumentHelper = new CwlbArgumentHelper();
            objArgumentHelper.Parse(astrArguments);
            if (objArgumentHelper.Help)
            {
                objArgumentHelper.ShowHelp();
                return;
            }
            else
            {
                objArgumentHelper.ShowConfiguration();
            }

            try
            {
                //// Definition des Modus über den Konstruktor
                //NetTcpContextBinding objBinding = new NetTcpContextBinding(SecurityMode.Transport);
                //// Oder alternativ über die Eigenschaft
                //objBinding.Security.Mode = SecurityMode.Transport;
                //objBinding.Security.Transport.ClientCredentialType = TcpClientCredentialType.None;
                //objBinding.Security.Transport.ProtectionLevel = System.Net.Security.ProtectionLevel.Sign;

                //// Definition des Modus über den Konstruktor
                //NetNamedPipeBinding objBinding = new NetNamedPipeBinding(NetNamedPipeSecurityMode.Transport);
                //objBinding.Name = "NamedPipeBinding";
                //// Oder alternativ über die Eigenschaft
                //objBinding.Security.Mode = NetNamedPipeSecurityMode.Transport;
                //objBinding.Security.Transport.ProtectionLevel = System.Net.Security.ProtectionLevel.EncryptAndSign;


                //// Definition des Modus über den Konstruktor
                //BasicHttpBinding objBinding = new BasicHttpBinding(BasicHttpSecurityMode.Transport);
                //objBinding.Name = "BasicHttpBinding";
                //// Oder alternativ über die Eigenschaft
                //objBinding.Security.Mode = BasicHttpSecurityMode.Transport;
                //objBinding.Security.Transport.ClientCredentialType = HttpClientCredentialType.Certificate;
                //objBinding.Security.Transport.ProxyCredentialType = HttpProxyCredentialType.Windows;
                //objBinding.Security.Transport.Realm = "Sample";


                // Definition des Modus über den Konstruktor
                WSHttpBinding objBinding = new WSHttpBinding(SecurityMode.Transport);
                objBinding.Name = "WSHttpBinding";
                // ... oder alternativ über die Eigenschaft
                objBinding.Security.Mode = SecurityMode.Transport;
                objBinding.Security.Transport.ClientCredentialType = HttpClientCredentialType.Certificate;
                objBinding.Security.Transport.ProxyCredentialType = HttpProxyCredentialType.Windows;
                objBinding.Security.Transport.Realm = "Sample";

                BasicHttpContextBinding obj2 = new BasicHttpContextBinding(BasicHttpSecurityMode.Transport);

                WebHttpBinding obj3 = new WebHttpBinding(WebHttpSecurityMode.Transport);

                WSHttpContextBinding obj5 = new WSHttpContextBinding(SecurityMode.Transport);

                WSDualHttpBinding obj6 = new WSDualHttpBinding(WSDualHttpSecurityMode.Message);
                obj6.Security.Mode = WSDualHttpSecurityMode.Message;


                WSFederationHttpBinding obj7 = new WSFederationHttpBinding(WSFederationHttpSecurityMode.Message);

                NetPeerTcpBinding obj11 = new NetPeerTcpBinding();

                NetMsmqBinding obj12 = new NetMsmqBinding(NetMsmqSecurityMode.Transport);

                MsmqIntegrationBinding obj13 = new MsmqIntegrationBinding(MsmqIntegrationSecurityMode.None);

                // Erstellen des Diensthost für den Dienst CwlbDemoService
                objHost = new ServiceHost(typeof(CwlbDemoService));
                // Defineren eines Kommunikationsendpunktes
                objHost.AddServiceEndpoint(
                    typeof(IwlbDemoService),
                    objArgumentHelper.CreateBinding(), 
                    objArgumentHelper.CreateAddress());
                // Diensthost starten
                objHost.Open();
                // Verhindern das Prozess beendet wird.
                Console.WriteLine("Service Host ist gestartet. Mit einem Tastendruck kann er beendet werden ...");
                Console.ReadKey(true);
            }
            catch (Exception excObject)
            {
                // Ausnahmebehandlung
                Console.WriteLine(excObject.Message);
            }
            finally
            {
                // Beenden des Dienstes im Normalfall
                if (objHost != null && objHost.State == CommunicationState.Opened)
                {
                    objHost.Close();
                }
                // Beenden des Dienstes im Ausnahmefall
                if (objHost != null && objHost.State == CommunicationState.Faulted)
                {
                    objHost.Abort();
                }
            }
        }
    }
}
