﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementierung des Dienstes IwlbDemoService.
    /// </summary>
    public class CwlbDemoService : IwlbDemoService
    {
        /// <summary>
        /// Liefert ein Array von Daten.
        /// </summary>
        /// <param name="demoData">Datenobjekt, das als Vorlage vom Client an den Server gesendet wird.</param>
        /// <param name="iParam">Anzahl der zu generierenden Elemente in der Rückgabe. Dieser Wert darf nicht kleiner als 1 sein.</param>
        /// <returns>Array von serverseitig erzeugten Elementen.</returns>
        public CwlbDemoData[] GetDemoData(CwlbDemoData objDemoData, int iParam)
        {
            // Prüfen der Eingabeparameter
            if (objDemoData == null)
            {
                CwlbFault objFault = new CwlbFault();
                objFault.Code = 1;
                objFault.Message = "Die Vorlage muss angegeben werden.";
                throw new FaultException<CwlbFault>(
                    objFault, 
                    new FaultReason("Parameterprüfung"),
                    new FaultCode("1"));
            }
            if (iParam < 1)
            {
                CwlbFault objFault = new CwlbFault();
                objFault.Code = 2;
                objFault.Message = "Die Anzahl der angeforderten Elemente darf nicht kleiner als 1 sein.";
                throw new FaultException<CwlbFault>(
                    objFault, 
                    new FaultReason("Parameterprüfung"), 
                    new FaultCode("1"));
            }

            // Herstellen der Daten
            List<CwlbDemoData> cobjData = new List<CwlbDemoData>();
            for (int iCount = 0; iCount < iParam; iCount++)
            {
                CwlbDemoData objData = new CwlbDemoData();
                objData.IntegerValue = iCount + 1;
                objData.StringValue = objDemoData.StringValue + " + Antwort";
                objData.DateTimeValue = new DateTime(2012, 1, 1) + TimeSpan.FromDays(10 * iCount);
                cobjData.Add(objData);
            }

            return cobjData.ToArray();
        }
    }
}
