﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using System.ServiceModel.Activation;
using System.Web;

namespace WeroSoft.Samples
{
    /// <summary>
    /// 
    /// </summary>
    [AspNetCompatibilityRequirements(RequirementsMode=AspNetCompatibilityRequirementsMode.Required)]
    [ServiceBehavior(InstanceContextMode=InstanceContextMode.PerCall)]
    public class CwlbAspNetCompatibilityDemoService : IwlbAspNetCompatibilityDemoService
    {
        private const string LastValue = "LastValue";

        /// <summary>
        /// Liefert ein Datenelement, dessen Zahl interne Zahl von der Anzahl der Zugriffe abhängt.
        /// </summary>
        /// <param name="demoData">Datenobjekt, das als Vorlage vom Client an den Server gesendet wird.</param>
        /// <returns>Serverseitig verändertes Element.</returns>
        public CwlbDemoData GetDemoData(CwlbDemoData objDemoData)
        {
            // Den letzten Wert aus den Sitzungsdaten auslesen
            int ? iValue = (int?)HttpContext.Current.Session[LastValue];
            iValue = iValue.GetValueOrDefault() + 1;

            // Herstellen der Daten
            CwlbDemoData objData = new CwlbDemoData();
            objData.IntegerValue = iValue ?? 0;
            objData.DateTimeValue = DateTime.Now;
            objData.StringValue = objDemoData.StringValue + " + Antwort";

            // Den neuen Wert speichern und die Daten zurückgeben
            HttpContext.Current.Session[LastValue] = iValue;
            return objData;
        }

        /// <summary>
        /// Liefert eine Auswahl von HTTP-Kontextinformationen.
        /// </summary>
        /// <returns>Verzeicnis mit Kontextinformation und Inhalt.</returns>
        public Dictionary<string, string> GetContextInfo()
        {
            Dictionary<string, string> cobjContextData = new Dictionary<string, string>();

            cobjContextData.Add("Request", new string('-', 40));
            cobjContextData.Add("Request.ApplicationPath", HttpContext.Current.Request.ApplicationPath);
            cobjContextData.Add("Request.AppRelativeCurrentExecutionFilePath", HttpContext.Current.Request.AppRelativeCurrentExecutionFilePath);
            cobjContextData.Add("Request.ContentLength", HttpContext.Current.Request.ContentLength.ToString());
            cobjContextData.Add("Request.CurrentExecutionFilePath", HttpContext.Current.Request.CurrentExecutionFilePath);
            cobjContextData.Add("Request.FilePath", HttpContext.Current.Request.FilePath);
            cobjContextData.Add("Request.HttpMethod", HttpContext.Current.Request.HttpMethod);
            cobjContextData.Add("Request.IsAuthenticated", HttpContext.Current.Request.IsAuthenticated.ToString());
            cobjContextData.Add("Request.IsLocal", HttpContext.Current.Request.IsLocal.ToString());
            cobjContextData.Add("Request.IsSecureConnection", HttpContext.Current.Request.IsSecureConnection.ToString());
            cobjContextData.Add("Request.Path", HttpContext.Current.Request.Path);
            cobjContextData.Add("Request.PathInfo", HttpContext.Current.Request.PathInfo);
            cobjContextData.Add("Request.PhysicalApplicationPath", HttpContext.Current.Request.PhysicalApplicationPath);
            cobjContextData.Add("Request.PhysicalPath", HttpContext.Current.Request.PhysicalPath);
            cobjContextData.Add("Request.RawUrl", HttpContext.Current.Request.RawUrl);
            cobjContextData.Add("Request.RequestType", HttpContext.Current.Request.RequestType);
            cobjContextData.Add("Request.TotalBytes", HttpContext.Current.Request.TotalBytes.ToString());
            cobjContextData.Add("Request.Url.OriginalString", HttpContext.Current.Request.Url.OriginalString);
            cobjContextData.Add("Request.UserAgent", HttpContext.Current.Request.UserAgent);
            cobjContextData.Add("Request.UserHostAddress", HttpContext.Current.Request.UserHostAddress);
            cobjContextData.Add("Request.UserHostName", HttpContext.Current.Request.UserHostName);
            cobjContextData.Add("Response", new string('-', 40));
            cobjContextData.Add("Response.ContentType", HttpContext.Current.Response.ContentType);
            cobjContextData.Add("Response.Expires", HttpContext.Current.Response.Expires.ToString());
            cobjContextData.Add("Response.IsClientConnected", HttpContext.Current.Response.IsClientConnected.ToString());
            cobjContextData.Add("Response.RedirectLocation", HttpContext.Current.Response.RedirectLocation);
            cobjContextData.Add("Response.Status", HttpContext.Current.Response.Status);
            cobjContextData.Add("Response.StatusCode", HttpContext.Current.Response.StatusCode.ToString());
            cobjContextData.Add("Response.StatusDescription", HttpContext.Current.Response.StatusDescription);
            cobjContextData.Add("Server", new string('-', 40));
            cobjContextData.Add("Server.MachineName", HttpContext.Current.Server.MachineName);
            cobjContextData.Add("Server.ScriptTimeout", HttpContext.Current.Server.ScriptTimeout.ToString());
            cobjContextData.Add("Server.Variables", new string('-', 40));
            foreach (var objItem in HttpContext.Current.Request.ServerVariables)
            {
                if (!objItem.ToString().StartsWith("ALL"))
                {
                    cobjContextData.Add(
                        string.Format("Servervariable[{0}]", objItem.ToString()),
                        string.Format("{0}", HttpContext.Current.Request.ServerVariables[objItem.ToString()]));
                }
            }
            return cobjContextData;
        }
    }
}
