﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel.Channels;
using System.Net;

namespace WeroSoft.Samples.Testclient
{
    /// <summary>
    /// Definiert eine Hilfsklasse für die ASP.NET Sitzung
    /// </summary>
    internal class CwlbAspNetSessionId
    {
        // Konstante für die Erkennnung des ASP.NET Session Cookie
        private const string AspNetSessionIdCookieName = "ASP.NET_SessionId";

        /// <summary>
        /// Liefert die Sitzungsidentifikation der ASP.NET Sitzung.
        /// </summary>
        public string AspNetSessionId { get; private set; }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbAspNetSessionId
        /// </summary>
        /// <param name="strCookie">Zeichenfolge des empfangenen Cookies.</param>
        public CwlbAspNetSessionId(string strCookie)
        {
            if (string.IsNullOrEmpty(strCookie))
            {
                return;
            }

            string[] astrCookie = strCookie.Split(';');
            for (int i = 0; i < astrCookie.Length; i++)
            {
                string[] astrCookieName = astrCookie[i].Split('=');
                if (astrCookieName[0] == AspNetSessionIdCookieName)
                {
                    AspNetSessionId = astrCookieName[1];
                    return;
                }
            }
        }

        /// <summary>
        /// Fügt das Cookie mit der Sitzungskennung in die Anfrage ein.
        /// </summary>
        /// <param name="objRequestMessageProperty">Anfrage.</param>
        public void InjectSessionId(HttpRequestMessageProperty objRequestMessageProperty)
        {
            if (string.IsNullOrEmpty(AspNetSessionId))
            {
                return;
            }

            string strSessionId = string.Format("{0}={1}", AspNetSessionIdCookieName, AspNetSessionId);
            string strCookie = (string)objRequestMessageProperty.Headers[HttpRequestHeader.Cookie];
            if (string.IsNullOrEmpty(strCookie))
            {
                strCookie = strSessionId;
            }
            else
            {
                strCookie = string.Format("{0}; {1}", strCookie, strSessionId);
            }

            objRequestMessageProperty.Headers[HttpRequestHeader.Cookie] = strCookie;
        }
    }
}
