﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Net;

namespace WeroSoft.Samples.Testclient
{
    /// <summary>
    /// Clientseitiges Testprogramm.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Einstiegspunkt für den Programmstart.
        /// </summary>
        static void Main()
        {
            DemoService.DemoServiceClient objProxy = null;
            DemoServiceLib.DemoServiceClient objProxyLib = null;
            AspNetCompatibility.AspNetCompatibilityDemoServiceClient objProxyAspNetCompatibility = null;
            Was.DemoServiceClient objProxyWas = null;

            try
            {
                // Normalen Client erstellen
                objProxy = new DemoService.DemoServiceClient();
                objProxyLib = new DemoServiceLib.DemoServiceClient();
                objProxyAspNetCompatibility = new AspNetCompatibility.AspNetCompatibilityDemoServiceClient();
                objProxyWas = new Was.DemoServiceClient();

                // Aufruf ohne Fehler
                Console.WriteLine("Aufruf ohne Fehler auf direkt implementierten Dienst ...");
                DemoService.DemoData objData = new DemoService.DemoData { DateTimeValue = DateTime.Now, StringValue = "Anfrage" };
                DemoService.DemoData[] aobjData = objProxy.GetDemoData(objData, 20);
                foreach (DemoService.DemoData objItem in aobjData)
                {
                    Console.WriteLine(string.Format("{0,5}. {1} - {2}",
                        objItem.IntegerValue,
                        objItem.DateTimeValue.ToShortDateString(),
                        objItem.StringValue));
                }

                // Aufruf des Dienstes in der Bibliothek
                Console.WriteLine();
                Console.WriteLine("Aufruf ohne Fehler auf indirekt implementierten Dienst (Lib) ...");
                DemoServiceLib.DemoData objDataLib = new DemoServiceLib.DemoData { DateTimeValue = DateTime.Now, StringValue = "Anfrage Library" };
                DemoServiceLib.DemoData[] aobjDataLib = objProxyLib.GetDemoData(objDataLib, 20);
                foreach (DemoServiceLib.DemoData objItem in aobjDataLib)
                {
                    Console.WriteLine(string.Format("{0,5}. {1} - {2}",
                        objItem.IntegerValue,
                        objItem.DateTimeValue.ToShortDateString(),
                        objItem.StringValue));
                }

                // Aufruf des Dienstes mit ASP.NET Sitzungskompatibilität
                Console.WriteLine();
                Console.WriteLine("Aufruf ohne Fehler auf ASP.NET kompatiblen Dienst ...");
                CwlbAspNetSessionId objAspNetSeesion = null;

                // Verwendung des Kontextes für die Anfragen
                using (new OperationContextScope((IContextChannel)objProxyAspNetCompatibility.InnerChannel))
                {

                    bool bFirstCall = true;

                    // 10 Anfragen nacheinander durchführen (gleiche Sitzung)
                    for (int iCall = 0; iCall < 10; iCall++)
                    {
                        // Anfragedaten bereitstellen
                        AspNetCompatibility.DemoData objAspNetCompatibilityItem = new AspNetCompatibility.DemoData { DateTimeValue = DateTime.Now, StringValue = "Anfrage AspNet Compatibility" };
                        if (bFirstCall)
                        {
                            // Fall erste Anfrage
                            objAspNetCompatibilityItem = objProxyAspNetCompatibility.GetDemoData(objAspNetCompatibilityItem);
                            HttpResponseMessageProperty responseProperty = OperationContext.Current.IncomingMessageProperties[HttpResponseMessageProperty.Name] as HttpResponseMessageProperty;
                            objAspNetSeesion = new CwlbAspNetSessionId((string)responseProperty.Headers[HttpResponseHeader.SetCookie]);
                            bFirstCall = false;
                        }
                        else
                        {
                            // Fall wiederholte Anfrage
                            HttpRequestMessageProperty requestProperty = new HttpRequestMessageProperty();
                            objAspNetSeesion.InjectSessionId(requestProperty);
                            OperationContext.Current.OutgoingMessageProperties[HttpRequestMessageProperty.Name] = requestProperty;
                        }

                        // Anfrage durchführen
                        objAspNetCompatibilityItem = objProxyAspNetCompatibility.GetDemoData(objAspNetCompatibilityItem);
                        // Daten ausgeben
                        Console.WriteLine(string.Format("{0,5}. {1} - {2}",
                            objAspNetCompatibilityItem.IntegerValue,
                            objAspNetCompatibilityItem.DateTimeValue.ToShortDateString(),
                            objAspNetCompatibilityItem.StringValue));
                    }

                    // Abfrage der HTTP-Kontextdaten (eine Auswahl)
                    Dictionary<string, string> cobjContextInfo = objProxyAspNetCompatibility.GetContextInfo();
                    foreach (KeyValuePair<string, string> objItem in cobjContextInfo)
                    {
                        Console.WriteLine("{0,-50}{1}", objItem.Key, objItem.Value);
                    }

                    // Aufruf ohne Fehler
                    Console.WriteLine("Aufruf ohne Fehler auf direkt implementierten Dienst ...");
                    Was.DemoData objDataWas = new Was.DemoData { DateTimeValue = DateTime.Now, StringValue = "Anfrage" };
                    Was.DemoData[] aobjDataWas = objProxyWas.GetDemoData(objDataWas, 20);
                    foreach (Was.DemoData objItem in aobjDataWas)
                    {
                        Console.WriteLine(string.Format("{0,5}. {1} - {2}",
                            objItem.IntegerValue,
                            objItem.DateTimeValue.ToShortDateString(),
                            objItem.StringValue));
                    }


                }
            }
            catch (CommunicationException excObject)
            {
                // Verarbeitung einer allgmeinen Kommunikationsausnahme
                Console.WriteLine(excObject.Message);
            }
            catch (Exception excObject)
            {
                // Verarbeitung aller andern Ausnahmen
                Console.WriteLine(excObject.Message);
            }
            finally
            {
                // Beenden des Clients im Normalfall
                if (objProxy != null && objProxy.State == CommunicationState.Opened)
                {
                    objProxy.Close();
                }
                // Beenden des Clients im Ausnahmefall
                if (objProxy != null && objProxy.State == CommunicationState.Faulted)
                {
                    objProxy.Abort();
                }
            }
        }
    }
}
