﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.ServiceProcess;
using System.Text;

namespace WeroSoft.Samples.DemoWindowsService
{
    /// <summary>
    /// Implementiert den eigentlichen Windows-Dienst.
    /// </summary>
    public partial class CwlbDemoService : ServiceBase
    {
        // Eigener Dienst verwenden
        private CwlbDemoService _objService;

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbDemoService.
        /// </summary>
        public CwlbDemoService()
        {
     #if DEBUG
            if (Environment.GetEnvironmentVariable("LauchDebugLearningService") == "Yes")
            {
                Debugger.Launch();
            }
     #endif

            InitializeComponent();
        }

        /// <summary>
        /// Wird vom SCM beim Starten des Windows-Dienstes aufgerufen.
        /// </summary>
        /// <param name="args">Argumente für den Start des Windows-Dienstes.</param>
        protected override void OnStart(string[] args)
        {
            _objService = new CwlbDemoService();
            _objService.OnStart(args);
        }

        /// <summary>
        /// Wird vom SCM beim Beenden des Windows-Dienstes aufgerufen.
        /// </summary>
        protected override void OnStop()
        {
            _objService.OnStop();
        }

        /// <summary>
        /// Wird vom SCM beim Pausieren des Windows-Dienstes aufgerufen.
        /// </summary>
        protected override void OnPause()
        {
            base.OnPause();
            EventLog.WriteEntry("Learning Windows Service - Pause.");
        }

        /// <summary>
        /// Wird vom SCM beim Weiterführen des Windows-Dienstes aufgerufen-.
        /// </summary>
        protected override void OnContinue()
        {
            base.OnContinue();
            EventLog.WriteEntry("Learning Windows Service - Continue.");
        }

        /// <summary>
        /// Wird vom SCM bei einem Ereignis im Zusammenhang mit der Energieversorgung des Compuerts aufgerufen.
        /// </summary>
        /// <param name="powerStatus">Aktueller Energiestatus.</param>
        /// <returns>Wird im Wesentlichen dazu verwendet, eine Arbeitsunterbrechung zu bewilligen oder nicht.</returns>
        protected override bool OnPowerEvent(PowerBroadcastStatus powerStatus)
        {
            EventLog.WriteEntry("Learning Windows Service - PowerEvent, " + powerStatus.ToString());
            return base.OnPowerEvent(powerStatus);
        }

        /// <summary>
        /// Wird vom SCM aufgerufen, wenn eine Änderung in den Sitzungen auf dem Computer eingetreten ist.
        /// </summary>
        /// <param name="changeDescription">Beschreibung der Änderung der Sitzungen.</param>
        protected override void OnSessionChange(SessionChangeDescription changeDescription)
        {
            EventLog.WriteEntry("Learning Windows Service - SessionChange, " + changeDescription.Reason.ToString());
            base.OnSessionChange(changeDescription);
        }

        /// <summary>
        /// Wird vom SCM aufgerufen, wenn der Computer heruntergefahren wird.
        /// </summary>
        protected override void OnShutdown()
        {
            EventLog.WriteEntry("Learning Windows Service - Shutdown.");
            base.OnShutdown();
        }
    }
}
