﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Threading;
using WeroSoft.Samples.DemoService;

namespace WeroSoft.Samples.Serv_iceSimulator
{
    class Program
    {
        private static object SimulatorSynchronization = new object();

        private static CwlbDemoService Service;

        static void Main(string[] args)
        {
            try
            {
                // Ausschalten des Cursors
                Console.CursorVisible = false;

                // Erstellen und starten des Dienstes
                Service = new CwlbDemoService();
                Service.OnStart(args);

                // Eingaben im Simulator verwalten
                CheckEntry();

                // Dienst beenden
                Service.OnStop();
            }
            catch (Exception exception)
            {
                WriteLine(ConsoleColor.Red, "Ein nicht erwartete Ausnahme ist aufgetreten. Die Meldung lautet: '{0}'.", exception.Message);
                if (exception.InnerException != null)
                {
                    WriteLine(ConsoleColor.Red, "Der Text der inneren Ausnahme lautet: '{0}'.", exception.InnerException.Message);
                }
            }
            finally
            {
                Console.CursorVisible = true;
            }
        }

        /// <summary>
        /// Check the keyboard and control methods which are requested by the user.
        /// </summary>
        private static void CheckEntry()
        {
            bool run = true;
            ShowHeader();
            while (run)
            {
                ConsoleKeyInfo keyInfo = Console.ReadKey(true);
                switch (keyInfo.Key)
                {
                    case ConsoleKey.F2:

                        ShowHeader();
                        break;
                    case ConsoleKey.F3:
                        ShowHeader();
                        break;
                    case ConsoleKey.F4:
                        ShowHeader();
                        break;
                    case ConsoleKey.F5:
                        ShowHeader();
                        break;
                    case ConsoleKey.F6:
                        ShowHeader();
                        break;
                    case ConsoleKey.F7:
                        ShowHeader();
                        break;
                    case ConsoleKey.F8:
                        ShowProcessInfo();
                        ShowHeader();
                        break;
                    case ConsoleKey.F12:
                        run = false;
                        break;
                    case ConsoleKey.F1:
                    default:
                        Console.Clear();
                        ShowHeader();
                        break;
                }
            }
        }

        /// <summary>
        /// Shows the header information of the simulator.
        /// </summary>
        private static void ShowHeader()
        {
            lock (SimulatorSynchronization)
            {
                ConsoleColor currentColor = Console.ForegroundColor;
                try
                {
                    Console.ForegroundColor = ConsoleColor.DarkBlue;
                    WriteLine();
                    WriteLine("    weroSoft Dienstsimulator");
                    WriteLine("    ------------------------");
                    WriteLine();
                    WriteLine("    Bedienung des Dienstsimulators:");
                    WriteLine();
                    WriteLine("      F1 -> Löscht die Konsole und zeigt diese Hilfe");
                    WriteLine("      F2 -> Dienst starten");
                    WriteLine("      F3 -> Dienst anhalten");
                    WriteLine("      F4 -> Dienst fortsetzen");
                    WriteLine("      F5 -> Dienst stoppen");
                    WriteLine("      F6 -> ");
                    WriteLine("      F7 -> ");
                    WriteLine("      F8 -> Zeige die aktuellen Prozessdaten");
                    WriteLine();
                    WriteLine("     F12 -> Simulator beenden");
                    WriteLine();
                }
                finally
                {
                    Console.ForegroundColor = currentColor;
                }
            }
        }

        /// <summary>
        /// Shows the process information.
        /// </summary>
        private static void ShowProcessInfo()
        {
            while (Console.KeyAvailable == false)
            {
                lock (SimulatorSynchronization)
                {
                    Console.Clear();
                    Process currentProcess = Process.GetCurrentProcess();
                    WriteLine();
                    WriteLine("Allgmeine Prozessinformationen");
                    WriteLine("- Maschinenname:              {0}", Environment.MachineName);
                    WriteLine("- Situngs id:                 {0}", currentProcess.SessionId);
                    WriteLine("- Prozess id:                 {0}", currentProcess.Id);
                    WriteLine("- Prozessname:                {0}", currentProcess.ProcessName);
                    WriteLine("- Basispriorität:             {0}", currentProcess.BasePriority);
                    WriteLine("- Prioritätsklasse:           {0}", currentProcess.PriorityClass);
                    WriteLine("- Anzahl Threads:             {0}", currentProcess.Threads.Count);
                    WriteLine("- Anzahll Handles:            {0}", currentProcess.HandleCount);
                    WriteLine();
                    WriteLine("Prozesseigene Zeitinformationen");
                    WriteLine("- Effektive Laufzeit:         {0}", DateTime.Now - currentProcess.StartTime);
                    WriteLine("- Total Prozessorzeit:        {0}", currentProcess.TotalProcessorTime);
                    WriteLine("- Zeit im geschützten Modus:  {0}", currentProcess.PrivilegedProcessorTime);
                    WriteLine("- Zeit im Benutzermodus:      {0}", currentProcess.UserProcessorTime);
                    WriteLine();
                    WriteLine("Speicherinformation");
                    WriteLine("- Aktuelles Workingset:         {0,9:N2} MB", (double)currentProcess.WorkingSet64 / 1024 / 1024);
                    WriteLine("- Grösse virtueller Speicher:   {0,9:N2} MB", (double)currentProcess.VirtualMemorySize64 / 1024 / 1024);
                    WriteLine("- Grösse privater Speicher:     {0,9:N2} MB", (double)currentProcess.PrivateMemorySize64 / 1024 / 1024);
                    WriteLine("- Nicht ausgelagerter Speicher: {0,9:N2} MB", (double)currentProcess.NonpagedSystemMemorySize64 / 1024 / 1024);
                    WriteLine("- Ausgelagerter Speicher:       {0,9:N2} MB", (double)currentProcess.PagedMemorySize64 / 1024 / 1024);
                    WriteLine("- Ausgelagerter Systemspeicher: {0,9:N2} MB", (double)currentProcess.PagedSystemMemorySize64 / 1024 / 1024);
                    WriteLine("- Peak virtueller Speicher:     {0,9:N2} MB", (double)currentProcess.PeakVirtualMemorySize64 / 1024 / 1024);
                    WriteLine("- Peak Workingset:              {0,9:N2} MB", (double)currentProcess.PeakWorkingSet64 / 1024 / 1024);
                    WriteLine("");
                    WriteLine(ConsoleColor.Magenta, "Dieser Bildschirm wird aktualisiert, bis eine Taste betätigt wird ...");
                }
                Thread.Sleep(1000);
            }
        }

        /// <summary>
        /// Writes an empty line to the console.
        /// </summary>
        static void WriteLine()
        {
            WriteLine(string.Empty);
        }

        /// <summary>
        /// Writes colored output to the console.
        /// </summary>
        /// <param name="color">Color to be used for this output.</param>
        /// <param name="messageFormat">Message which is written to console.</param>
        /// <param name="arguments">Arguments for injection into the message.</param>
        static void WriteLine(ConsoleColor color, string messageFormat, params object[] arguments)
        {
            lock (SimulatorSynchronization)
            {
                ConsoleColor currentColor = Console.ForegroundColor;
                try
                {
                    Console.ForegroundColor = color;
                    WriteLine(messageFormat, arguments);
                }
                finally
                {
                    Console.ForegroundColor = currentColor;
                }
            }
        }

        /// <summary>
        /// Writes output in the current color to the console.
        /// </summary>
        /// <param name="messageFormat">Message which is written to console.</param>
        /// <param name="arguments">Arguments for injection into the message.</param>
        static void WriteLine(string messageFormat, params object[] arguments)
        {
            lock (SimulatorSynchronization)
            {
                try
                {
                    Console.WriteLine(messageFormat, arguments);
                }
                catch
                {
                    // Stabilizing the method in case that arguments and messageformat does not fit.
                }
            }
        }

        /// <summary>
        /// Writes output in the current color to the console.
        /// </summary>
        /// <param name="message">Message which is written to console.</param>
        static void WriteLine(string message)
        {
            lock (SimulatorSynchronization)
            {
                try
                {
                    Console.WriteLine(message);
                }
                catch
                {
                    // Stabilizing the method in case that arguments and messageformat does not fit.
                }
            }
        }
    }
}
