﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Threading;

namespace WeroSoft.Samples.Client
{
    /// <summary>
    /// Testprogramm für den Client.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Einstiegspunkt in das Testprogramm
        /// </summary>
        static void Main()
        {
            // ----------------------------------------------------------------
            // Einfacher Client
            // ----------------------------------------------------------------
            // Clientobjekt deklarieren
            IwlbDemoService objClient = null;

            try
            {
                // Identifikation
                Console.WriteLine("Client Factory imperativ");

                // Client erstellen
                //// Bindung konfigurieren
                //NetTcpBinding objBinding = new NetTcpBinding(SecurityMode.None);
                //objBinding.CloseTimeout = TimeSpan.FromSeconds(Properties.Settings.Default.ClientOpenTimeout);
                //objBinding.ReaderQuotas.MaxArrayLength = int.MaxValue;
                //objBinding.ReliableSession.Ordered = true;

                //// Adresse konfigurieren
                //EndpointAddress objEndpointAddress = new EndpointAddress("net.tcp://localhost:4711/TcpDemoService");

                //// Client erstellen 
                //objClient = ChannelFactory<IwlbDemoService>.CreateChannel(objBinding, objEndpointAddress);
                objClient = ChannelFactory<IwlbDemoService>.CreateChannel(
                    new NetTcpBinding(),
                    new EndpointAddress("net.tcp://localhost:4711/TcpDemoService"));
                using (objClient as IDisposable)
                {
                    // Daten bereitstellen und Kommunikation durchführen
                    CwlbDemoData objDemoData = new CwlbDemoData { StringValue = "Anforderung" };
                    CwlbDemoData[] aobjDemoData = objClient.GetDemoData(objDemoData, 10);
                    foreach (CwlbDemoData objItem in aobjDemoData)
                    {
                        Console.WriteLine(string.Format("{0} {1,2}. {2}",
                            objItem.DateTimeValue.ToShortDateString(),
                            objItem.IntegerValue,
                            objItem.StringValue));
                    }
                }
            }
            catch (CommunicationException excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            catch (Exception excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            finally
            {
                // Client schließen
                CloseClient(objClient as IClientChannel);
            }

            // ----------------------------------------------------------------
            // Client mit Rückruf testen
            // ----------------------------------------------------------------
            // Clientobjekt deklarieren
            IwlbCallbackService objCallbackClient = null;
            // Rückrufobjekt instanziieren
            CwlbCallback objCallback = new CwlbCallback();
            objCallback.Tick += new EventHandler<CwlbServerTimeEventArgs>(Callback_Tick);

            try
            {
                // Identifikation
                Console.WriteLine("Client Factory imperativ");

                // Client erstellen
                objCallbackClient = DuplexChannelFactory<IwlbCallbackService>.CreateChannel(
                    objCallback,
                    new NetTcpBinding(),
                    new EndpointAddress("net.tcp://localhost:4711/TcpDemoCallbackService"));

                // Kommunikation durchführen
                objCallbackClient.RegisterListener();
                Thread.Sleep(10000);
                objCallbackClient.UnregisterListener();
            }
            catch (CommunicationException excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            catch (Exception excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            finally
            {
                // Client schließen
                CloseClient(objCallbackClient as IClientChannel);
            }
        }

        /// <summary>
        /// SChliesst den Client
        /// </summary>
        /// <param name="objChannel">Client der geschlossen werden soll.</param>
        private static void CloseClient(IClientChannel objChannel)
        {
            if (objChannel != null)
            {
                if (objChannel.State == CommunicationState.Opened)
                {
                    objChannel.Close();
                }
                if (objChannel.State == CommunicationState.Faulted)
                {
                    objChannel.Abort();
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung für den Rückruf vom Server.
        /// </summary>
        static void Callback_Tick(object sender, CwlbServerTimeEventArgs e)
        {
            Console.WriteLine("Serverzeit: {0}", e.ServerTime.ToLongTimeString());
        }
    }
}
