﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Threading.Tasks;

namespace WeroSoft.Samples.Client
{
    /// <summary>
    /// Testprogramm für den Client.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Einstiegspunkt in das Testprogramm
        /// </summary>
        static void Main()
        {
            // Client deklarieren
            DemoServiceClient objClient = null;

            try
            {
                // Identifikation
                Console.WriteLine("Client generiert mit SvcUtil");

                // Client erstellen
                objClient = new DemoServiceClient("TCP");

                // Daten erstellen und Kommunikation synchron durchführen
                DemoData objDemoData = new DemoData { StringValue = "Anforderung" };
                DemoData[] aobjDemoData = objClient.GetDemoData(objDemoData, 10);
                foreach (DemoData objItem in aobjDemoData)
                {
                    Console.WriteLine(string.Format("{0} {1,2}. {2}", 
                        objItem.DateTimeValue.ToShortDateString(),
                        objItem.IntegerValue,
                        objItem.StringValue));
                }

                // Kommunikation asynchron durchführen (taskorientiert)
                // Der Aufruf ist gekapselt in einer Methode, da er das
                // taskorientierte Aufrufmuster von .NET 4.5 verwendet.
                CallAsynchronus(objClient, objDemoData).Wait();
                Console.WriteLine("Demo beendet");
            }
            catch (CommunicationException excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            catch (Exception excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            finally
            {
                if (objClient != null && objClient.State == CommunicationState.Opened)
                {
                    objClient.Close();
                }
                if (objClient != null && objClient.State == CommunicationState.Faulted)
                {
                    objClient.Abort();
                }
            }
        }

        /// <summary>
        /// Ruft die Demomethode des Dienstes asynchron auf
        /// </summary>
        /// <param name="objClient">Client für die Kommunikation zum Dienst.</param>
        /// <param name="objDemoData">Vorbereitetes Datenobjekt.</param>
        /// <returns>Task für Verarbeitung.</returns>
        private static async Task CallAsynchronus(DemoServiceClient objClient, DemoData objDemoData)
        {
            Task<DemoData[]> objServerTask = objClient.GetDemoDataAsync(objDemoData, 10);
            foreach (DemoData objItem in await objServerTask)
            {
                Console.WriteLine(string.Format("{0} {1,2}. {2}",
                    objItem.DateTimeValue.ToShortDateString(),
                    objItem.IntegerValue,
                    objItem.StringValue));
            }
        }
    }
}
