﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Threading;
using System.ServiceModel.Description;

namespace WeroSoft.Samples.Client
{
    /// <summary>
    /// Testprogramm für den Client.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Einstiegspunkt in das Testprogramm
        /// </summary>
        static void Main()
        {
            // ----------------------------------------------------------------
            // Normalen Client
            // ----------------------------------------------------------------
            // Client deklarieren
            CwlbClient objClient = null;

            try
            {
                // Identifikation
                Console.WriteLine("Manuell erstellter Client");

                // Client erstellen
                //objClient = new CwlbClient("TCP");

                // Client erstellen imperativ
                ContractDescription objContractDescription = ContractDescription.GetContract(typeof(IwlbDemoService));
                ServiceEndpoint objEndpoint = new ServiceEndpoint(objContractDescription);
                objEndpoint.Binding = new NetTcpBinding();
                objEndpoint.Address = new EndpointAddress("net.tcp://localhost:4711/TcpDemoService");
                objClient = new CwlbClient(objEndpoint);
                objClient.Open();


                // Daten erstellen und Kommunikation durchführen
                CwlbDemoData objDemoData = new CwlbDemoData { StringValue = "Anforderung" };
                CwlbDemoData[] aobjDemoData = objClient.GetDemoData(objDemoData, 10);
                foreach (CwlbDemoData objItem in aobjDemoData)
                {
                    Console.WriteLine(string.Format("{0} {1,2}. {2}",
                        objItem.DateTimeValue.ToShortDateString(),
                        objItem.IntegerValue,
                        objItem.StringValue));
                }
            }
            catch (CommunicationException excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            catch (Exception excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            finally
            {
                if (objClient != null && objClient.State == CommunicationState.Opened)
                {
                    objClient.Close();
                }
                if (objClient != null && objClient.State == CommunicationState.Faulted)
                {
                    objClient.Abort();
                }
            }

            // ----------------------------------------------------------------
            // Client mit Rückruf
            // ----------------------------------------------------------------
            // Client deklarieren
            CwlbCallbackClient objCallbackClient = null;

            try
            {
                // Identifikation
                Console.WriteLine("Manuell erstellter Rückrufclient");

                // Rückrufobjekt instanziieren
                CwlbCallback objCallback = new CwlbCallback();
                objCallback.Tick += new EventHandler<CwlbServerTimeEventArgs>(Callback_Tick);

                // Client erstellen
                objCallbackClient = new CwlbCallbackClient(objCallback, "TcpCallback");
                objCallbackClient.Open();

                // Kommunikation durchführen
                objCallbackClient.RegisterListener();
                Thread.Sleep(10000);
                objCallbackClient.UnregisterListener();
            }
            catch (CommunicationException excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            catch (Exception excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            finally
            {
                // Client schließen
                if (objCallbackClient != null && objCallbackClient.State == CommunicationState.Opened)
                {
                    objCallbackClient.Close();
                }
                if (objCallbackClient != null && objCallbackClient.State == CommunicationState.Faulted)
                {
                    objCallbackClient.Abort();
                }
            }
        }


        /// <summary>
        /// Ereignisbehandlung für den Rückruf vom Server.
        /// </summary>
        static void Callback_Tick(object sender, CwlbServerTimeEventArgs e)
        {
            Console.WriteLine("Serverzeit: {0}", e.ServerTime.ToLongTimeString());
        }
    }
}
