﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.Timers;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert den Zeitserver
    /// </summary>
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
    public class CwlbCallbackService : IwlbCallbackService, IDisposable
    {
        List<IwlbCallbackContract> _cobjRegisteredClients;
        List<IwlbCallbackContract> _cobjFaultedClients;

        Timer _objTimer;

        public CwlbCallbackService()
        {
            _cobjRegisteredClients = new List<IwlbCallbackContract>();
            _cobjFaultedClients = new List<IwlbCallbackContract>();
            _objTimer = new Timer();
            _objTimer.Interval = 1000;
            _objTimer.Elapsed += new ElapsedEventHandler(Timer_Elapsed);
        }

        void Timer_Elapsed(object sender, ElapsedEventArgs e)
        {
            CwlbServerTimeEventArgs objEventArgs = new CwlbServerTimeEventArgs();
            objEventArgs.ServerTime = DateTime.UtcNow;
            _cobjFaultedClients.Clear();
            Console.WriteLine("Aktuelle Zeit: {0}", objEventArgs.ServerTime.ToLongTimeString());

            foreach (IwlbCallbackContract objCallback in _cobjRegisteredClients)
            {
                try
                {
                    objCallback.CurrentServerTime(Environment.MachineName, objEventArgs);
                }
                catch
                {
                    _cobjFaultedClients.Add(objCallback);
                }
            }

            // Nicht mehr funktionierende Clients entfernen
            _cobjFaultedClients.ForEach(objCallback => 
                {
                    Console.WriteLine("Ein Callback automatisch deregistriert.");
                    _cobjRegisteredClients.Remove(objCallback);
                });
            if (_cobjRegisteredClients.Count == 0)
            {
                _objTimer.Stop();
            }
        }

        public void RegisterListener()
        {
            IwlbCallbackContract objCallback = OperationContext.Current.GetCallbackChannel<IwlbCallbackContract>();
            _cobjRegisteredClients.Add(objCallback);
            if (_cobjRegisteredClients.Count > 0)
            {
                _objTimer.Start();
            }
        }

        public void UnregisterListener()
        {
            IwlbCallbackContract objCallback = OperationContext.Current.GetCallbackChannel<IwlbCallbackContract>();
            _cobjRegisteredClients.Remove(objCallback);
            if (_cobjRegisteredClients.Count == 0)
            {
                _objTimer.Stop();
            }
        }

        public void Dispose()
        {
            _objTimer.Stop();
            _objTimer.Dispose();
        }
    }
}
