﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Threading;

namespace WeroSoft.Samples.LearningWcfClientUI
{
    /// <summary>
    /// Definiert einen clienten mit einem Rückrufvertrag.
    /// </summary>
    /// <typeparam name="T">Type der Schnittstelle des Dienstes, die mit diesem Client benutzt wird.</typeparam>
    /// <typeparam name="TCallback">Typ des Schnittstellenvertrags, der mit dem Rückruf benutzt wird.</typeparam>
    public abstract class CwlbCallbackClientBase<T, TCallback> : CwlbClientBase<T> 
        where T: class 
        where TCallback: IwlbCallbackBase, new()
    {
        /// <summary>
        /// Liefert das Rückrufobjekt.
        /// </summary>
        public TCallback Callback { get; private set; }

        /// <summary>
        /// Initialisiert ein neues Objekt des Typs CwlbCallbackClientBase.
        /// </summary>
        /// <param name="objBinding">Bindung, die der Client verwendet.</param>
        /// <param name="objEndpointAddress">Endpunktadresse, die der Client verwendet.</param>
        /// <param name="objDispatcher">Dispatcher, der bei einer Integration in einen WPF-Client verwendet wird.</param>
        public CwlbCallbackClientBase(Binding objBinding, EndpointAddress objEndpointAddress, Dispatcher objDispatcher)
            : base(objBinding, objEndpointAddress, objDispatcher)
        {
            Callback = new TCallback();
            Callback.ClientUIDispatcher = ClientUIDispatcher;
        }

        /// <summary>
        /// Erstell das eigentliche Proxyobjekt.
        /// </summary>
        /// <returns>Liefert das Proxyobjekt.</returns>
        protected override T CreateProxy()
        {
            return DuplexChannelFactory<T>.CreateChannel(Callback, Binding, EndpointAddress);
        }
    }
}
