﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel.Channels;
using System.Text;
using System.Threading.Tasks;
using WeroSoft.Samples.LearningWcfContracts;

namespace WeroSoft.Samples.LearningWcfClientUI
{
    /// <summary>
    /// Definiert einen funktionellen Client für den Demodienst mit Rückruf.
    /// </summary>
    public class CwlbDemoServiceClient : CwlbCallbackClientBase<IwlbDemoService, CwlbDemoServiceCallback>
    {
        /// <summary>
        /// Initialisiert ein neues Objekt des Typs CwlbCallbackClientBase.
        /// </summary>
        /// <param name="objBinding">Bindung, die der Client verwendet.</param>
        /// <param name="objEndpointAddress">Endpunktadresse, die der Client verwendet.</param>
        /// <param name="objDispatcher">Dispatcher, der bei einer Integration in einen WPF-Client verwendet wird.</param>
        public CwlbDemoServiceClient(Binding objBinding, System.ServiceModel.EndpointAddress objEndpointAddress, System.Windows.Threading.Dispatcher objDispatcher)
            : base(objBinding, objEndpointAddress, objDispatcher)
        {
            Reconnect();
        }

        /// <summary>
        /// Ruft die Metode Ping synchron auf.
        /// </summary>
        /// <returns>Rüc kgabedaten vom Server.</returns>
        public CwlbPing Ping()
        {
            CheckChannel();
            CwlbPing objPing = new CwlbPing();
            objPing.ClientName = Environment.MachineName;
            objPing.ClientSent = DateTime.UtcNow;
            objPing.Load = 1024;
            return ExecuteFunc<CwlbPing, CwlbPing>(Proxy.Ping, objPing);
        }

        /// <summary>
        /// Sendet einen Text zum Server.
        /// </summary>
        /// <param name="strGreetingText">Text, der an den Server gesendet wird.</param>
        public void Greeting(string strGreetingText)
        {
            CheckChannel();
            ExecuteAction<string>(Proxy.Greeting, strGreetingText);
        }

        /// <summary>
        /// Ruft eine lange dauernde Methode auf dem Server clientseitig asynchron auf.
        /// </summary>
        /// <param name="objCallback">Callbackmethode für die Entgegennahme des Resultats</param>
        /// <param name="iDuration">Gewünschte Laufzeit der Methode in Sekunden.</param>
        /// <param name="iAmount">Anzahl der erzeugten Integerwerte.</param>
        /// <param name="iMinValue">Kleinster numerischer Wert der Rückgabe.</param>
        /// <param name="iMaxValue">Grösster numerischer Wert der Rückgabe.</param>
        public void BeginLongTermAction(Action<int[]> objCallback, int iDuration, int iAmount, int iMinValue, int iMaxValue)
        {
            CheckChannel();
            BeginExecutionAction<int, int, int, int, int[]>(LongTermAction, objCallback, iDuration, iAmount, iMinValue, iMaxValue);
        }

        /// <summary>
        /// Ruft eine lange dauernde Methode synchron auf.
        /// </summary>
        /// <param name="iDuration">Gewünschte Laufzeit der Methode in Sekunden.</param>
        /// <param name="iAmount">Anzahl der erzeugten Integerwerte.</param>
        /// <param name="iMinValue">Kleinster numerischer Wert der Rückgabe.</param>
        /// <param name="iMaxValue">Grösster numerischer Wert der Rückgabe.</param>
        /// <returns>Resultatdaten des Servers.</returns>
        public int[] LongTermAction(int iDuration, int iAmount, int iMinValue, int iMaxValue)
        {
            CheckChannel();
            return ExecuteFunc<int, int, int, int, int[]>(Proxy.LongTermAction, iDuration, iAmount, iMinValue, iMaxValue);
        }

        /// <summary>
        /// Registriert den Client für Rückrufe vom Server.
        /// </summary>
        public void Register()
        {
            CheckChannel();
            ExecuteAction(Proxy.Register);
        }

        /// <summary>
        /// Hebt die Registrierung für Rückrufe auf.
        /// </summary>
        public void Unregister()
        {
            CheckChannel();
            ExecuteAction(Proxy.Unregister);
        }

        /// <summary>
        /// Ruft die serverseitig asynchrone Verarbeitung auf.
        /// </summary>
        /// <param name="objData">Datein der Verarbeitung.</param>
        /// <param name="objCallback">Clientseitige Rückrufmethode.</param>
        public IAsyncResult BeginLongTermActionFull(CwlbAsyncData objData, AsyncCallback objCallback)
        {
            CheckChannel();
            return ExecuteFunc<CwlbAsyncData, AsyncCallback, object, IAsyncResult>(Proxy.BeginLongTermActionFull, objData, objCallback, Proxy);
        }
    }
}
