﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using WeroSoft.Samples.LearningWcfContracts;

namespace WeroSoft.Samples.LearningWcfClientUI
{
    /// <summary>
    /// Interaktionslogik für das Dialogfeld JobDialog.xaml
    /// </summary>
    public partial class JobDialog : Window
    {
        /// <summary>
        /// Liefert oder definiert die Jobs auf der Serverseite.
        /// </summary>
        public ObservableCollection<CwlbJob> JobCollection { get; private set; }

        /// <summary>
        /// Liefert oder definiert die Clientfactory für die Herstellung des Clients.
        /// </summary>
        public ChannelFactory<IwlbDemoWorkerService> ClientFactory { get; set; }

        /// <summary>
        /// Initialisiert ein neues Dialogfeld.
        /// </summary>
        public JobDialog()
        {
            ClientFactory = new ChannelFactory<IwlbDemoWorkerService>(new NetTcpBinding(), new EndpointAddress("net.tcp://localhost:4711/WorkerService"));
            JobCollection = new ObservableCollection<CwlbJob>();
            InitializeComponent();
            Refresh();
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Aktualisieren.
        /// </summary>
        private void Refresh_Clicked(object sender, RoutedEventArgs e)
        {
            Refresh();
        }

        /// <summary>
        /// Aktualisiert den Inhalt der Liste.
        /// </summary>
        private void Refresh()
        {
            IwlbDemoWorkerService objClient = null;
            JobCollection.Clear();
            try
            {
                objClient = ClientFactory.CreateChannel();
                foreach (CwlbJob objJob in objClient.GetAllJobs())
                {
                    JobCollection.Add(objJob);
                }
            }
            finally
            {
                CloseClient(objClient as IClientChannel);
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Neu.
        /// </summary>
        private void New_Clicked(object sender, RoutedEventArgs e)
        {
            IwlbDemoWorkerService objClient = null;
            try
            {
                objClient = ClientFactory.CreateChannel();
                CwlbJob objJob = new CwlbJob();
                objJob.Id = Guid.NewGuid();
                objJob.Name = "Demojob " + DateTime.Now.Ticks.ToString();
                objJob.Workload = new Random().Next(0, 130);
                objClient.RegisterJob(objJob);
            }
            finally
            {
                CloseClient(objClient as IClientChannel);
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Löschen.
        /// </summary>
        private void Delete_Clicked(object sender, RoutedEventArgs e)
        {
            // Prüfen ob ein Element selektiert ist
            if (_lvJob.SelectedItem != null)
            {
                IwlbDemoWorkerService objClient = null;
                try
                {
                    objClient = ClientFactory.CreateChannel();
                    CwlbJob objJob = _lvJob.SelectedItem as CwlbJob;
                    if (!objClient.DeleteJob(objJob.Id))
                    {
                        MessageBox.Show("Job kann nicht gelöscht werden.");
                    }
                    else
                    {
                        Refresh();
                    }
                }
                finally
                {
                    CloseClient(objClient as IClientChannel);
                }

            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Betätigen der Schaltfläche Beenden.
        /// </summary>
        private void Exit_Clicked(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Closes a client used by this program.
        /// </summary>
        /// <param name="objClientChannel"></param>
        private void CloseClient(IClientChannel objClientChannel)
        {
            if (objClientChannel == null)
            {
                return;
            }
            else if (objClientChannel.State == CommunicationState.Opened)
            {
                objClientChannel.Close();
            }
            else if (objClientChannel.State == CommunicationState.Faulted)
            {
                objClientChannel.Abort();
            }
        }
    }
}
