﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using WeroSoft.Samples.LearningWcfContracts;

namespace WeroSoft.Samples.LearningWcfClientUI
{
    /// <summary>
    /// Interaktionslogik für MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        /// <summary>
        /// Hält den Clienten zum Serversystem.
        /// </summary>
        private CwlbDemoServiceClient _objClient;

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse MainWindow.
        /// </summary>
        public MainWindow()
        {
            // Einschalten der allgemeinen Ausnahmebehandlung
            App.Current.DispatcherUnhandledException += new System.Windows.Threading.DispatcherUnhandledExceptionEventHandler(DispatcherUnhandledException);

            // Generierte Komponenten initialisieren
            InitializeComponent();

            // Erstellen des Client
            _objClient = new CwlbDemoServiceClient(new NetTcpBinding(), new EndpointAddress("net.tcp://localhost:4711/DemoServicePerSession"), this.Dispatcher);
            _objClient.Callback.Time += new Action<DateTime>(Callback_Time);
        }

        /// <summary>
        /// Ereignisbehandlung für die allgemeine Ausnahmebehandlung.
        /// </summary>
        private void DispatcherUnhandledException(object sender, System.Windows.Threading.DispatcherUnhandledExceptionEventArgs e)
        {
            if (e.Exception.InnerException != null)
            {
                MessageBox.Show(this, e.Exception.InnerException.Message, "Fehler");
            }
            else if (e.Exception is ApplicationException)
            {
                MessageBox.Show(this, e.Exception.Message, "Fehler");
            }
            else
            {
                MessageBox.Show(this, "Oops da ist was faul.", "Fehler");
            }
            e.Handled = true;
        }

        /// <summary>
        /// Erignisbehandlung für das Auswählen der Schaltfläche Ping.
        /// </summary>
        private void Ping_Clicked(object sender, RoutedEventArgs e)
        {
            CwlbPing objPing = _objClient.Ping();
            WriteToList("Ping von '{0}' nach '{1}' benötigte '{2:N6}' Millisekunden", objPing.ClientName, objPing.ServerName, (objPing.ServerSent - objPing.ClientSent).TotalMilliseconds);
        }

        /// <summary>
        /// Erignisbehandlung für das Auswählen der Schaltfläche Grüsse.
        /// </summary>
        private void Greeting_Clicked(object sender, RoutedEventArgs e)
        {
            _objClient.Greeting(string.Format("Hello von '{0}'", Environment.MachineName));
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LongTerm_Clicked(object sender, RoutedEventArgs e)
        {
            MessageBoxResult objResult = MessageBox.Show(this, "Diese Demo friert das Fenster für 10 Sekunden ein. Wollen Sie weiterfahren?", "Demo", MessageBoxButton.YesNo, MessageBoxImage.Question);
            if (objResult == MessageBoxResult.Yes)
            {
                 _objClient.LongTermAction(10, 20, 0, 100);
                WriteToList("Clientseitiger asynchroner Aufruf gestartet ...");
            }
        }

        /// <summary>
        /// Erignisbehandlung für das Auswählen der Schaltfläche Clientseitig asynchron.
        /// </summary>
        private void Asynchronous_Clicked(object sender, RoutedEventArgs e)
        {
            _objClient.BeginLongTermAction(AsynchronousResult, 10, 20, 0, 100);
            WriteToList("Clientseitiger asynchroner Aufruf gestartet ...");
        }

        /// <summary>
        /// Erignisbehandlung beim Erhalt des Resultats des asynchronen Aufrufs.
        /// </summary>
        private void AsynchronousResult(int[] aiResult)
        {
            string strOut = "Asynchroner Aufruf beendet. gelieferte Werte sind: ";
            foreach (int iResult in aiResult)
            {
                strOut += string.Format("-{0}", iResult);
            }
            WriteToList(strOut);
        }

        /// <summary>
        /// Erignisbehandlung für das Auswählen der Schaltfläche Serverseitig asynchron.
        /// </summary>
        private void ServersideAsynchronous_Clicked(object sender, RoutedEventArgs e)
        {
            CwlbAsyncData objData = new CwlbAsyncData();
            objData.Duration = new Random().Next(5, 20);
            objData.MinimalValue = 0;
            objData.MaximalValue = 100;
            objData.Load = 10;
            _objClient.BeginLongTermActionFull(objData, ServerAsyncFullEnd);
            WriteToList("Serverseitiger asynchroner Aufruf gestartet ...");
        }

        /// <summary>
        /// Eventhandler der serverseitigen Verarbeitung.
        /// </summary>
        /// <param name="objAsyncResult">Resultat der Verarbeitung.</param>
        private void ServerAsyncFullEnd(IAsyncResult objAsyncResult)
        {
                // Check changing the thread
            if (!Dispatcher.CheckAccess())
            {
                Action<IAsyncResult> objAction = ServerAsyncFullEnd;
                Dispatcher.BeginInvoke(objAction, objAsyncResult);
            }
            else
            {
                CwlbAsyncData objData = _objClient.Proxy.EndLongTermActionFull(objAsyncResult);
                WriteToList("Die serverseitige Verarbeitung dauerte: {0} Sekunden", objData.Duration);
            }
        }

        /// <summary>
        /// Erignisbehandlung für das Auswählen der Schaltfläche Registrieren.
        /// </summary>
        private void Register_Clicked(object sender, RoutedEventArgs e)
        {
            _objClient.Register();
            WriteToList("Client hat sich das Zeitsignal registriert");
        }

        /// <summary>
        /// Erignisbehandlung für das Auswählen der Schaltfläche Deregistrieren.
        /// </summary>
        private void Unregister_Clicked(object sender, RoutedEventArgs e)
        {
            _objClient.Unregister();
            WriteToList("Client hat die Registrierung für das Zeitsignal aufgehoben");
        }

        /// <summary>
        /// Erignisbehandlung für das Auswählen der Schaltfläche Liste löschen.
        /// </summary>
        private void ClearList_Clicked(object sender, RoutedEventArgs e)
        {
            _lbMessages.Items.Clear();
        }

        /// <summary>
        /// Erignisbehandlung für das Auswählen der Schaltfläche Beenden.
        /// </summary>
        private void Exit_Clicked(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Ereignisbehandlung beim Erhalt des Zeitsignals des Servers.
        /// </summary>
        /// <param name="time">Serverseitige Zeit.</param>
        private void Callback_Time(DateTime time)
        {
            WriteToList("Zeitsignal vom Server: {0}", time.ToLongTimeString());
        }

        /// <summary>
        /// Formatiert einen Text und leitet diesen in die angezeigte Liste.
        /// </summary>
        /// <param name="strFormat">Definition der Formatierung.</param>
        /// <param name="aobjParams">Daten für das Einfügen in das Fromat.</param>
        private void WriteToList(string strFormat, params object[] aobjParams)
        {
            WriteToList(string.Format(strFormat, aobjParams));
        }

        /// <summary>
        /// Schreibt einen Text mit einem Zeitstempel in angezeigte Liste.
        /// </summary>
        /// <param name="strContent">Inhalt der in die Liste geschreiben wird.</param>
        private void WriteToList(string strContent)
        {
            _lbMessages.Items.Insert(0, string.Format("{0} - {1}", DateTime.Now.ToString("HH:mm:ss.fff"), strContent));
        }

        /// <summary>
        /// Started das modale Dialogfeld für die unabhängige serverseitige asynchrone Verarbeitung.
        /// </summary>
        private void ServersideWorker_Clicked(object sender, RoutedEventArgs e)
        {
            JobDialog objDialog = new JobDialog();
            objDialog.Owner = this;
            objDialog.ShowDialog();
        }

        /// <summary>
        /// Startet das modale Dialogfeld für dei Demo der serverseitigen Verarbeitung mit einem Dispatcher.
        /// </summary>
        private void Dispatcher_Clicked(object sender, RoutedEventArgs e)
        {
            DispatcherDialog objDialog = new DispatcherDialog();
            objDialog.Owner = this;
            objDialog.Show();
        }
    }
}
