﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace WeroSoft.Samples.LearningWcfContracts
{
    /// <summary>
    /// Daten eines Jobs einer unabhängigen serverseitigen Verarbeitung.
    /// </summary>
    public class CwlbDispatcherJob : INotifyPropertyChanged
    {
        /// <summary>
        /// Liefert oder definiert die Identifikation des Jobs.
        /// </summary>
        public Guid Id { get; set; }

        /// <summary>
        /// Liefert oder definiert den Namen des Jobs.
        /// </summary>
        public string Name { get; set; }

        /// <summary>
        /// Liefert oder definiert den Absender des Jobs.
        /// </summary>
        public Guid Sender { get; set; }

        /// <summary>
        /// Liefert oder definiert den Workload, der dieser Job verurschat.
        /// </summary>
        public int Workload { get; set; }

        /// <summary>
        /// Liefert oder definiert den Startzeitpunkt des Jobs auf dem Server.
        /// </summary>
        public DateTime SentFromClient { get; set; }

        /// <summary>
        /// Liefert oder definiert den Endzeitpunkt des Jobs.
        /// </summary>
        public DateTime ReceivedCallback { get; set; }

        /// <summary>
        /// Liefert oder definiert den Status des dispatcher Jobs.
        /// </summary>
        public EwlbDispatcherJobState Status { get; set; }

        /// <summary>
        /// Wird ausgelöst wenn ein Datenelement verändert wird.
        /// </summary>
        /// <remarks>
        /// Dieses Ereignis wird hier nur verwendet, für das Datum und den Status des Rückrufs.
        /// </remarks>
        public event PropertyChangedEventHandler PropertyChanged;

        /// <summary>
        /// Definiert den Zeitpunkt des Rückrufs sowie den Status und löst das Ereignis PropertyChanged aus.
        /// </summary>
        public void SetCallbackTime()
        {
            ReceivedCallback = DateTime.Now;
            Status = EwlbDispatcherJobState.ReceivedCallback;

            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs("ReceivedCallback"));
                PropertyChanged(this, new PropertyChangedEventArgs("Status"));
            }
        }
    }
}
