﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;

namespace WeroSoft.Samples.LearningWcfContracts
{
    /// <summary>
    /// Definiert die Schnittstelle für den Demo Service.
    /// </summary>
    [ServiceContract(Name="DemoService", Namespace=GwlbGlobals.WcfNamespace, CallbackContract=typeof(IwlbDemoServiceCallback))]
    public interface IwlbDemoService : IDisposable
    {
        /// <summary>
        /// Einfacher Aufruf mit einer Datenübergabe und einer Datenrückgabe.
        /// </summary>
        /// <param name="objPing">Vom Client geliefertes Datenobjekt.</param>
        /// <returns>Modifiziertes Datenobjekt vom Server.</returns>
        [OperationContract(Name="Ping")]
        CwlbPing Ping(CwlbPing objPing);

        /// <summary>
        /// Einseitiger Aufruf des Cleints mit einer Textübergabe.
        /// </summary>
        /// <param name="strGreetingText"></param>
        [OperationContract(Name = "Greeting", IsOneWay=true)]
        void Greeting(string strGreetingText);

        /// <summary>
        /// Methode für dei Simulation eines synchronen, lange andauernden Aufrufs einer Methode.
        /// </summary>
        /// <param name="iDuration">Gewünschte Laufzeit der Methode in Sekunden.</param>
        /// <param name="iAmount">Anzahl der erzeugten Integerwerte.</param>
        /// <param name="iMinValue">Kleinster numerischer Wert der Rückgabe.</param>
        /// <param name="iMaxValue">Grösster numerischer Wert der Rückgabe.</param>
        /// <returns>Array von ganzzahligen Werten der Rückgabe.</returns>
        [OperationContract(Name = "LongTermAction")]
        int[] LongTermAction(int iDuration, int iAmount, int iMinValue, int iMaxValue);

        /// <summary>
        /// Registriert einen Client für Rückrufe.
        /// </summary>
        [OperationContract(Name = "Register", IsOneWay=true)]
        void Register();

        /// <summary>
        /// Hebt die Registrierung eines Clients für Rückrufe auf.
        /// </summary>
        [OperationContract(Name = "Unregister", IsOneWay = true)]
        void Unregister();

        #region Serverseitige asynchrone Verarbeitung
        /// <summary>
        /// Serverseitig asynchrone Demomethode.
        /// </summary>
        /// <param name="objData">Datenobjekt das vom Client gesendet wird.</param>
        /// <param name="objAsyncCallback">Rückrufmethode für die Verarbeitung.</param>
        /// <param name="objAsyncState">Statusdaten der Verarbeitung.</param>
        /// <returns>Array von ganzzahligen Werten der Rückgabe.</returns>
        [OperationContract(Name = "LongTermActionFull", AsyncPattern = true)]
        [FaultContract(typeof(string))]
        IAsyncResult BeginLongTermActionFull(CwlbAsyncData objData, AsyncCallback objAsyncCallback, object objAsyncState);

        /// <summary>
        /// Methode für die Beendigung der serverseitigen asynchronen Verarbeitung.
        /// </summary>
        /// <param name="objResult">Resultat der Verarbeitung.</param>
        /// <returns>Ausgabedaten der Verarbeitung.</returns>
        CwlbAsyncData EndLongTermActionFull(IAsyncResult objResult);
        #endregion
    }
}
