﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace WeroSoft.Samples.LearningWcfService
{
    /// <summary>
    /// Repräsentiert das asynchrone Resultat der serverseitigen Verarbeitung.
    /// </summary>
    internal class CwlbAsyncResult<T> : IAsyncResult
    {
        // Instanzdaten
        private System.Threading.AutoResetEvent WaitHandle { get; set; }
        private object State { get; set; }

        /// <summary>
        /// Initialisiert eine Instanz der Klasse CwlbAsyncResult<T> .
        /// </summary>
        /// <param name="bSynchronousExecution">True, wenn die Verarbeitung synchron geschieht.</param>
        /// <param name="objState">WCF Datenobjekt.</param>
        public CwlbAsyncResult(bool bSynchronousExecution, object objState)
        {
            CompletedSynchronously = bSynchronousExecution;
            State = objState;
        }

        /// <summary>
        /// Liefert oder definiert die Daten des Resultats.
        /// </summary>
        public T Data { get; set; }

        /// <summary>
        /// Liefert oder definiert die Ausnahme der asynchronen Verarbeitung.
        /// </summary>
        public Exception WorkerException { get; set; }

        /// <summary>
        /// Liefert oder definiert einen Wert, der den Abbruch der Verarbeitung definiert.
        /// </summary>
        public bool CancellationPending { get; set; }

        #region IAsyncResult Members
        /// <summary>
        /// Liefert das Statusobjekt der Verarbeitung.
        /// </summary>
        public object AsyncState
        {
            get
            {
                return State;
            }
        }

        /// <summary>
        /// Liefert den Wait Handle für die Synchronisierung des Arbeitsthreads.
        /// </summary>
        public WaitHandle AsyncWaitHandle
        {
            get
            {
                if (WaitHandle == null)
                {
                    WaitHandle = new System.Threading.AutoResetEvent(false);
                }

                return WaitHandle;
            }
        }

        /// <summary>
        /// Liefert true, wenn die Verarbeirtung 
        /// Gets a value indicating that the work has been finished synchronously
        /// </summary>
        public bool CompletedSynchronously {get; private set; }

        /// <summary>
        /// Liefert oder definiert true, wenn die Verarbeitung beendet wurde.
        /// </summary>
        public bool IsCompleted { get; set; }
        #endregion

        /// <summary>
        /// Synchronisiere die Beendigung der Verarbeitung.
        /// </summary>
        public void Complete()
        {
            IsCompleted = true;
            if (WaitHandle == null)
            {
                WaitHandle = new System.Threading.AutoResetEvent(true);
            }
            else
            {
                WaitHandle.Set();
            }
        }
    }
}
