﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using WeroSoft.Samples.LearningWcfContracts;

namespace WeroSoft.Samples.LearningWcfService
{
    /// <summary>
    /// Implementiert eine Jobverarbeitung.
    /// </summary>
    [ServiceBehavior(InstanceContextMode=InstanceContextMode.Single)]
    public class CwlbDemoDispatcherService : IwlbDemoDispatcherService
    {
        /// <summary>
        /// Definiert das Synchronisationsobjekt für die Handhabung des Dienstes.
        /// </summary>
        private readonly object _objSync = new object();

        /// <summary>
        /// Hält eine Instanz des Jobkontrollers für die Verarbeitung.
        /// </summary>
        private CwlbJobDispatcher _objDispatcher;

        /// <summary>
        /// Definiert das Verzewichnis für die Kontrolle der Rückrufe pro Client.
        /// </summary>
        private Dictionary<Guid, IwlbDemoDispatcherCallback> _cobjCallbackDictionary = new Dictionary<Guid, IwlbDemoDispatcherCallback>();

        /// <summary>
        /// Initialisiert die neue Serviceinstanz.
        /// </summary>
        public CwlbDemoDispatcherService()
        {
            _objDispatcher = new CwlbJobDispatcher();
        }

        /// <summary>
        /// Registriert den Client beim Server für die Rückrufe.
        /// </summary>
        /// <param name="objSenderId">Identifikation des Senders für den Rückruf.</param>
        public void Register(Guid objSenderId)
        {
            // Kontext abfragen und Registration prüfen
            IwlbDemoDispatcherCallback objCallback = OperationContext.Current.GetCallbackChannel<IwlbDemoDispatcherCallback>();
            lock (_objSync)
            {
                // Prüfen ob bereits ein Eintrag für den Client vorhanden ist
                if (_cobjCallbackDictionary.ContainsKey(objSenderId))
                {
                    // Ersetzen des Rückrufkanals
                    _cobjCallbackDictionary[objSenderId] = objCallback;
                }
                else
                {
                    // Neu registrieren des Rückrufkanals
                    _cobjCallbackDictionary.Add(objSenderId, objCallback);
                }
            }
        }

        /// <summary>
        /// Führt einen Job im Dispatcher aus.
        /// </summary>
        /// <param name="objDispatcherJob">Job der für die Ausführung vorgesehen ist.</param>
        public void Execute(CwlbDispatcherJob objDispatcherJob)
        {
            // Auftrag in die Warteschlangen einreihen
            _objDispatcher.Add( new CwlbDispatcherJobWorker(objDispatcherJob, this));
        }

        /// <summary>
        /// Hebt die Registrierung des Client beim Server für die Rückrufe auf.
        /// </summary>
        /// <param name="objSenderId">Identifikation des Senders.</param>
        public void Unregister(Guid objSenderId)
        {
            lock (_objSync)
            {
                // Prüfen ob bereits ein Eintrag für den Client vorhanden ist
                if (_cobjCallbackDictionary.ContainsKey(objSenderId))
                {
                    // Löschen des Rückrufkanals
                    _cobjCallbackDictionary.Remove(objSenderId);
                }
            }
        }

        /// <summary>
        /// Führt den Rückruf zum Client aus.
        /// </summary>
        /// <param name="objCurrentJob">Job der erledigt ist.</param>
        internal void OnFinished(CwlbDispatcherJob objCurrentJob)
        {
            lock (_objSync)
            {
                // Prüfen ob bereits ein Eintrag für den Client vorhanden ist
                if (_cobjCallbackDictionary.ContainsKey(objCurrentJob.Sender))
                {
                    // Löschen des Rückrufkanals
                    _cobjCallbackDictionary[objCurrentJob.Sender].OnFinished(objCurrentJob.Id);
                }
            }
        }
    }
}
