﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using WeroSoft.Samples.LearningWcfContracts;

namespace WeroSoft.Samples.LearningWcfService
{
    /// <summary>
    /// Implementiert einen PerSession-Dienst multithreaded.
    /// </summary>
    [ServiceBehavior(InstanceContextMode=InstanceContextMode.PerSession, ConcurrencyMode=ConcurrencyMode.Multiple)]
    public class CwlbDemoServicePerSession : CwlbDemoService, IwlbDemoService, IDisposable
    {
        /// <summary>
        /// Hält eine Referenz zum Callback objekt der Sitzung.
        /// </summary>
        private IwlbDemoServiceCallback objCallback;

        /// <summary>
        /// Hält eine Referenz zum intern verwendeten Timer.
        /// </summary>
        private System.Threading.Timer objTimer;

        /// <summary>
        /// Hält die Identifikation für die Sitzung.
        /// </summary>
        private string _strSessionId;

        /// <summary>
        /// Initialisiert eine neues Sitzungsobjekt des Dienstes.
        /// </summary>
        public CwlbDemoServicePerSession()
        {
            _strSessionId = OperationContext.Current.SessionId;
            WriteLine(string.Format("Sitzungsobjekt für Sitzung '{0}' erstellt", _strSessionId));
        }

        /// <summary>
        /// Gibt die Ressourcen eines Sitzungsobjektes frei.
        /// </summary>
        public override void Dispose()
        {
            Unregister();
            WriteLine(string.Format("Sitzungsobjekt für Sitzung '{0}' freigegeben", _strSessionId));
        }

        /// <summary>
        /// Registriert den Client für Rückrufe.
        /// </summary>
        public override void Register()
        {
            lock (this)
            {
                if (objCallback == null)
                {
                    WriteLine("Registrierung vorgenommen");
                    objCallback = OperationContext.Current.GetCallbackChannel<IwlbDemoServiceCallback>();
                    objTimer = new System.Threading.Timer(OnTime, null, 0, 1000);
                }
            }
        }

        /// <summary>
        /// Hebt die Regsitration von Rückrufen des Clients auf.
        /// </summary>
        public override void Unregister()
        {
            lock (this)
            {
                if (objCallback != null)
                {
                    WriteLine("Registrierung aufgehoben");
                    objTimer.Dispose();
                    objCallback = null;
                }
            }
        }

        /// <summary>
        /// Führt den Rückruf zum Client aus.
        /// </summary>
        /// <param name="state">Parameter (wird nicht verwendet).</param>
        private void OnTime(object state)
        {
            lock (this)
            {
                if (objCallback != null)
                {
                    WriteLine("Sende Zeitsignal");
                    try
                    {
                        objCallback.OnTime(DateTime.UtcNow);
                    }
                    catch
                    {
                        Unregister();
                    }
                }
            }
        }
    }
}
