﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using WeroSoft.Samples.LearningWcfContracts;

namespace WeroSoft.Samples.LearningWcfService
{
    /// <summary>
    /// Implementiert eine Jobverarbeitung.
    /// </summary>
    [ServiceBehavior(InstanceContextMode=InstanceContextMode.Single)]
    public class CwlbDemoWorkerService : IwlbDemoWorkerService
    {
        /// <summary>
        /// Hält eine Instanz des Jobkontrollers für die Verarbeitung.
        /// </summary>
        private CwlbJobController _objJobController = new CwlbJobController();

        /// <summary>
        /// Liefert die Liste der aktuellen Jobs im System.
        /// </summary>
        /// <returns>Liste der Jobs.</returns>
        public List<CwlbJob> GetAllJobs()
        {
            return _objJobController.GetAllJobs();
        }

        /// <summary>
        /// Registriert einen neuen Job im System.
        /// </summary>
        /// <param name="objJob">Daten des neuen Jobs.</param>
        public void RegisterJob(CwlbJob objJob)
        {
            _objJobController.RegisterJob(objJob);
        }

        /// <summary>
        /// Löscht einen bestehenden Job aus dem System.
        /// </summary>
        /// <param name="objJobId">Identifikation des zu löschenden Jobs.</param>
        /// <returns>True, wenn der Job gelöscht wurde, anderfalls false.</returns>
        public bool DeleteJob(Guid objJobId)
        {
            return _objJobController.DeleteJob(objJobId);
        }

        /// <summary>
        /// Beendet den Dienst
        /// </summary>
        public void Dispose()
        {
            _objJobController.Dispose();
        }
    }
}
