﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using WeroSoft.Samples.LearningWcfContracts;

namespace WeroSoft.Samples.LearningWcfService
{
    /// <summary>
    /// Implementiert ein Arbeitsobjekt für einen Job.
    /// </summary>
    public class CwlbJobWorker
    {
        /// <summary>
        /// Liefert oder definiert die Jobdaten.
        /// </summary>
        public CwlbJob Job { get; set; }

        /// <summary>
        /// Initialisiert ein neues Job Worker Objekt.
        /// </summary>
        /// <param name="objJob"></param>
        public CwlbJobWorker(CwlbJob objJob)
        {
            Job = objJob;
        }

        /// <summary>
        /// Started die Verarbeitung des Jobs.
        /// </summary>
        internal void Start()
        {
            // Status auf laufend wechseln
            Job.Status = EwlbJobState.Running;
            ThreadPool.QueueUserWorkItem(DoWork);
        }

        /// <summary>
        /// Arbeitsmethode des Threads.
        /// </summary>
        /// <param name="objStatus">Statusobjekt (nicht benutzt).</param>
        private void DoWork(object objStatus)
        {
            bool bRun = true;
            try
            {
                // Startzeit definieren
                lock (CwlbJobController.Synchronize)
                {
                    // Job Status auf beendet wechseln
                    Job.Started = DateTime.Now;
                    if (Job.Workload < 10 || Job.Workload > 120)
                    {
                        throw new Exception();
                    }
                }

                // Job OK, verarbeiten
                while (bRun)
                {
                    // Arbeit simulieren
                    Thread.SpinWait(10000000);
                    Thread.Sleep(200);

                    // Prüfen ob der Job beendet werden soll.
                    lock (CwlbJobController.Synchronize)
                    {
                        // Dekrementieren des Workloads
                        Job.Workload--;
                        if (Job.Workload <= 0)
                        {
                            bRun = false;
                        }

                        // Abbruch prüfen
                        if (Job.Status == EwlbJobState.Cancelling)
                        {
                            bRun = false;
                        }
                    }
                }

                // Endzeit definieren
                lock (CwlbJobController.Synchronize)
                {
                    // Job Status auf beendet drehen
                    if (Job.Status == EwlbJobState.Cancelling)
                    {
                        Job.Status = EwlbJobState.Cancelled;
                    }
                    else
                    {
                        Job.Status = EwlbJobState.Ended;
                    }
                }
            }
            catch
            {
                lock (CwlbJobController.Synchronize)
                {
                    // Job Status auf Ausnahme wechseln
                    Job.Status = EwlbJobState.Exception;
                }
            }
            finally
            {
                // Endzeit definieren
                lock (CwlbJobController.Synchronize)
                {
                    Job.Ended = DateTime.Now;
                }
            }
        }
    }
}
