﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;
using System.Transactions;
using WeroSoft.Samples.LearningWcfContracts;

namespace WeroSoft.Samples.LearningWcfClient
{
    /// <summary>
    /// Clientseitige Nutzung des transaktionellen Verhaltens.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Einstiegspunkt in das Demoprogramm.
        /// </summary>
        static void Main()
        {
            // Client erstellen
            Client.AccountingServiceClient objProxy = new Client.AccountingServiceClient();

            // ----------------------------------------------------------------
            // Vorbereitungen
            // ----------------------------------------------------------------
            // Daten zurückstellen
            objProxy.Reset();
            // Person abfragen
            Client.CwlbPerson[] aobjPerson = objProxy.EnumeratePersons();
            // Konten abfragen
            Client.CwlbAccount[] aobjAccount = objProxy.EnumerateAccounts(aobjPerson.Single(objPerson => objPerson.Name == "Isamillionaire").PersonId);

            // ----------------------------------------------------------------
            // Serverseitige Transaktion nutzen
            // ----------------------------------------------------------------
            Client.CwlbMoneyTransaction objMoneyTransaction = null;
            // ohne Fehler
            try
            {
                objMoneyTransaction = new Client.CwlbMoneyTransaction();
                objMoneyTransaction.AccountIdCredit = aobjAccount[0].AccountId;
                objMoneyTransaction.AccountIdDebit = aobjAccount[1].AccountId;
                objMoneyTransaction.Amount = 125000.00;
                objMoneyTransaction.ValidityDate = DateTime.Now.AddDays(1).Date;
                objProxy.Accounting(objMoneyTransaction);
            }
            catch (FaultException excObject)
            {
                Console.WriteLine(excObject.Message);
            }

            // mit Fehler
            try
            {
                objMoneyTransaction = new Client.CwlbMoneyTransaction();
                objMoneyTransaction.AccountIdCredit = aobjAccount[0].AccountId;
                objMoneyTransaction.AccountIdDebit = aobjAccount[1].AccountId;
                objMoneyTransaction.Amount = 10000000.00;
                objMoneyTransaction.ValidityDate = DateTime.Now.AddDays(1).Date;
                objProxy.Accounting(objMoneyTransaction);
            }
            catch (FaultException excObject)
            {
                Console.WriteLine(excObject.Message);
            }

            // ----------------------------------------------------------------
            // Clientseitige Transaktion nutzen
            // ----------------------------------------------------------------
            // ohne Fehler

            // Stellen Sie die Transaktionsoption IsolationLevel gleich ein wie das
            // auf der Serverseite auch der Fall ist
            TransactionOptions objTransactionOption = new TransactionOptions();
            objTransactionOption.IsolationLevel = IsolationLevel.ReadCommitted;
            objTransactionOption.Timeout = TimeSpan.FromSeconds(30);

            try
            {
                using (TransactionScope objTransaction = 
                    new TransactionScope(TransactionScopeOption.Required, objTransactionOption))
                {
                    objMoneyTransaction = new Client.CwlbMoneyTransaction();
                    objMoneyTransaction.AccountIdCredit = aobjAccount[0].AccountId;
                    objMoneyTransaction.AccountIdDebit = aobjAccount[1].AccountId;
                    objMoneyTransaction.Amount = 100000;
                    objMoneyTransaction.ValidityDate = DateTime.Now.AddDays(1).Date;
                    Guid objMoneyTransactionId = objProxy.Debit(objMoneyTransaction);
                    objProxy.Credit(objMoneyTransactionId);
                    objTransaction.Complete();
                }
            }
            catch (FaultException excObject)
            {
                Console.WriteLine(excObject.Message);
            }

            // mit Fehler
            try
            {
                using (TransactionScope objTransaction = 
                    new TransactionScope(TransactionScopeOption.Required, objTransactionOption))
                {
                    objMoneyTransaction = new Client.CwlbMoneyTransaction();
                    objMoneyTransaction.AccountIdCredit = aobjAccount[0].AccountId;
                    objMoneyTransaction.AccountIdDebit = aobjAccount[1].AccountId;
                    objMoneyTransaction.Amount = 10000000.00;
                    objMoneyTransaction.ValidityDate = DateTime.Now.AddDays(1).Date;
                    Guid objMoneyTransactionId = objProxy.Debit(objMoneyTransaction);
                    objProxy.Credit(objMoneyTransactionId);
                    objTransaction.Complete();
                }
            }
            catch (FaultException excObject)
            {
                Console.WriteLine(excObject.Message);
            }

            // ----------------------------------------------------------------
            // Clientseitige Transaktion nutzen (smart client)
            // ----------------------------------------------------------------
            // ohne Fehler
            WSHttpBinding objBinding = new WSHttpBinding();
            objBinding.TransactionFlow = true;
            IwlbAccountingService objSmartProxy = ChannelFactory<IwlbAccountingService>.CreateChannel(
                objBinding, new EndpointAddress("http://localhost:4711/Accounting"));
            CwlbMoneyTransaction objMoneyTransactionDirect = null;
            try
            {
                using (TransactionScope objTransaction = 
                    new TransactionScope(TransactionScopeOption.Required, objTransactionOption))
                {
                    objMoneyTransactionDirect = new CwlbMoneyTransaction();
                    objMoneyTransactionDirect.AccountIdCredit = aobjAccount[0].AccountId;
                    objMoneyTransactionDirect.AccountIdDebit = aobjAccount[1].AccountId;
                    objMoneyTransactionDirect.Amount = 100000;
                    objMoneyTransactionDirect.ValidityDate = DateTime.Now.AddDays(1).Date;
                    Guid objMoneyTransactionId = objSmartProxy.Debit(objMoneyTransactionDirect);
                    objSmartProxy.Credit(objMoneyTransactionId);
                    objTransaction.Complete();
                }
            }
            catch (FaultException excObject)
            {
                Console.WriteLine(excObject.Message);
            }

            // mit Fehler
            try
            {
                using (TransactionScope objTransaction = 
                    new TransactionScope(TransactionScopeOption.Required, objTransactionOption))
                {
                    objMoneyTransactionDirect = new CwlbMoneyTransaction();
                    objMoneyTransactionDirect.AccountIdCredit = aobjAccount[0].AccountId;
                    objMoneyTransactionDirect.AccountIdDebit = aobjAccount[1].AccountId;
                    objMoneyTransactionDirect.Amount = 10000000.00;
                    objMoneyTransactionDirect.ValidityDate = DateTime.Now.AddDays(1).Date;
                    Guid objMoneyTransactionId = objSmartProxy.Debit(objMoneyTransactionDirect);
                    objSmartProxy.Credit(objMoneyTransactionId);
                    objTransaction.Complete();
                }
            }
            catch (FaultException excObject)
            {
                Console.WriteLine(excObject.Message);
            }


            // ----------------------------------------------------------------
            // Serverseitige Transaktion mit clientseitiger Steuerung kombinieren
            // ----------------------------------------------------------------
            // ohne Fehler
            try
            {
                using (TransactionScope objTransaction = new TransactionScope())
                {
                    objMoneyTransaction = new Client.CwlbMoneyTransaction();
                    objMoneyTransaction.AccountIdCredit = aobjAccount[0].AccountId;
                    objMoneyTransaction.AccountIdDebit = aobjAccount[1].AccountId;
                    objMoneyTransaction.Amount = 125000.00;
                    objMoneyTransaction.ValidityDate = DateTime.Now.AddDays(1).Date;
                    objProxy.Accounting(objMoneyTransaction);
                }
            }
            catch (FaultException excObject)
            {
                Console.WriteLine(excObject.Message);
            }

            // mit Fehler
            try
            {
                using (TransactionScope objTransaction = new TransactionScope())
                {
                    objMoneyTransaction = new Client.CwlbMoneyTransaction();
                    objMoneyTransaction.AccountIdCredit = aobjAccount[0].AccountId;
                    objMoneyTransaction.AccountIdDebit = aobjAccount[1].AccountId;
                    objMoneyTransaction.Amount = 10000000.00;
                    objMoneyTransaction.ValidityDate = DateTime.Now.AddDays(1).Date;
                    objProxy.Accounting(objMoneyTransaction);
                }
            }
            catch (FaultException excObject)
            {
                Console.WriteLine(excObject.Message);
            }

            // ----------------------------------------------------------------
            // Client schließen
            // ----------------------------------------------------------------
            objProxy.Close();
        }
    }
}
