﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;

namespace WeroSoft.Samples.LearningWcfContracts
{
    /// <summary>
    /// Definiert die Funktionalität des Demodienstes für transaktionelles Verhalten der WCF.
    /// </summary>
    [ServiceContract(Name="AccountingService", Namespace=GwlbGlobals.WcfNamespace, SessionMode=SessionMode.Required)]
    public interface IwlbAccountingService
    {
        /// <summary>
        /// Setzt die Demodaten auf den Anfangswert zurück.
        /// </summary>
        [OperationContract(Name = "Reset")]
        void ResetData();

        /// <summary>
        /// Liest alle erfassten Personen aus der Datenbank.
        /// </summary>
        /// <returns></returns>
        [OperationContract(Name = "EnumeratePersons")]
        List<CwlbPerson> EnumeratePersons();

        /// <summary>
        /// Liefert alle Konten einer Person.
        /// </summary>
        /// <returns>Liste der Konten einer Person.</returns>
        [OperationContract(Name = "EnumerateAccounts")]
        List<CwlbAccount> EnumerateAccounts(Guid objPersonId);

        /// <summary>
        /// Führt eine Belastung eines definierten Kontos durch.
        /// </summary>
        /// <param name="objTransaction">Definition der Belastung</param>
        /// <returns>Identifikation des Geldtransfers.</returns>
        [FaultContract(typeof(FaultException))]
        [OperationContract(Name = "Debit")]
        [TransactionFlow(TransactionFlowOption.Mandatory)]
        Guid Debit(CwlbMoneyTransaction objBlTransaction);

        /// <summary>
        /// Führt eine Gutschrift auf ein definiertes Kontos durch.
        /// </summary>
        /// <param name="objTransaction">Definition der Gutschrift.</param>
        [FaultContract(typeof(FaultException))]
        [OperationContract(Name = "Credit")]
        [TransactionFlow(TransactionFlowOption.Mandatory)]
        void Credit(Guid objMonyTransactionId);

        /// <summary>
        /// Führt eine Buchung aus (Belastung und Gutschrift).
        /// </summary>
        /// <param name="objTransaction">Durchzuführende Transaktion.</param>
        [FaultContract(typeof(FaultException))]
        [OperationContract(Name = "Accounting")]
        void Accounting(CwlbMoneyTransaction objBlTransaction);
    }
}
