﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;
using System.Transactions;
using WeroSoft.Samples.LearningWcfContracts;
using WeroSoft.Samples.LearningWcfDatabase;
using Db = WeroSoft.Samples.LearningWcfDatabase;

namespace WeroSoft.Samples.LearningWcfService
{
    /// <summary>
    /// Implementiert den Demodienst für transaktionelles Verhalten.
    /// </summary>
    [ServiceBehavior(
        InstanceContextMode = InstanceContextMode.PerSession, 
        ConcurrencyMode = ConcurrencyMode.Single, 
        ReleaseServiceInstanceOnTransactionComplete = true,
        TransactionIsolationLevel = IsolationLevel.ReadCommitted,
        TransactionAutoCompleteOnSessionClose = false,
        TransactionTimeout="00:00:30")]
    public class CwlbAccountingService : IwlbAccountingService
    {
        #region Resetdata
        /// <summary>
        /// Setzt die Daten in der Datenbank auf ihren Anfangswert zurück.
        /// </summary>
        public void ResetData()
        {
            // Transaktionsstatus anzeigen
            ShowTransactionInfo("Reset von Daten: Vor der internen Erstellung der Transaktion.");

            // Alle Daten mit Hilfe einer Transaktion zurücksetzen.
            using (TransactionScope objTransaction = new TransactionScope())
            using (CwlbDatabase objDb = new CwlbDatabase())
            {
                // Transaktionsstatus anzeigen
                ShowTransactionInfo("Reset von Daten: In der Transaktion.");
                EmptyDatabase(objDb);
                CreatePersons(objDb);
                CreateAccounts(objDb);
                CreateTransactions(objDb);
                objTransaction.Complete();
            }
        }

        /// <summary>
        /// Löscht alle Daten der Datenbank.
        /// </summary>
        /// <param name="objDb">Datenbankkontext für das Löschen der Daten.</param>
        private void EmptyDatabase(CwlbDatabase objDb)
        {
            objDb.ExecuteStoreCommand("DELETE FROM MoneyTransactionSet");
            objDb.ExecuteStoreCommand("DELETE FROM AccountSet");
            objDb.ExecuteStoreCommand("DELETE FROM PersonSet");
        }

        /// <summary>
        /// Erstellt die Personenobjekte
        /// </summary>
        /// <param name="objDb">Datenbankkontext.</param>
        private void CreatePersons(CwlbDatabase objDb)
        {
            objDb.PersonSet.AddObject(new Person { PersonId = Guid.NewGuid(), Name = "Alwayszero", FirstName = "Peter" });
            objDb.PersonSet.AddObject(new Person { PersonId = Guid.NewGuid(), Name = "Nomoremany", FirstName = "Gusti" });
            objDb.PersonSet.AddObject(new Person { PersonId = Guid.NewGuid(), Name = "Hasalot", FirstName = "Suzanna" });
            objDb.PersonSet.AddObject(new Person { PersonId = Guid.NewGuid(), Name = "Isamillionaire", FirstName = "Roberto" });
            objDb.PersonSet.AddObject(new Person { PersonId = Guid.NewGuid(), Name = "Lostalot", FirstName = "Karin" });
            objDb.SaveChanges();
        }

        /// <summary>
        /// Erstellt die Konti der Personen.
        /// </summary>
        /// <param name="objDb">Datenbankkontext.</param>
        private void CreateAccounts(CwlbDatabase objDb)
        {
            // Neuer Account
            Account objAccount = new Account();
            objAccount.AccountId = Guid.NewGuid();
            objAccount.Person = objDb.PersonSet.Single(item => item.Name == "Alwayszero");
            objAccount.Name = "Kontokorrent";
            objAccount.Currency = "EURO";
            objAccount.StartingBalance = 0.0;
            objAccount.CurrentBalance = 0.0;
            objDb.AccountSet.AddObject(objAccount);

            // Neuer Account
            objAccount = new Account();
            objAccount.AccountId = Guid.NewGuid();
            objAccount.Person = objDb.PersonSet.Single(item => item.Name == "Nomoremany");
            objAccount.Name = "Kontokorrent";
            objAccount.Currency = "EURO";
            objAccount.StartingBalance = -100.0;
            objAccount.CurrentBalance = objAccount.StartingBalance;
            objDb.AccountSet.AddObject(objAccount);

            // Neuer Account
            objAccount = new Account();
            objAccount.AccountId = Guid.NewGuid();
            objAccount.Person = objDb.PersonSet.Single(item => item.Name == "Hasalot");
            objAccount.Name = "Kontokorrent";
            objAccount.Currency = "EURO";
            objAccount.StartingBalance = 212695.25;
            objAccount.CurrentBalance = objAccount.StartingBalance;
            objDb.AccountSet.AddObject(objAccount);

            // Neuer Account
            objAccount = new Account();
            objAccount.AccountId = Guid.NewGuid();
            objAccount.Person = objDb.PersonSet.Single(item => item.Name == "Hasalot");
            objAccount.Name = "Sparkonto";
            objAccount.Currency = "EURO";
            objAccount.StartingBalance = 956875.12;
            objAccount.CurrentBalance = objAccount.StartingBalance;
            objDb.AccountSet.AddObject(objAccount);

            // Neuer Account
            objAccount = new Account();
            objAccount.AccountId = Guid.NewGuid();
            objAccount.Person = objDb.PersonSet.Single(item => item.Name == "Isamillionaire");
            objAccount.Name = "Kontokorrent";
            objAccount.Currency = "EURO";
            objAccount.StartingBalance = 1500000.00;
            objAccount.CurrentBalance = objAccount.StartingBalance;
            objDb.AccountSet.AddObject(objAccount);

            // Neuer Account
            objAccount = new Account();
            objAccount.AccountId = Guid.NewGuid();
            objAccount.Person = objDb.PersonSet.Single(item => item.Name == "Isamillionaire");
            objAccount.Name = "Sparkonto";
            objAccount.Currency = "EURO";
            objAccount.StartingBalance = 4500000.00;
            objAccount.CurrentBalance = objAccount.StartingBalance;
            objDb.AccountSet.AddObject(objAccount);

            // Neuer Account
            objAccount = new Account();
            objAccount.AccountId = Guid.NewGuid();
            objAccount.Person = objDb.PersonSet.Single(item => item.Name == "Lostalot");
            objAccount.Name = "Kontokorrent";
            objAccount.Currency = "EURO";
            objAccount.StartingBalance = 10.0;
            objAccount.CurrentBalance = objAccount.StartingBalance;
            objDb.AccountSet.AddObject(objAccount);
            objDb.SaveChanges();
        }

        /// <summary>
        /// Erstellt erste Transaktionen.
        /// </summary>
        /// <param name="objDb">Datenbankkontext.</param>
        private void CreateTransactions(CwlbDatabase objDb)
        {
            Person objPerson = objDb.PersonSet.Single(item => item.Name == "Isamillionaire");
            Db.MoneyTransaction objTransaction = new Db.MoneyTransaction();
            objTransaction.TransactionId = Guid.NewGuid();
            objTransaction.AccountDebit = objPerson.Account.Single(item => item.Name == "Kontokorrent");
            objTransaction.AccountCredit = objPerson.Account.Single(item => item.Name == "Sparkonto");
            objTransaction.Amount = 500000.00;
            objTransaction.AccountingDate = DateTime.UtcNow;
            objTransaction.ValidityDate = DateTime.Today;
            objDb.AddToMoneyTransactionSet(objTransaction);
            objDb.SaveChanges();

            DebitInternal(objTransaction, objDb);
            CreditInternal(objTransaction, objDb);
        }
        #endregion

        /// <summary>
        /// Liefert alle Personen objekte.
        /// </summary>
        /// <returns>Liste der Personenobjekte.</returns>
        public List<CwlbPerson> EnumeratePersons()
        {
            List<CwlbPerson> cobjPerson = new List<CwlbPerson>();
            using (CwlbDatabase objDb = new CwlbDatabase())
            {
                foreach (Person objPerson in objDb.PersonSet)
                {
                    cobjPerson.Add(new CwlbPerson
                    {
                        PersonId = objPerson.PersonId,
                        Name = objPerson.Name,
                        FirstName = objPerson.FirstName
                    });
                }
            }
            return cobjPerson;
        }

        /// <summary>
        /// Liefert alle Konten einer Person.
        /// </summary>
        /// <param name="objPersonId">Identifikation der person für die die Konti geliefert werden müssen.</param>
        /// <returns>Liste der Konten.</returns>
        public List<CwlbAccount> EnumerateAccounts(Guid objPersonId)
        {
            List<CwlbAccount> cobjAccount = new List<CwlbAccount>();
            using (CwlbDatabase objDb = new CwlbDatabase())
            {
                Person objPerson = objDb.PersonSet.Single(person => person.PersonId == objPersonId);
                foreach (Account objAccount in objPerson.Account)
                {
                    cobjAccount.Add(new CwlbAccount
                    {
                        AccountId = objAccount.AccountId,
                        PersonId = objPerson.PersonId,
                        Name = objAccount.Name,
                        Currency = objAccount.Currency,
                        CurrentBalance = objAccount.CurrentBalance,
                        StartingBalance = objAccount.StartingBalance
                    });
                }
            }
            return cobjAccount;
        }

        /// <summary>
        /// Führt eine Kontobelastung durch. Diese Funktion ist nur die halbe Transaktion. Sie muss immer
        /// im paar mit der Funktion Credit aufgerufen (Debit zuerst).
        /// </summary>
        /// <param name="objBlTransaction">Geldtransaktion, die durchgeführt werden soll.</param>
        /// <returns>Identifikation der Geldtransaktion.</returns>
        [OperationBehavior(TransactionAutoComplete = false, TransactionScopeRequired=true)]
        public Guid Debit(CwlbMoneyTransaction objBlTransaction)
        {
            try
            {
                // Transaktionsstatus anzeigen
                ShowTransactionInfo("Methode Debit", false);
                // Datenbankzugriff durchführen
                using (CwlbDatabase objDb = new CwlbDatabase())
                {
                    // Transformiere BL-Transaktion in DB-Transaktion
                    MoneyTransaction objTransaction = new MoneyTransaction();
                    objTransaction.TransactionId = Guid.NewGuid();
                    objTransaction.AccountDebit = objDb.AccountSet.Single(
                        account => account.AccountId == objBlTransaction.AccountIdDebit);
                    objTransaction.AccountCredit = objDb.AccountSet.Single(
                        account => account.AccountId == objBlTransaction.AccountIdCredit);
                    objTransaction.Amount = objBlTransaction.Amount;
                    objTransaction.ValidityDate = objBlTransaction.ValidityDate;
                    objTransaction.AccountingDate = DateTime.UtcNow;

                    // Speichern der Daten der Geldtransaktion
                    objDb.AddToMoneyTransactionSet(objTransaction);

                    // Saldo nachführen
                    DebitInternal(objTransaction, objDb);

                    // TransaktionsId zurückgeben
                    return objTransaction.TransactionId;
                }
            }
            catch (InvalidOperationException excObject)
            {
                Console.WriteLine("--> Fehler: {0}", excObject.Message);
                throw new FaultException(new FaultReason(excObject.Message));
            }
        }

        /// <summary>
        /// Führt eine Kontogutschrift durch. Diese Funktion ist nur die halbe Transaktion. Sie wird
        /// nachläufig zu der Methode Debit aufgerufen.
        /// </summary>
        /// <param name="objMonyTransactionId">Identifikation der Geldtransaktion, die deren Gutschrift verbucht werden soll.</param>
        [OperationBehavior(TransactionAutoComplete = true, TransactionScopeRequired = true)]
        public void Credit(Guid objMonyTransactionId)
        {
            try
            {
                // Transaktionsstatus anzeigen
                ShowTransactionInfo("Methode Credit");
                // Datenbankzugriff durchführen
                using (CwlbDatabase objDb = new CwlbDatabase())
                {
                    // Transaktionsdaten lesen
                    MoneyTransaction objTransaction = objDb.MoneyTransactionSet.Single(
                        objMoneyTransaction => objMoneyTransaction.TransactionId == objMonyTransactionId);
                    // Saldo nachführen
                    CreditInternal(objTransaction, objDb);
                }
            }
            catch (InvalidOperationException excObject)
            {
                Console.WriteLine("--> Fehler: {0}", excObject.Message);
                throw new FaultException(new FaultReason(excObject.Message));
            }
        }

        /// <summary>
        /// Führt eine vollständige Buchung durch.
        /// </summary>
        /// <param name="objTransactionId">Geldtransaktion, die durchgeführt werden soll.</param>
        [OperationBehavior(TransactionAutoComplete = true, TransactionScopeRequired = true)]
        public void Accounting(CwlbMoneyTransaction objBlTransaction)
        {
            try
            {
                // Transaktionsstatus anzeigen
                ShowTransactionInfo("Methode Accounting");
                // Datenbankzugriff durchführen
                using (CwlbDatabase objDb = new CwlbDatabase())
                {
                    // Transformiere BL-Transaktion in DB-Transaktion
                    MoneyTransaction objTransaction = new MoneyTransaction();
                    objTransaction.TransactionId = Guid.NewGuid();
                    objTransaction.AccountDebit = objDb.AccountSet.Single(account => account.AccountId == objBlTransaction.AccountIdDebit);
                    objTransaction.AccountCredit = objDb.AccountSet.Single(account => account.AccountId == objBlTransaction.AccountIdCredit);
                    objTransaction.Amount = objBlTransaction.Amount;
                    objTransaction.ValidityDate = objBlTransaction.ValidityDate;
                    objTransaction.AccountingDate = DateTime.UtcNow;

                    // Save transaction
                    objDb.AddToMoneyTransactionSet(objTransaction);

                    // Speichern der Daten der Geldtransaktion
                    DebitInternal(objTransaction, objDb);
                    CreditInternal(objTransaction, objDb);
                }
            }
            catch (InvalidOperationException excObject)
            {
                Console.WriteLine("--> Fehler: {0}", excObject.Message);
                throw new FaultException(new FaultReason(excObject.Message));
            }
            catch (Exception excObject)
            {
                Console.WriteLine("--> Fehler: {0}", excObject.Message);
                throw new FaultException(new FaultReason(excObject.Message));
            }
        }


        #region Interne Methoden
        /// <summary>
        /// Bucht die Belastung auf dem Konto.
        /// </summary>
        /// <param name="objTransaction">Geldtransaktion, die gebucht werden soll.</param>
        /// <param name="objDb">Datenbankkontext.</param>
        private void DebitInternal(Db.MoneyTransaction objTransaction, CwlbDatabase objDb)
        {
            Db.Account objAccountDebit = objTransaction.AccountDebit;
            objAccountDebit.CurrentBalance -= objTransaction.Amount;
            // Prüfen ob das Belastungskonto negativ wird
            if (objAccountDebit.CurrentBalance < 0)
            {
                throw new InvalidOperationException("Es ist nicht genügend Saldo vorhanden.");
            }
            objDb.SaveChanges();
        }

        /// <summary>
        /// Bucht die Gutschrift auf einem Konto.
        /// </summary>
        /// <param name="objTransaction">Geldtransaktion, die gebucht werden soll.</param>
        /// <param name="objDb">Datenbankkontext.</param>
        private void CreditInternal(Db.MoneyTransaction objTransaction, CwlbDatabase objDb)
        {
            Db.Account objAccountCredit = objTransaction.AccountCredit;
            objAccountCredit.CurrentBalance += objTransaction.Amount;
            objDb.SaveChanges();
        }

        /// <summary>
        /// Zeigt den aktuellen Transaktionsstatus.
        /// </summary>
        private static void ShowTransactionInfo(string strTransaktionsInfo, bool bRegister = true)
        {
            Console.WriteLine();
            Console.WriteLine(strTransaktionsInfo);
            if (Transaction.Current == null)
            {
                Console.WriteLine("--> Keine Transaktion aktiv");
            }
            else
            {
                TransactionInformation info = Transaction.Current.TransactionInformation;
                Console.WriteLine("- Zeitpunkt der Erstellung der verteilten Transaktion: {0}",
                    info.CreationTime);
                Console.WriteLine("- Identifikation der verteilten Transaktion: . . . . . {0}",
                    info.DistributedIdentifier);
                Console.WriteLine("- Lokale Identifikation der verteilten Transaktion:. . {0}",
                    info.LocalIdentifier);
                Console.WriteLine("- Status der verteilten Transaktion: . . . . . . . . . {0}",
                    info.Status);
                Console.WriteLine("- Transaction.Current.IsolationLevel:. . . . . . . . . {0}",
                    Transaction.Current.IsolationLevel);

                // Registration des Transaktionsabschlusses, sofern erwünscht
                if (bRegister)
                {
                    Transaction.Current.TransactionCompleted += Current_TransactionCompleted;
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Abschluss einer Transaktion.
        /// </summary>
        static void Current_TransactionCompleted(object sender, TransactionEventArgs objEventArgs)
        {
            objEventArgs.Transaction.TransactionCompleted -= Current_TransactionCompleted;
            Console.WriteLine("- Status der Transaktion nach Abschluss: . . . . . . . {0}",
                objEventArgs.Transaction.TransactionInformation.Status);
        }
        #endregion
    }
}
