﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Transactions;
using WeroSoft.Samples.LearningWcfContracts;

namespace WeroSoft.Samples.LearningWcfService
{
    /// <summary>
    /// Implementiert den Dienst für Dokumentenverwaltung
    /// </summary>
    public class CwlbDocumentManager : IwlbDocumentManager
    {
        /// <summary>
        /// Definiert den Namen des Verzeichnisses, das die zum Sevrer hochgeladenen Dateien aufnimmt.
        /// </summary>
        internal const string UPLOAD = "Upload";

        /// <summary>
        /// Liefert die Dokumentenheader der Dateien im Upload-Verzeichnis.
        /// </summary>
        /// <returns></returns>
        public List<CwlbDocumentHeader> GetDocumentList()
        {
            List<CwlbDocumentHeader> cobjDocumentHeader = new List<CwlbDocumentHeader>();
            if (Directory.Exists(UPLOAD))
            {
                foreach (string strFile in Directory.GetFiles(UPLOAD))
                {
                    FileInfo objInfo = new FileInfo(strFile);
                    cobjDocumentHeader.Add(new CwlbDocumentHeader
                    {
                        Name = objInfo.Name,
                        Size = objInfo.Length
                    });
                }
            }

            return cobjDocumentHeader.ToList();
        }

        /// <summary>
        /// Registriert eine neue Datei, die auf den Server übertragen wurde.
        /// </summary>
        /// <param name="objDocumentHeader">Dokumentenheader der Datei, die registriert werden soll.</param>
        public void RegisterDocument(CwlbDocumentHeader objDocumentHeader)
        {
            // Umbenennen der Datei
            string strFileSource = Path.Combine(UPLOAD, objDocumentHeader.Id);
            string strFileTarget = Path.Combine(UPLOAD, objDocumentHeader.Name);
            try
            {
                File.Move(strFileSource, strFileTarget);

                // Clientrückruf für die Modellhandhabung
                OperationContext.Current.GetCallbackChannel<IwlbDocumentManagementCallback>().OnAddedDocument(objDocumentHeader);
            }
            catch (Exception excObject)
            {
                File.Delete(strFileSource);
                Console.WriteLine(excObject.Message);
                throw new FaultException(excObject.Message);
            }
        }

        /// <summary>
        /// Löscht eine Datei auf dem Server.
        /// </summary>
        /// <param name="objDocumentHeader"></param>
        public void RemoveDocument(CwlbDocumentHeader objDocumentHeader)
        {
            try
            {
                string strFileName = Path.Combine(UPLOAD, objDocumentHeader.Name);
                File.Delete(strFileName);

                // Clientrückruf für die Modellhandhabung
                OperationContext.Current.GetCallbackChannel<IwlbDocumentManagementCallback>().OnRemovedDocument(objDocumentHeader);
            }
            catch (Exception excObject)
            {
                throw new FaultException(excObject.Message);
            }
        }

        /// <summary>
        /// Prüft, ob das Verzeichnis für die Speicherung exisitiert und erstellt das Verzeichnis wenn nicht.
        /// </summary>
        internal static void CheckUploadFolder()
        {
            if (!Directory.Exists(UPLOAD))
            {
                Directory.CreateDirectory(UPLOAD);
            }
        }
    }
}
