﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;
using WeroSoft.Samples.LearningWcfContracts;

namespace WeroSoft.Samples.LearningWcfService
{
    /// <summary>
    /// Implementiert den Dienst für die Übertragung der Dateien.
    /// </summary>
    public class CwlbDocumentStream : IwlbDocumentStream
    {
        /// <summary>
        /// Überträgt eine Datei vom Server zum Server.
        /// </summary>
        /// <param name="strName">Name der Datei, die zum Client übertragen werden soll.</param>
        /// <returns>Datenstrom der Übertragung.</returns>
        public Stream DownloadDocument(string strName)
        {
            // Vorbereitungen
            FileStream streamFile = null;
            string strFileName = Path.Combine(CwlbDocumentManager.UPLOAD, strName);

            // Prüfen ob die angeforderte Datei existiert
            if (File.Exists(strFileName))
            {
                try
                {
                    // Übertragen der Daten
                    streamFile = new FileStream(strFileName, FileMode.Open, FileAccess.Read, FileShare.Read);
                }
                catch (Exception excObject)
                {
                    if (streamFile != null)
                    {
                        streamFile.Close();
                    }
                    throw new FaultException(excObject.Message);
                }
            }
            else
            {
                throw new FaultException("Die angeforderte Datei konnte nicht gefunden werden.");
            }

            return streamFile;
        }

        /// <summary>
        /// Empfängt eine Datei vom Client.
        /// </summary>
        /// <param name="streamDocument">Datenstrom der empfangen wird.</param>
        /// <returns>Identifikation der empfangenen Datei.</returns>
        public string UploadDocument(Stream streamDocument)
        {
            // Vorbereitungen
            string strTempFileName = Guid.NewGuid().ToString();
            CwlbDocumentManager.CheckUploadFolder();

            // Daten hochladen
            using (FileStream streamFile = new FileStream(
                Path.Combine(CwlbDocumentManager.UPLOAD, strTempFileName),
                FileMode.OpenOrCreate, FileAccess.Write))
            {
                // Definieren des internen Pufferspeichers für die Übertragung
                int iBufferSize = 81920;
                byte[] abyteStreamed = new byte[iBufferSize];

                // Erster Lesezugriff
                int iBytesRead = streamDocument.Read(abyteStreamed, 0, iBufferSize);

                // Schleife für das Übertragen der Daten
                while (iBytesRead > 0)
                {
                    streamFile.Write(abyteStreamed, 0, iBytesRead);
                    iBytesRead = streamDocument.Read(abyteStreamed, 0, iBufferSize);
                }

                // Den Übertragungsdatenstrom schliessen
                streamDocument.Close();
            }
            return strTempFileName;
        }
    }
}
