﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;
using WeroSoft.Samples.LearningWcfContract;

namespace WeroSoft.Samples.LearningWcfClient
{
    /// <summary>
    /// Implementiert den Client für den REST-Dienst.
    /// </summary>
    class CwlbRestClient : ClientBase<IwlbRestDemoService>, IwlbRestDemoService
    {
        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbRestClient
        /// </summary>
        /// <param name="strEndpointConfiguration">Name der Endpunktkonfiguration</param>
        public CwlbRestClient(string strEndpointConfiguration)
            : base(strEndpointConfiguration)
        {
        }

        /// <summary>
        /// Liefert alle Daten des Servers
        /// </summary>
        /// <returns>Array der angeforderten Datenelementen.</returns>
        public CwlbDemoData[] GetAllDataJson()
        {
            return this.Channel.GetAllDataJson();
        }

        /// <summary>
        /// Liefert alle Daten des Servers
        /// </summary>
        /// <returns>Array der angeforderten Datenelementen.</returns>
        public CwlbDemoData[] GetAllData(string sort = null)
        {
            return this.Channel.GetAllData(sort);
        }

        /// <summary>
        /// Liefert die Daten eines Jahres.
        /// </summary>
        /// <param name="year">Jahr, von dem die Daten geliefert werden sollen.</param>
        /// <returns>Array der angeforderten Datenelementen.</returns>
        public CwlbDemoData[] GetDataByYear(string year)
        {
            return this.Channel.GetDataByYear(year);
        }

        /// <summary>
        /// Liefert ein Array von Daten eines Monats
        /// </summary>
        /// <param name="year">Jahr, von dem die Daten geliefert werden sollen.</param>
        /// <param name="month">Monat, von dem die Daten geliefert werden sollen.</param>
        /// <returns>Array der angeforderten Datenelementen.</returns>
        public CwlbDemoData[] GetDataByMonth(string year, string month)
        {
            return this.Channel.GetDataByMonth(year, month);
        }

        /// <summary>
        /// Liefert ein definiertes Element eines Tages.
        /// </summary>
        /// <param name="year">Jahr, von dem die Daten geliefert werden sollen.</param>
        /// <param name="month">Monat, von dem die Daten geliefert werden sollen.</param>
        /// <param name="day">Tag von dem die Daten geliefert werden sollen.</param>
        /// <returns>Angefordertes Datenelement.</returns>
        public CwlbDemoData GetDataByDay(string year, string month, string day)
        {
            return this.Channel.GetDataByDay(year, month, day);
        }

        /// <summary>
        /// Liefert ein definiertes Element eines Tages.
        /// </summary>
        /// <param name="year">Jahr, von dem die Daten geliefert werden sollen.</param>
        /// <param name="month">Monat, von dem die Daten geliefert werden sollen.</param>
        /// <param name="day">Tag von dem die Daten geliefert werden sollen.</param>
        /// <returns>Angefordertes Datenelement.</returns>
        public CwlbDemoData GetDataByDayJson(string year, string month, string day)
        {
            return this.Channel.GetDataByDayJson(year, month, day);
        }

        /// <summary>
        /// Definiert ein Element.
        /// </summary>
        /// <param name="demoData">Neues Datenelement, das aufgenommen wurde.</param>
        /// <returns>Das erzeugte Element.</returns>
        public CwlbDemoData DefineDemoData(CwlbDemoData demoData)
        {
            return this.Channel.DefineDemoData(demoData);
        }

        /// <summary>
        /// Löscht ein Element.
        /// </summary>
        /// <param name="demoData">Datenelement, das gelöscht werden soll.</param>
        public void DeleteDemoData(CwlbDemoData demoData)
        {
            this.Channel.DeleteDemoData(demoData);
        }
    }
}
