﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Text;
using System.Threading.Tasks;
using WeroSoft.Samples.LearningWcfContract;

namespace WeroSoft.Samples.LearningWcfClient
{
    /// <summary>
    /// Implementiert den Konsolenclient der Demo für REST.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Einstiegspunkt in die Demoanwendung.
        /// </summary>
        static void Main()
        {
            // Erzeugen des Client und durchführen aller Zugriffsarten
            using (CwlbRestClient objClient = new CwlbRestClient("RestService"))
            {
                try
                {
                    Console.WriteLine("Alle Daten abfragen");
                    Console.WriteLine("--------------------");
                    foreach (CwlbDemoData objData in objClient.GetAllData())
                    {
                        WriteObjectData(objData);
                    }

                    Console.WriteLine("Alle Daten abfragen");
                    Console.WriteLine("--------------------");
                    foreach (CwlbDemoData objData in objClient.GetAllData("descending"))
                    {
                        WriteObjectData(objData);
                    }

                    Console.WriteLine();
                    Console.WriteLine("Ganzes Jahr abfragen");
                    Console.WriteLine("--------------------");
                    foreach (CwlbDemoData objData in objClient.GetDataByYear("2012"))
                    {
                        WriteObjectData(objData);
                    }

                    Console.WriteLine();
                    Console.WriteLine("Einen Monat abfragen");
                    Console.WriteLine("--------------------");
                    foreach (CwlbDemoData objData in objClient.GetDataByMonth("2012", "2"))
                    {
                        WriteObjectData(objData);
                    }

                    Console.WriteLine();
                    Console.WriteLine("Einen Tag abfragen");
                    Console.WriteLine("------------------");
                    CwlbDemoData objDay = objClient.GetDataByDay("2012", "2", "20");
                    WriteObjectData(objDay);

                    Console.WriteLine();
                    Console.WriteLine("Neue Daten ergänzen");
                    Console.WriteLine("--------------------");
                    CwlbDemoData objNew = new CwlbDemoData();
                    objNew.DateTimeValue = DateTime.Now.AddYears(1);
                    objNew.IntegerValue = 999;
                    objNew.StringValue = "Onlinetest";
                    CwlbDemoData objNewReturn = objClient.DefineDemoData(objNew);

                    foreach (CwlbDemoData objData in objClient.GetDataByYear(DateTime.Now.AddYears(1).Year.ToString()))
                    {
                        WriteObjectData(objData);
                    }
                }
                catch (CommunicationException excObject)
                {
                    Console.WriteLine(excObject.Message);
                }
                catch (Exception excObject)
                {
                    Console.WriteLine(excObject.Message);
                }
            }


            // Nutzung mit einem Smart-Client
            using (WebChannelFactory<IwlbRestDemoService> objFactory = new WebChannelFactory<IwlbRestDemoService>(new Uri("http://localhost:50738/RestDemoService.svc")))
            {
                IwlbRestDemoService objSmartClient = null;
                try
                {
                    // Client erzeugen
                    objSmartClient = objFactory.CreateChannel();

                    Console.WriteLine();
                    Console.WriteLine("Ganzes Jahr abfragen");
                    Console.WriteLine("--------------------");
                    foreach (CwlbDemoData objData in objSmartClient.GetDataByYear("2012"))
                    {
                        WriteObjectData(objData);
                    }

                    Console.WriteLine();
                    Console.WriteLine("Einen Monat abfragen");
                    Console.WriteLine("--------------------");
                    foreach (CwlbDemoData objData in objSmartClient.GetDataByMonth("2012", "2"))
                    {
                        WriteObjectData(objData);
                    }

                    Console.WriteLine();
                    Console.WriteLine("Einen Tag abfragen");
                    Console.WriteLine("------------------");
                    CwlbDemoData objDay = objSmartClient.GetDataByDay("2012", "2", "20");
                    WriteObjectData(objDay);

                    Console.WriteLine();
                    Console.WriteLine("Neue Daten ergänzen");
                    Console.WriteLine("--------------------");
                    CwlbDemoData objNew = new CwlbDemoData();
                    objNew.DateTimeValue = DateTime.Now.AddYears(2);
                    objNew.IntegerValue = 999;
                    objNew.StringValue = "Onlinetest";
                    CwlbDemoData objNewReturn = objSmartClient.DefineDemoData(objNew);
                    CwlbDemoData objDataToDelete = null;
                    foreach (CwlbDemoData objData in objSmartClient.GetDataByYear(DateTime.Now.AddYears(2).Year.ToString()))
                    {
                        WriteObjectData(objData);
                        objDataToDelete = objData;
                    }

                    Console.WriteLine();
                    Console.WriteLine("Bestehende Daten löschen");
                    Console.WriteLine("------------------------");
                    objSmartClient.DeleteDemoData(objDataToDelete);
                    CwlbDemoData[] aobjData = objSmartClient.GetDataByYear(DateTime.Now.AddYears(2).Year.ToString());
                    if (aobjData.Length == 0)
                    {
                        Console.WriteLine("Daten erfolgreich gelöscht.");
                    }
                    else
                    {
                        Console.WriteLine("Daten nicht erfolgreich gelöscht.");
                    }
                }
                catch (Exception excObject)
                {
                    Console.WriteLine(excObject.Message);
                }
            }
        }

        /// <summary>
        /// Einen gelesenen Datensatz ausgeben.
        /// </summary>
        /// <param name="objData">Objekt, dessen Daten auf die Konsole geschriben werden sollen.</param>
        private static void WriteObjectData(CwlbDemoData objData)
        {
            Console.WriteLine("{0}: {1} - {2}", objData.DateTimeValue.ToShortDateString(), objData.StringValue, objData.IntegerValue);
        }
    }
}
