﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Web;

namespace WeroSoft.Samples.LearningWcfContract
{
    /// <summary>
    /// Definiert die Schnittstelle des Dienstes.
    /// </summary>
    [ServiceContract(Name = "RestDemoService", Namespace = GwlbConstants.XmlNamespaceService)]
    public interface IwlbRestDemoService
    {
        /// <summary>
        /// Liefert alle Daten des Dienstes im JSON-Format.
        /// </summary>
        /// <returns>Array der angeforderten Datenelementen.</returns>
        [OperationContract]
        [WebGet(UriTemplate = "/Json/", ResponseFormat = WebMessageFormat.Json)]
        CwlbDemoData[] GetAllDataJson();

        /// <summary>
        /// Liefert alle Daten des Dienstes im XML-Format.
        /// </summary>
        /// <returns>Array der angeforderten Datenelementen.</returns>
        [OperationContract]
        [WebGet(UriTemplate = "/?Sort={sort}")]
        CwlbDemoData[] GetAllData(string sort);

        /// <summary>
        /// Liefert die Daten eines Jahres.
        /// </summary>
        /// <param name="year">Jahr, von dem die Daten geliefert werden sollen.</param>
        /// <returns>Array der angeforderten Datenelementen.</returns>
        [OperationContract]
        [WebGet(UriTemplate = "/{year}")]
        CwlbDemoData[] GetDataByYear(string year);

        /// <summary>
        /// Liefert ein Array von Daten eines Monats
        /// </summary>
        /// <param name="year">Jahr, von dem die Daten geliefert werden sollen.</param>
        /// <param name="month">Monat, von dem die Daten geliefert werden sollen.</param>
        /// <returns>Array der angeforderten Datenelementen.</returns>
        [OperationContract]
        [WebInvoke(UriTemplate = "/{year}/{month}", Method="GET")]
        CwlbDemoData[] GetDataByMonth(string year, string month);

        /// <summary>
        /// Liefert ein definiertes Element eines Tages.
        /// </summary>
        /// <param name="year">Jahr, von dem die Daten geliefert werden sollen.</param>
        /// <param name="month">Monat, von dem die Daten geliefert werden sollen.</param>
        /// <param name="day">Tag von dem die Daten geliefert werden sollen.</param>
        /// <returns>Angefordertes Datenelement.</returns>
        [OperationContract]
        [WebGet(UriTemplate = "/{year}/{month}/{day}")]
        CwlbDemoData GetDataByDay(string year, string month, string day);

        /// <summary>
        /// Liefert ein definiertes Element eines Tages.
        /// </summary>
        /// <param name="year">Jahr, von dem die Daten geliefert werden sollen.</param>
        /// <param name="month">Monat, von dem die Daten geliefert werden sollen.</param>
        /// <param name="day">Tag von dem die Daten geliefert werden sollen.</param>
        /// <returns>Angefordertes Datenelement.</returns>
        [OperationContract]
        [WebGet(UriTemplate = "/Json/{year}/{month}/{day}", ResponseFormat = WebMessageFormat.Json)]
        CwlbDemoData GetDataByDayJson(string year, string month, string day);

        /// <summary>
        /// Definiert ein Element.
        /// </summary>
        /// <param name="demoData">Neues Datenelement, das aufgenommen wurde.</param>
        /// <returns>Das erzeugte Element.</returns>
        [OperationContract]
        [WebInvoke(UriTemplate = "/", Method = System.Net.WebRequestMethods.Http.Post)]
        CwlbDemoData DefineDemoData(CwlbDemoData demoData);

        /// <summary>
        /// Definiert ein Element.
        /// </summary>
        /// <param name="demoData">Neues Datenelement, das aufgenommen wurde.</param>
        /// <returns>Das erzeugte Element.</returns>
        [OperationContract]
        [WebInvoke(UriTemplate = "/", Method = "DELETE")]
        void DeleteDemoData(CwlbDemoData demoData);
    }
}
