﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Activation;
using WeroSoft.Samples.LearningWcfContract;
using System.ServiceModel.Web;

namespace WeroSoft.Samples.LearningWcfImplementation
{
    /// <summary>
    /// Implementierung des Dienstes IwlbRestDemoService.
    /// </summary>
    [JavascriptCallbackBehavior]
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerCall)]
    public class CwlbRestDemoService : IwlbRestDemoService
    {
        /// <summary>
        /// Definiert die Liste der Daten des Dienstes.
        /// </summary>
        private static List<CwlbDemoData> _cobjData = new List<CwlbDemoData>();

        /// <summary>
        /// Initialisiert die statischen Daten der Klasse CwlbDemoService.
        /// </summary>
        static CwlbRestDemoService()
        {
            DateTime dtData = new DateTime(2012, 1, 1);
            for (int iCount = 1; iCount <= 366; iCount++)
            {
                CwlbDemoData objData = new CwlbDemoData();
                objData.IntegerValue = dtData.Day;
                objData.StringValue = string.Format("Datenelement Nummer {0}", iCount);
                objData.DateTimeValue = dtData;
                _cobjData.Add(objData);
                dtData = dtData.AddDays(1);
            }
        }

        /// <summary>
        /// Liefert alle Daten des Servers
        /// </summary>
        /// <returns>Array der angeforderten Datenelementen.</returns>
        public CwlbDemoData[] GetAllDataJson()
        {
            // Verhindern das die Daten im Zwischenspeicher landen
            WebOperationContext.Current.OutgoingResponse.Headers.Add("Cache-Control", "no-cache");
            // Daten direkt liefern
            return _cobjData.ToArray();
        }


        /// <summary>
        /// Liefert alle Daten des Servers
        /// </summary>
        /// <returns>Array der angeforderten Datenelementen.</returns>
        public CwlbDemoData[] GetAllData(string sort)
        {
            // Verhindern das die Daten im Zwischenspeicher landen
            WebOperationContext.Current.OutgoingResponse.Headers.Add("Cache-Control", "no-cache");
            // Daten in abhängigkeit der Sortierung leifern
            if (sort == null || string.Compare(sort, "ascending", true) == 0)
            {
                return _cobjData.ToArray();
            }
            else
            {
                List<CwlbDemoData> cobjTempList = _cobjData.ToList();
                cobjTempList.Reverse();
                return cobjTempList.ToArray();
            }
        }

        /// <summary>
        /// Liefert die Daten eines Jahres.
        /// </summary>
        /// <param name="year">Jahr, von dem die Daten geliefert werden sollen.</param>
        /// <returns>Array der angeforderten Datenelementen.</returns>
        public CwlbDemoData[] GetDataByYear(string year)
        {
            return _cobjData.Where(objItem => objItem.DateTimeValue.Year == Convert.ToInt32(year)).ToArray();
        }

        /// <summary>
        /// Liefert ein Array von Daten eines Monats
        /// </summary>
        /// <param name="year">Jahr, von dem die Daten geliefert werden sollen.</param>
        /// <param name="month">Monat, von dem die Daten geliefert werden sollen.</param>
        /// <returns>Array der angeforderten Datenelementen.</returns>
        public CwlbDemoData[] GetDataByMonth(string year, string month)
        {
            return _cobjData.Where(objItem => objItem.DateTimeValue.Year == Convert.ToInt32(year) &&
                                              objItem.DateTimeValue.Month == Convert.ToInt32(month)).ToArray();
        }

        /// <summary>
        /// Liefert ein definiertes Element eines Tages.
        /// </summary>
        /// <param name="year">Jahr, von dem die Daten geliefert werden sollen.</param>
        /// <param name="month">Monat, von dem die Daten geliefert werden sollen.</param>
        /// <param name="day">Tag von dem die Daten geliefert werden sollen.</param>
        /// <returns>Angefordertes Datenelement.</returns>
        public CwlbDemoData GetDataByDay(string year, string month, string day)
        {
            return _cobjData.FirstOrDefault(objItem => 
                objItem.DateTimeValue.Year == Convert.ToInt32(year) &&
                objItem.DateTimeValue.Month == Convert.ToInt32(month) &&
                objItem.DateTimeValue.Day == Convert.ToInt32(day));
        }

        /// <summary>
        /// Liefert ein definiertes Element eines Tages.
        /// </summary>
        /// <param name="year">Jahr, von dem die Daten geliefert werden sollen.</param>
        /// <param name="month">Monat, von dem die Daten geliefert werden sollen.</param>
        /// <param name="day">Tag von dem die Daten geliefert werden sollen.</param>
        /// <returns>Angefordertes Datenelement.</returns>
        public CwlbDemoData GetDataByDayJson(string year, string month, string day)
        {
            return GetDataByDay(year, month, day);
        }


        /// <summary>
        /// Definiert ein Element.
        /// </summary>
        /// <param name="demoData">Neues Datenelement, das aufgenommen wird, respektive verändert wird.</param>
        /// <returns>Das neue oder veränderte Element.</returns>
        public CwlbDemoData DefineDemoData(CwlbDemoData demoData)
        {
            CwlbDemoData objData = _cobjData.FirstOrDefault(objSearch => objSearch.DateTimeValue.Date == demoData.DateTimeValue.Date);
            if (objData == null)
            {
                _cobjData.Add(demoData);
            }
            else
            {
                objData.IntegerValue = demoData.IntegerValue;
                objData.StringValue = demoData.StringValue;
            }
            return demoData;
        }

        /// <summary>
        /// Löscht ein Element.
        /// </summary>
        /// <param name="demoData">Datenelement, das gelöscht werden soll.</param>
        public void DeleteDemoData(CwlbDemoData demoData)
        {
            CwlbDemoData objData = _cobjData.FirstOrDefault(objSearch => objSearch.DateTimeValue.Date == demoData.DateTimeValue.Date);
            if (objData != null)
            {
                _cobjData.Remove(objData);
            }
        }
    }
}
