﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using WeroSoft.Samples.LearningWcfContract;

namespace WeroSoft.Samples.LearningWcfClient
{
    /// <summary>
    /// Implementiert den clientseitigen Democode.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Einstiegspunkt in die Anwendung.
        /// </summary>
        static void Main()
        {
            // Client definieren
            CwlbRichClient objClient = null;

            try
            {
                // Client initialisieren
                objClient = new CwlbRichClient(new NetTcpBinding(), new EndpointAddress(" net.tcp://localhost:4711/DemoService"));

                // ----------------------------------------------------------------------
                // Demo Parameterprüfung per Inspektion Teil I
                // ----------------------------------------------------------------------
                DemoParamCheckPart1(objClient);
                // ----------------------------------------------------------------------

                // ----------------------------------------------------------------------
                // Demo Parameterprüfung per Inspektion Teil II
                // ----------------------------------------------------------------------
                DemoParamCheckPart2(objClient);
                // ----------------------------------------------------------------------

                // ----------------------------------------------------------------------
                // Demo generelles Fehlermanagement
                // ----------------------------------------------------------------------
                try
                {
                    Console.WriteLine();
                    Console.WriteLine("Demo allgmeine Fehlerhandhabung.");
                    objClient.GetSpecialName("TEST 1", "TEST 2", 99);
                }
                catch (Exception excObject)
                {
                    Console.WriteLine(excObject.Message);
                }

                // Demo 2
                try
                {
                    Console.WriteLine();
                    Console.WriteLine("Demo Parameterprüfung per Inspektion (5)");
                    objClient.TestMethod("TEST 1", "TEST 2", "TEST 3", "TEST 4", "TEST 5", "TEST 6", "TEST 7", "TEST 8");
                }
                catch (FaultException excObject)
                {
                    Console.WriteLine(excObject.Reason);
                }
                catch (CommunicationException excObject)
                {
                    Console.WriteLine(excObject.Message);
                }
                catch (Exception excObject)
                {
                    Console.WriteLine(excObject.Message);
                }
                // ----------------------------------------------------------------------



                // ----------------------------------------------------------------------
                // Demo Lokalisierung
                // ----------------------------------------------------------------------
                Console.WriteLine();
                Console.WriteLine("Demo Lokalisierung");
                ShowServerSideLocalizedData(objClient, "en-US"); // Auf Server verfügbar
                ShowServerSideLocalizedData(objClient, "de-DE"); // Auf Server verfügbar
                ShowServerSideLocalizedData(objClient, "de-CH"); // Auf Server verfügbar
                ShowServerSideLocalizedData(objClient, "fr-CH"); // Auf Server verfügbar
                ShowServerSideLocalizedData(objClient, "fr-FR"); // Auf Server NICHT verfügbar
                // ----------------------------------------------------------------------

                // Client ordentlich schliessen
                objClient.Close();
                objClient = null;
            }
            catch (Exception exception)
            {
                Console.WriteLine(exception.Message);
            }
            finally
            {
                // Client ausserordentlich schliessen
                if (objClient != null && objClient.State == CommunicationState.Opened)
                {
                    objClient.Close();
                    objClient = null;
                }
                if (objClient != null && objClient.State == CommunicationState.Faulted)
                {
                    objClient.Close();
                    objClient = null;
                }
            }
        }

        /// <summary>
        /// Demonstriert die Fehlerprüfung mit der WCF.
        /// </summary>
        /// <param name="objClient">Client der verwendet werden soll.</param>
        private static void DemoParamCheckPart1(CwlbRichClient objClient)
        {
            // Demo 1
            try
            {
                Console.WriteLine("Demo Parameterprüfung per Inspektion (1)");
                objClient.GetSpecialName(null, null, 0);
            }
            catch (ArgumentNullException excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            catch (Exception excObject)
            {
                Console.WriteLine(excObject.Message);
            }

            // Demo 2
            try
            {
                Console.WriteLine();
                Console.WriteLine("Demo Parameterprüfung per Inspektion (2)");
                objClient.GetSpecialName("TEST", null, 0);
            }
            catch (ArgumentNullException excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            catch (Exception excObject)
            {
                Console.WriteLine(excObject.Message);
            }

            // Zum Abschluss noch ein korrekter Aufruf
            Console.WriteLine();
            Console.WriteLine("Demo Parameterprüfung per Inspektion (3)");
            objClient.GetSpecialName("TEST 1", "TEST 2", 15);
        }

        /// <summary>
        /// Demonstriert die Fehlerprüfung mit der WCF.
        /// </summary>
        /// <param name="objClient">Client der verwendet werden soll.</param>
        private static void DemoParamCheckPart2(CwlbRichClient objClient)
        {
            // Demo 1
            try
            {
                Console.WriteLine();
                Console.WriteLine("Demo Parameterprüfung per Inspektion (4)");
                objClient.TestMethod(null, null, null, null, null, null, null, null);
            }
            catch (ArgumentNullException excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            catch (Exception excObject)
            {
                Console.WriteLine(excObject.Message);
            }

            // Demo 2
            try
            {
                Console.WriteLine();
                Console.WriteLine("Demo Parameterprüfung per Inspektion (5)");
                objClient.TestMethod("TEST 1", "TEST 2", "TEST 3", "TEST 4", "TEST 5", "TEST 6", "TEST 7", null);
            }
            catch (ArgumentNullException excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            catch (Exception excObject)
            {
                Console.WriteLine(excObject.Message);
            }
        }

        /// <summary>
        /// Ruft die serverseitige Methode für die Demo unter EInstellung der eigenen Sprache auf.
        /// </summary>
        /// <param name="objClient">Client der Verwendet werden soll.</param>
        /// <param name="strCultureName">Name der Kultur, die für die Demo verwendet werden soll.</param>
        private static void ShowServerSideLocalizedData(CwlbRichClient objClient, string strCultureName)
        {
            Thread.CurrentThread.CurrentCulture = new CultureInfo(strCultureName);
            Thread.CurrentThread.CurrentUICulture = new CultureInfo(strCultureName);
            Console.WriteLine(objClient.GetLanguageData(6523445.123456789, DateTime.Now));
        }
    }
}
