﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.ServiceModel.Dispatcher;
using System.Text;
using System.Threading.Tasks;

namespace WeroSoft.Samples.LearningWcfContract
{
    /// <summary>
    /// Definiert eine Basisklasse für die einfachere Implementierung einer Parameterprüfung.
    /// </summary>
    public abstract class AwlbOperationBehaviorAttribute : Attribute, IOperationBehavior
    {
        public virtual void AddBindingParameters(OperationDescription objOperationDescription, BindingParameterCollection objBindingParameters)
        {
            // Diese Methode wird im Beispiel nicht benutzt und ist nur 
            // notwendig um die Schnittstellenimplementierung vorzunehmen
        }

        /// <summary>
        /// Definiert das Verhalten für einen Operation-Dispatcher auf der Clientseite.
        /// </summary>
        /// <param name="objOperationDescription">Beschreibung der Oparation, die konfiguriert wird.</param>
        /// <param name="objClientOperation">Operation-Dispatcher der konfiguriert wird.</param>
        public abstract void ApplyClientBehavior(OperationDescription objOperationDescription, ClientOperation objClientOperation);

        /// <summary>
        /// Definiert das Verhalten für einen Operation-Dispatcher auf der Serverseite.
        /// </summary>
        /// <param name="objOperationDescription">Beschreibung der Oparation, die konfiguriert wird.</param>
        /// <param name="objDispatchOperation">Operation-Dispatcher der konfiguriert wird.</param>
        public abstract void ApplyDispatchBehavior(OperationDescription objOperationDescription, DispatchOperation objDispatchOperation);

        public virtual void Validate(OperationDescription objOperationDescription)
        {
            // Diese Methode wird im Beispiel nicht benutzt und ist nur 
            // notwendig um die Schnittstellenimplementierung vorzunehmen
        }
    }
}
