﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel.Description;
using System.ServiceModel.Dispatcher;
using System.Text;
using System.Threading.Tasks;
using WeroSoft.Samples.LearningWcfContract;

namespace WeroSoft.Samples.LearningWcfContract
{
    /// <summary>
    /// Implementiert das Attribut für die Nullprüfung von Parametern von Operationen.
    /// </summary>
    public sealed class AwlbParameterNotNullAttribute : AwlbOperationBehaviorAttribute 
    {
        /// <summary>
        /// Definiert die Parameter, die auf einen Null-Wert geprüft werden sollen.
        /// </summary>
        public string[] ParameterNotNull { get; private set; }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse AwlbNotNullOperationBehaviorAttribute.
        /// </summary>
        /// <param name="astrParameterNotNull">Definiert die Parameter der Operation, die nicht einen Null-Wert aufgweisen dürfen.</param>
        public AwlbParameterNotNullAttribute(string[] astrParameterNotNull)
        {
            ParameterNotNull = astrParameterNotNull;
        }

        /// <summary>
        /// Definiert das Verhalten für einen Operation-Dispatcher auf der Clientseite.
        /// </summary>
        /// <param name="objOperationDescription">Beschreibung der Oparation, die konfiguriert wird.</param>
        /// <param name="objClientOperation">Operation-Dispatcher der konfiguriert wird.</param>
        public override void ApplyClientBehavior(OperationDescription objOperationDescription, ClientOperation objClientOperation)
        {
            objClientOperation.ClientParameterInspectors.Add(
                new CwlbParameterNotNullInspector(objOperationDescription.SyncMethod, ParameterNotNull));
        }

        /// <summary>
        /// Definiert das Verhalten für einen Operation-Dispatcher auf der Serverseite.
        /// </summary>
        /// <param name="objOperationDescription">Beschreibung der Oparation, die konfiguriert wird.</param>
        /// <param name="objDispatchOperation">Operation-Dispatcher der konfiguriert wird.</param>
        public override void ApplyDispatchBehavior(OperationDescription objOperationDescription, DispatchOperation objDispatchOperation)
        {
            objDispatchOperation.ParameterInspectors.Add(
                new CwlbParameterNotNullInspector(objOperationDescription.SyncMethod, ParameterNotNull));
        }
    }
}
