﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Dispatcher;
using System.Text;
using System.Threading.Tasks;
using WeroSoft.Samples.LearningWcfContract;

namespace WeroSoft.Samples.LearningWcfImplementation
{
    /// <summary>
    /// Implementiert einen allgemeingültige Fehlerbehandlung auf der Serverseite.
    /// </summary>
    public class CwlbErrorHandler : IErrorHandler
    {
        /// <summary>
        /// Aktiviert die Verarbeitung und gibt einen Wert zurück, der angibt, ob der Verteiler die Sitzung und den Instanzkontext in bestimmten Fällen abbricht. 
        /// </summary>
        /// <param name="excObject">Ausnahme die von der WCF gefangen wurde.</param>
        /// <returns>
        /// True, falls die WCF die Sitzung (sofern vorhanden) und den Instanzkontext nicht abbrechen soll, 
        /// falls der Instanzkontext nicht Single lautet; andernfalls false. Die Standardeinstellung ist false.</returns>
        public bool HandleError(Exception excObject)
        {
            Console.WriteLine(excObject.Message);
            return true;
        }

        /// <summary>
        /// Aktiviert die Erstellung einer benutzerdefinierten FaultException<TDetail>, die während einer Dienstmethode von einer Ausnahme zurückgegeben wird. 
        /// </summary>
        /// <param name="excObject">Ausnahme die von der WCF gefangen wurde.</param>
        /// <param name="objVersion">Die Version der SOAP-Nachricht.</param>
        /// <param name="objFault">Das Nachrichtenobjekt, das dem Client oder bei Duplexkommunikation dem Dienst zurückgegeben wird.</param>
        public void ProvideFault(Exception excObject, MessageVersion objVersion, ref Message objFault)
        {
            // Hat der Dienst schon eine korrekte Nachricht erzeugt.
            if (objFault == null)
            {
                if (excObject is InvalidOperationException)
                {
                    FaultException<CwlbDemoFault> objFaultException = new FaultException<CwlbDemoFault>(
                        new CwlbDemoFault { Message = "Dienstfehler gefangen" },
                        new FaultReason(excObject.Message));
                    MessageFault objMessageFault = objFaultException.CreateMessageFault();
                    objFault = Message.CreateMessage(objVersion, objMessageFault, objFaultException.Action);
                }
                else
                {
                    FaultException<CwlbDemoFault> objFaultException = new FaultException<CwlbDemoFault>(
                        new CwlbDemoFault { Message = "Allgemeiner Dienstfehler gefangen" },
                        new FaultReason(excObject.Message));
                    MessageFault objMessageFault = objFaultException.CreateMessageFault();
                    objFault = Message.CreateMessage(objVersion, objMessageFault, objFaultException.Action);
                }
            }
        }
    }
}
