﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.Security.Cryptography.X509Certificates;
using System.Net;
using System.Net.Security;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert den Client.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Einsprungpunkt in das Programm.
        /// </summary>
        static void Main(string[] astrArguments)
        {
            IwlbDemoService objProxy = null;
            CwlbArgumentHelper objArgumentHelper = new CwlbArgumentHelper();
            objArgumentHelper.Parse(astrArguments);
            if (objArgumentHelper.Help)
            {
                objArgumentHelper.ShowHelp();
                return;
            }
            else
            {
                objArgumentHelper.ShowConfiguration();
            }

            try
            {
                // Erstellen der Clientfactory für die notwendige Konfiguration des Proxy
                ChannelFactory<IwlbDemoService> objFactory = new ChannelFactory<IwlbDemoService>(
                    objArgumentHelper.CreateBinding(),
                    new EndpointAddress(objArgumentHelper.CreateAddress()));

                // Wenn ein Zertifikat verwendet werden soll, dieses Konfigurieren und eine 
                // Validierungsmethode registrieren. Das ist nur nötig, damit die Fehler des Zertifikats
                // erfahren würden.
                if (objArgumentHelper.ClientCredentialType == HttpClientCredentialType.Certificate)
                {
                    objFactory.Credentials.ClientCertificate.SetCertificate(StoreLocation.LocalMachine, StoreName.My, X509FindType.FindByThumbprint, objArgumentHelper.Thumbprint );
                    ServicePointManager.ServerCertificateValidationCallback += RemoteCertificateValidate;
                }
                objProxy = objFactory.CreateChannel();

                // Aufruf ohne Fehler
                Console.WriteLine();
                Console.WriteLine("   Aufruf des Servers ...");
                CwlbDemoData objData = new CwlbDemoData { DateTimeValue = DateTime.Now, StringValue = "Anfrage" };
                CwlbDemoData[] aobjData = objProxy.GetDemoData(objData, 20);
                foreach (CwlbDemoData objItem in aobjData)
                {
                    Console.WriteLine("   " + objItem);
                }
            }
            catch (FaultException<CwlbFault> excObject)
            {
                // Verarbeitung des Serverfehlers
                Console.WriteLine("Eigener Fehler gefangen:");
                Console.WriteLine(string.Format("   Fehlertext:          {0}", excObject.Message));
                Console.WriteLine(string.Format("   Fehlergrund:         {0}", excObject.Reason));
                Console.WriteLine(string.Format("   Fehlercode:          {0}", excObject.Code.Name));
                Console.WriteLine(string.Format("   Text Fehlerdetail:   {0}", excObject.Detail.Message));
                Console.WriteLine(string.Format("   Code Fehlerdetail:   {0}", excObject.Detail.Code));
                Console.WriteLine(string.Format("   Datum Fehlerdetail:  {0}", excObject.Detail.Created));
            }
            catch (CommunicationException excObject)
            {
                // Verarbeitung einer allgmeinen Kommunikationsausnahme
                Console.WriteLine(excObject.Message);
            }
            catch (Exception excObject)
            {
                // Verarbeitung aller andern Ausnahmen
                Console.WriteLine(excObject.Message);
            }
            finally
            {
                // Beenden des Clients im Normalfall
                if (objProxy != null && ((IChannel)objProxy).State == CommunicationState.Opened)
                {
                    ((IChannel)objProxy).Close();
                }
                // Beenden des Clients im Ausnahmefall
                if (objProxy != null && ((IChannel)objProxy).State == CommunicationState.Faulted)
                {
                    ((IChannel)objProxy).Abort();
                }
            }
        }

        /// <summary>
        /// Ereignisbehandlung beim Validieren des entfernten Zertifikates.
        /// </summary>
        /// <param name="objSender">Absender der Validierungsanfrage.</param>
        /// <param name="objCertificate">Zertifikate das verwendet werden soll.</param>
        /// <param name="objChain">Zertifikate die geprüft werden können.</param>
        /// <param name="enmError">Aufgetretener Fehler.</param>
        /// <returns>True, wenn das Zertifikat akzeptiert wird, anderfalls False.</returns>
        private static bool RemoteCertificateValidate(object objSender, X509Certificate objCertificate, X509Chain objChain, SslPolicyErrors enmError)
        {
            // Bestätigung der Zertifikatsverwendung
            if (enmError != SslPolicyErrors.None)
            {
                Console.WriteLine("Fehler bei der Verwendung des Zertifikates. Definieren Sie mit der option -m den Maschinennamen.");
            }
            return true;
        }

    }
}
