﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.Security.Cryptography.X509Certificates;
using System.Net;
using System.Net.Security;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert den Client.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Einsprungpunkt in das Programm.
        /// </summary>
        static void Main(string[] astrArguments)
        {
            IwlbDemoService objProxy = null;

            try
            {
                // Herstellen der Bindung für die Nachrichtenebene
                WSHttpBinding objBinding = new WSHttpBinding(SecurityMode.Message);
                objBinding.Security.Message.ClientCredentialType = MessageCredentialType.Windows;
                objBinding.Security.Message.EstablishSecurityContext = true;
                objBinding.Security.Message.AlgorithmSuite =
                    System.ServiceModel.Security.SecurityAlgorithmSuite.Basic256Sha256;

                // Erstellen der Clientfactory
                ChannelFactory<IwlbDemoService> objFactory = new ChannelFactory<IwlbDemoService>(
                    objBinding, new EndpointAddress("http://localhost:4712/DemoService"));
                    
                objProxy = objFactory.CreateChannel();

                // Aufruf ohne Fehler
                Console.WriteLine();
                Console.WriteLine("   Aufruf des Servers ...");
                CwlbDemoData objData = new CwlbDemoData { DateTimeValue = DateTime.Now, StringValue = "Anfrage" };
                CwlbDemoData[] aobjData = objProxy.GetDemoData(objData, 20);
                aobjData = objProxy.GetDemoData(objData, 20);
                foreach (CwlbDemoData objItem in aobjData)
                {
                    Console.WriteLine("   " + objItem);
                }
            }
            catch (FaultException<CwlbFault> excObject)
            {
                // Verarbeitung des Serverfehlers
                Console.WriteLine("Eigener Fehler gefangen:");
                Console.WriteLine(string.Format("   Fehlertext:          {0}", excObject.Message));
                Console.WriteLine(string.Format("   Fehlergrund:         {0}", excObject.Reason));
                Console.WriteLine(string.Format("   Fehlercode:          {0}", excObject.Code.Name));
                Console.WriteLine(string.Format("   Text Fehlerdetail:   {0}", excObject.Detail.Message));
                Console.WriteLine(string.Format("   Code Fehlerdetail:   {0}", excObject.Detail.Code));
                Console.WriteLine(string.Format("   Datum Fehlerdetail:  {0}", excObject.Detail.Created));
            }
            catch (CommunicationException excObject)
            {
                // Verarbeitung einer allgmeinen Kommunikationsausnahme
                Console.WriteLine(excObject.Message);
            }
            catch (Exception excObject)
            {
                // Verarbeitung aller andern Ausnahmen
                Console.WriteLine(excObject.Message);
            }
            finally
            {
                // Beenden des Clients im Normalfall
                if (objProxy != null && ((IChannel)objProxy).State == CommunicationState.Opened)
                {
                    ((IChannel)objProxy).Close();
                }
                // Beenden des Clients im Ausnahmefall
                if (objProxy != null && ((IChannel)objProxy).State == CommunicationState.Faulted)
                {
                    ((IChannel)objProxy).Abort();
                }
            }
        }
    }
}
