﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.Security.Cryptography.X509Certificates;
using System.Net;
using System.Net.Security;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementiert den Client.
    /// </summary>
    class Program
    {
        /// <summary>
        /// Einsprungpunkt in das Programm.
        /// </summary>
        static void Main(string[] astrArguments)
        {
            Console.WriteLine();
            Console.WriteLine("Zugriff mit Administrator");
            Console.WriteLine("-------------------------");
            PerformDemoCalls("Admin", "adminPassword");

            Console.WriteLine();
            Console.WriteLine("Zugriff mit ungültigen Userdaten");
            Console.WriteLine("--------------------------------");
            PerformDemoCalls("Admin", "ungültiges Passwort");

            Console.WriteLine();
            Console.WriteLine("Zugriff mit TestUser");
            Console.WriteLine("--------------------");
            PerformDemoCalls("testUser", "password");
        }

        /// <summary>
        /// Führt den Zugriff auf den Dienst mit einer gemischten Sicherheitskonfiguration durch
        /// </summary>
        /// <param name="strUserName">Zu verwendender Benutzername.</param>
        /// <param name="strPassword">Zu verwendendes Passwort</param>
        static void PerformDemoCalls(string strUserName, string strPassword)
        {
            ChannelFactory<IwlbDemoService> objFactory = null;
            IwlbDemoService objSmartClient = null;

            try
            {
                // Bindung erstellen
                WSHttpBinding objBinding = new WSHttpBinding(SecurityMode.TransportWithMessageCredential);
                objBinding.Security.Transport.ClientCredentialType = HttpClientCredentialType.Certificate;
                objBinding.Security.Message.ClientCredentialType = MessageCredentialType.UserName;

                // Erstellen der Client-Factory
                objFactory = new ChannelFactory<IwlbDemoService>(
                    objBinding, new EndpointAddress("https://localhost:4712/DemoService"));

                // Zertifikat konfigurieren und Kontoangaben definieren
                objFactory.Credentials.ClientCertificate.SetCertificate(
                    StoreLocation.LocalMachine,
                    StoreName.My,
                    X509FindType.FindByThumbprint,
                    "7cb5cbdab4ce743dcff4e2b232f0588ede2a13cd");
                objFactory.Credentials.UserName.UserName = strUserName;
                objFactory.Credentials.UserName.Password = strPassword;

                objSmartClient = objFactory.CreateChannel();

                Console.WriteLine(objSmartClient.MethodAny());
                Console.WriteLine(objSmartClient.MethodMember());
                Console.WriteLine(objSmartClient.MethodContributor());
                Console.WriteLine(objSmartClient.MethodAdministration());
                Console.WriteLine(objSmartClient.MethodImpersonatedDeclarative());
                Console.WriteLine(objSmartClient.MethodImpersonatedImperative());
            }
            catch (CommunicationException excObject)
            {
                // Verarbeitung einer allgemeinen Kommunikationsausnahme
                if (excObject.InnerException != null)
                {
                    Console.WriteLine(excObject.InnerException.Message);
                }
                else
                {
                    Console.WriteLine(excObject.Message);
                }
            }
            catch (Exception excObject)
            {
                // Verarbeitung aller andern Ausnahmen
                Console.WriteLine(excObject.Message);
            }
            finally
            {
                // Beenden des Clients im Normalfall
                if (objSmartClient != null && ((IChannel)objSmartClient).State == CommunicationState.Opened)
                {
                    ((IChannel)objSmartClient).Close();
                }
                // Beenden des Clients im Ausnahmefall
                if (objSmartClient != null && ((IChannel)objSmartClient).State == CommunicationState.Faulted)
                {
                    ((IChannel)objSmartClient).Abort();
                }
            }
        }
    }
}
