﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.Security.Principal;
using System.Threading;
using System.IdentityModel.Policy;
using System.IdentityModel.Claims;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Definiert eine eigene Autorisierungsstelle.
    /// </summary>
    public class CwlbAuthorizationPolicy : IAuthorizationPolicy
    {
        /// <summary>
        /// Identifikation der Autorisierungsstelle.
        /// </summary>
        private string _strIdentifier;

        /// <summary>
        /// Liefert den Aussteller des Claims.
        /// </summary>
        public System.IdentityModel.Claims.ClaimSet Issuer { get; private set; }

        /// <summary>
        /// Liefert die Identifikation der Autorisierungsstelle.
        /// </summary>
        public string Id
        {
            get { return _strIdentifier; }
        }

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse CwlbAuthorizationPolicy.
        /// </summary>
        public CwlbAuthorizationPolicy()
        {
            _strIdentifier = Guid.NewGuid().ToString();
            Issuer = ClaimSet.System;
        }

        /// <summary>
        /// Prüft die Autorisierung einer Anforderung.
        /// </summary>
        /// <param name="objEvaluationContext">Kontext der Autorisierung.</param>
        /// <param name="objState">Aktueller Status der Autorisierung.</param>
        /// <returns>True, wenn die Autorisierung abgeschlossen wurde.</returns>
        public bool Evaluate(EvaluationContext objEvaluationContext, ref object objState)
        {
            // Überprüfen Sie hier die Identität
            List<IIdentity> cobjIdentities = objEvaluationContext.Properties["Identities"] as List<IIdentity>;
            IIdentity objCurrentIdentity = cobjIdentities[0] as IIdentity;

            // Hier können Sie zusätzlichen Prüfcode für die Entität einbringen
            // ...

            // Generische identität herstellen
            GenericIdentity objIdentity = new GenericIdentity(objCurrentIdentity.Name, "weroSoft");

            // Generischer Prinzipal herstellen
            string[] astrRoles;
            if (string.Compare(objIdentity.Name, "admin", true) == 0)
            {
                astrRoles = new string[]
                { 
                    GwlbLearningWcf.Members,
                    GwlbLearningWcf.Contributers,
                    GwlbLearningWcf.Administrators
                };
            }
            else if (string.Compare(objIdentity.Name, "testuser", true) == 0)
            {
                astrRoles = new string[]
                { 
                    GwlbLearningWcf.Members,
                    GwlbLearningWcf.Contributers
                };
            }
            else
            {
                astrRoles = new string[] { };
            }

            // Herstellen des generischen Prinzipals und diesen an den Sicherheitskontext übergeben
            GenericPrincipal objGenericPrinzipal = new GenericPrincipal(objIdentity, astrRoles);
            objEvaluationContext.Properties["Principal"] = objGenericPrinzipal;

            return true;
        }
    }
}
