﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using System.Runtime.CompilerServices;
using System.Security.Permissions;
using System.Security.Principal;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Implementierung des Dienstes IwlbDemoService.
    /// </summary>
    [AwlbServiceErrorHandler(typeof(CwlbErrorHandler))]
    public class CwlbDemoService : IwlbDemoService
    {
        /// <summary>
        /// Definiert eine Testmethode, die nur von Administratoren verwendet werden kann.
        /// </summary>
        /// <returns>Liefert eine Zeichenfolge, mit dem Methodennamen und weiteren Aufrufdaten.</returns>
        public string MethodAdministration()
        {
            // Imperative Prüfung der Sicherheitsrichtline für Codezugriff
            PrincipalPermission objPermission = new PrincipalPermission("Admin", null);
            objPermission.Demand();
            return GetCallerInfo();
        }

        /// <summary>
        /// Definiert eine Testmethode, die von Benutzern der Mitarbeitergruppe verwendet werden kann.
        /// </summary>
        /// <returns>Liefert eine Zeichenfolge, mit dem Methodennamen und weiteren Aufrufdaten.</returns>
        [PrincipalPermission(SecurityAction.Demand, Role = GwlbLearningWcf.Contributers)]
        [PrincipalPermission(SecurityAction.Demand, Role = GwlbLearningWcf.Special)]
        public string MethodContributor()
        {
            return GetCallerInfo();
        }

        /// <summary>
        /// Definiert eine Testmethode, die von Benutzern der Mitgliedergruppe verwendet werden kann.
        /// </summary>
        /// <returns>Liefert eine Zeichenfolge, mit dem Methodennamen und weiteren Aufrufdaten.</returns>
        [PrincipalPermission(SecurityAction.Demand, Role = GwlbLearningWcf.Members)]
        public string MethodMember()
        {
            return GetCallerInfo();
        }

        /// <summary>
        /// Definiert eine Testmethode, die von allen gültigen Benutzern verwendet werden kann.
        /// </summary>
        /// <returns>Liefert eine Zeichenfolge, mit dem Methodennamen und weiteren Aufrufdaten.</returns>
        public string MethodAny()
        {
            return GetCallerInfo();
        }

        /// <summary>
        /// Methode mit einem Identitätswechsel.
        /// </summary>
        /// <returns></returns>
        //[OperationBehavior(Impersonation = ImpersonationOption.Required)]
        public string MethodImpersonatedDeclarative()
        {
            return GetCallerInfo();
        }

        /// <summary>
        /// Methode mit einem imperativen Identitätswechsel.
        /// </summary>
        /// <returns></returns>
        public string MethodImpersonatedImperative()
        {
            WindowsIdentity callerWindowsIdentity = ServiceSecurityContext.Current.WindowsIdentity;
            if (callerWindowsIdentity == null)
            {
                throw new InvalidOperationException("Der Identitätswechsel ist nicht möglich, da keine Benutzerinformationen vorliegen.");
            }
            using (WindowsImpersonationContext objImpersonationContext = callerWindowsIdentity.Impersonate())
            {
                // Hier mit der neuen Identität zugreifen
                return GetCallerInfo();
            }
        }


        /// <summary>
        /// Erzeugt eine Zeichenfolge, für die generelle Rückgabe der Dienstmethoden.
        /// </summary>
        /// <param name="strMemberName">Name der Methode, die diese Methode aufgerufen hat.</param>
        /// <returns>Formatierte Zeichenfolge mit Zeitstempel.</returns>
        private string GetCallerInfo([CallerMemberName]string strMemberName = null)
        {
            string strMessage = string.Format("{0} Called {1}", 
                DateTime.UtcNow.ToString("HH:mm:ss.fff"),
                strMemberName);
            Console.WriteLine(strMessage);
            return strMessage;
        }
    }
}
