﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Principal;
using System.Security.Permissions;
using System.Reflection;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Threading;
using System.Threading.Tasks;

namespace WeroSoft.Samples.SecurityDemo
{
    /// <summary>
    /// Demoprogramm
    /// </summary>
    class Program
    {
        // Deklaration der Interop-Methoden mit WIN32 für Identitätswechsel
        // Notwendige Konstanten
        private const int LOGON32_PROVIDER_DEFAULT = 0;
        private const int LOGON32_LOGON_INTERACTIVE = 2;

        // Anmeldemethode
        [DllImport("advapi32.dll", SetLastError = true)]
        private static extern bool LogonUser(
            string lpszUsername,
            string lpszDomain,
            string lpszPassword,
            int dwLogonType,
            int dwLogonProvider,
            ref IntPtr phToken);

        // CloseHandle-Methode
        [DllImport("kernel32.dll")]
        private extern static bool CloseHandle(IntPtr handle);

        // Notwendige Variablen für die Identitätswechsel
        private static IntPtr _tokenIdentity = new IntPtr(0);
        private static WindowsIdentity _idNewUser = null;

        /// <summary>
        /// Einsprungpunkt in das Programm.
        /// </summary>
        static void Main()
        {
            try
            {
                Console.WriteLine();
                Console.WriteLine("Für die erfolgreiche Durchführung dieser Demo muss");
                Console.WriteLine("ein Benutzer mit folgenden Daten lokal im System");
                Console.WriteLine("angelegt werden:");
                Console.WriteLine("   Benutzername: . . . .  'TestUser'");
                Console.WriteLine("   Kennwort: . . . . . .  'TestUser'");
                Console.WriteLine("   Gruppenmitgliedschaft: 'Member'");
                Console.WriteLine();
                Console.WriteLine("Achten Sie darauf, dass bei einer Neudefinition des ");
                Console.WriteLine("Bnutzerkontos der Kennwortwechsel deaktiviert wurde.");
                Console.WriteLine();
                Console.WriteLine("Weiter mit einer Taste ...");
                Console.ReadKey(false);


                Console.WriteLine();
                Console.WriteLine("Demo aktueller User");
                Console.WriteLine("*******************");
                ShowCurrentIdentity();

                Console.WriteLine();
                Console.WriteLine("Demo TestUser");
                Console.WriteLine("*************");
                DemoImpersonationWindows();

            }
            catch (Exception excObejct)
            {
                Console.WriteLine(excObejct.Message);
            }

            try
            {
                Console.WriteLine();
                Console.WriteLine();
                Console.WriteLine("Generischer Prinzipal");
                Console.WriteLine("*********************");
                DemoGenericPrincipal();
            }
            catch (Exception excObejct)
            {
                Console.WriteLine(excObejct.Message);
            }
        }

        /// <summary>
        /// Demonstriert die Verwendung von Identitätswechsel und rollenbasierter Sicherheit mit einem Windows-Prinzipal.
        /// </summary>
        private static void DemoImpersonationWindows()
        {
            // Identitäswechsel vorbereiten
            Console.WriteLine("Impersonate TestUser");
            bool bLogon = LogonUser("TestUser", ".", "TestUser", LOGON32_LOGON_INTERACTIVE, LOGON32_PROVIDER_DEFAULT, ref _tokenIdentity);
            // Rückgabewert prüfen
            if (!bLogon)
            {
                throw new Win32Exception(Marshal.GetLastWin32Error());
            }
            try
            {
                // Identitätswechsel durchführen
                _idNewUser = new WindowsIdentity(_tokenIdentity);
                using (WindowsImpersonationContext _objImpersonatedUser = _idNewUser.Impersonate())
                {
                    // Daten der aktuellen Identität ausgeben
                    ShowCurrentIdentity();

                    // Demo prinzip
                    PlayWithWindowsPrincipal();
                    DemoRoleBasedSecurity();
                }
            }
            catch (Exception excObject)
            {
                Console.WriteLine(excObject.Message);
            }
            finally
            {
                // Identitätswechsel rückgängig machen
                if (_tokenIdentity != IntPtr.Zero)
                {
                    CloseHandle(_tokenIdentity);
                }
            }
        }

        /// <summary>
        /// Zeigt die Daten der aktuellen Identität
        /// </summary>
        private static void ShowCurrentIdentity()
        {
            Console.WriteLine("Der aktuelle Benutzer ist '{0}'", Environment.UserName);
            IIdentity objCurrentIdentity = WindowsIdentity.GetCurrent();
            Console.WriteLine("Der aktuelle Benutzer ist '{0}'", objCurrentIdentity.Name);
            Console.WriteLine("Der aktuelle Benutzer ist authentifiziert '{0}'", objCurrentIdentity.IsAuthenticated);
            Console.WriteLine("Der Herausgeber der Identität ist '{0}'", objCurrentIdentity.AuthenticationType);

            Console.WriteLine();
            WindowsIdentity objCurrentWindowsIdentity = WindowsIdentity.GetCurrent();
            Console.WriteLine("Name . . . . . . . . . . '{0}'", objCurrentWindowsIdentity.Name);
            Console.WriteLine("Besitzer . . . . . . . . '{0}'", objCurrentWindowsIdentity.Owner);
            Console.WriteLine("User . . . . . . . . . . '{0}'", objCurrentWindowsIdentity.User);
            Console.WriteLine("Systemuser . . . . . . . '{0}'", objCurrentWindowsIdentity.IsSystem);
            Console.WriteLine("Gast . . . . . . . . . . '{0}'", objCurrentWindowsIdentity.IsGuest);
            Console.WriteLine("Authentifiziert  . . . . '{0}'", objCurrentWindowsIdentity.IsAuthenticated);
            Console.WriteLine("Anonym . . . . . . . . . '{0}'", objCurrentWindowsIdentity.IsAnonymous);
            Console.WriteLine("Gruppen:");
            foreach (var objGroup in objCurrentWindowsIdentity.Groups)
            {
                Console.WriteLine("   - {0}; SID: {1}",
                    objGroup.Translate(typeof(System.Security.Principal.NTAccount)),
                    objGroup.Value);
            }
        }

        /// <summary>
        /// Demonstration der rollebasierte Sicherheit.
        /// </summary>
        private static void DemoRoleBasedSecurity()
        {
            // Definieren des Prinzipals auf Domänenstufe
            Console.WriteLine(new string('-', 40));
            SetPrinzipalAppDomain(GwlbLearningWcf.Members);
            Console.WriteLine(new string('-', 40));
            SetPrinzipalAppDomain(GwlbLearningWcf.Administrators);

            // Deklarative, rollenbasierte Methoden aufrufen (Administrator)
            try
            {
                Console.WriteLine(new string('-', 40));
                Console.WriteLine("RoleBaseSecurityDeclarativeAdministrators");
                RoleBaseSecurityDeclarativeAdministrators();
            }
            catch (Exception excObject)
            {
                WriteNotSuccessfull("RoleBaseSecurityDeclarativeAdministrators", excObject);
            }

            // Deklarative, rollenbasierte Methoden aufrufen (Member)
            try
            {
                Console.WriteLine(new string('-', 40));
                Console.WriteLine("RoleBaseSecurityDeclarativeMember");
                RoleBaseSecurityDeclarativeMember();
            }
            catch (Exception excObject)
            {
                WriteNotSuccessfull("RoleBaseSecurityDeclarativeMember", excObject);
            }

            // Imperative, rollenbasierte Methoden aufrufen (beide Gruppen)
            try
            {
                Console.WriteLine(new string('-', 40));
                RoleBaseSecurityImperative(GwlbLearningWcf.Members);
                Console.WriteLine(new string('-', 40));
                RoleBaseSecurityImperative(GwlbLearningWcf.Administrators);
            }
            catch (Exception excObject)
            {
                WriteNotSuccessfull("RoleBaseSecurityImperative", excObject);
            }
        }

        /// <summary>
        /// Ausgeben von Beziehungen zu Gruppen (Windows-Prinzipal)
        /// </summary>
        private static void PlayWithWindowsPrincipal()
        {
            Console.WriteLine();
            Console.WriteLine("Prinzipal Demo");

            // Aktuellen Prinzip auf der Domain festlegen
            AppDomain.CurrentDomain.SetPrincipalPolicy(PrincipalPolicy.WindowsPrincipal);

            // Aktuellen Prinzipal abfragen
            WindowsPrincipal objPrincipal = (WindowsPrincipal)Thread.CurrentPrincipal;
            Console.WriteLine("{0} ist in folgenden Rollen: ", objPrincipal.Identity.Name.ToString());
            Array aobjWindowsBuiltInroles = Enum.GetValues(typeof(WindowsBuiltInRole));
            foreach (var roleName in aobjWindowsBuiltInroles)
            {
                try
                {
                    // Die Rolle umsetzen
                    Console.WriteLine("{0}: {1} - RID {2}", 
                        roleName, 
                        objPrincipal.IsInRole((WindowsBuiltInRole)roleName),
                        ((int)roleName).ToString("x"));

                }
                catch (Exception)
                {
                    Console.WriteLine("{0}: Konnte die Rolle nicht in Erfahrung bringen.",
                        roleName);
                }
            }

            // Abfragen der Rolle über eine Zeichenfolge
            Console.WriteLine("{0} ist in der Gruppe Administratoren: {1}", 
                objPrincipal.Identity.Name,
                objPrincipal.IsInRole(GwlbLearningWcf.Administrators));
            Console.WriteLine("{0} ist in der Gruppe User: {1}",
                objPrincipal.Identity.Name,
                objPrincipal.IsInRole(GwlbLearningWcf.Users));
            Console.WriteLine("{0} ist in der Gruppe Member: {1}",
                objPrincipal.Identity.Name,
                objPrincipal.IsInRole(GwlbLearningWcf.Members));

            // Abfragen der Rolle über die Aufzählung
            Console.WriteLine("{0} ist in der Gruppe {1}: {2}",
                objPrincipal.Identity.Name, WindowsBuiltInRole.Administrator.ToString(), 
                objPrincipal.IsInRole(WindowsBuiltInRole.Administrator));
            Console.WriteLine("{0} ist in der Gruppe {1}: {2}",
                objPrincipal.Identity.Name, WindowsBuiltInRole.User.ToString(),
                objPrincipal.IsInRole(WindowsBuiltInRole.User));

            // Abfragen der Rolle über den Code
            Console.WriteLine("{0} ist in der Gruppe {1}: {2}",
                objPrincipal.Identity.Name,
                WindowsBuiltInRole.Administrator.ToString(), objPrincipal.IsInRole(0x220));
            Console.WriteLine("{0} ist in der Gruppe {1}: {2}",
                objPrincipal.Identity.Name, WindowsBuiltInRole.User.ToString(),
                objPrincipal.IsInRole(0x221));

            // Abfragen über die SID
            SecurityIdentifier objSid = new SecurityIdentifier(
                WellKnownSidType.BuiltinAdministratorsSid, null);
            Console.WriteLine("{0} ist in der Gruppe mit SID {1}: {2}", 
                objPrincipal.Identity.Name, objSid.Value,
                objPrincipal.IsInRole(objSid));
        }

        /// <summary>
        /// Demonstriert die Verwendung von rollenbasierter Sicherheit mit einem generischen Prinzipal.
        /// </summary>
        private static void DemoGenericPrincipal()
        {
            // Generische Identität herstellen
            GenericIdentity objGenericIdentity = new GenericIdentity("MyIdentity", "weroSoft");

            // Generischer Prinzipal herstellen
            String[] astrRoles = { GwlbLearningWcf.Members, "IrgendeineRolle" };
            GenericPrincipal objGenericPrinzipal = new GenericPrincipal(objGenericIdentity, astrRoles);

            // Generischer Prinzipal mit dem Thread verbinden 
            Thread.CurrentPrincipal = objGenericPrinzipal;

            // Werte ausgeben
            Console.WriteLine("Der Name des generischen Prinzipals ist: {0}", objGenericPrinzipal.Identity.Name);
            Console.WriteLine("Der Prinzipal ist authentifiziert: {0}", objGenericPrinzipal.Identity.IsAuthenticated);
            Console.WriteLine("Ist Mitglied in Gruppe: {0}", objGenericPrinzipal.IsInRole(GwlbLearningWcf.Members));

            DemoRoleBasedSecurity();
        }

        /// <summary>
        /// Principal auf Domaänenstufe.
        /// </summary>
        /// <param name="strGroupName">Gruppennamen</param>
        private static void SetPrinzipalAppDomain(string strGroupName)
        {
            try
            {
                Console.WriteLine("Anfrage für Prinzipal: '{0}'", strGroupName);
                AppDomain.CurrentDomain.SetPrincipalPolicy(PrincipalPolicy.WindowsPrincipal);
                PrincipalPermission principalPerm = new PrincipalPermission(null, strGroupName);
                principalPerm.Demand();
                Console.WriteLine("Erfolgreich.");
            }
            catch (Exception excObject)
            {
                Console.WriteLine("NICHT erfolgreich. {0}{1}", Environment.NewLine, excObject.Message);
            }
        }

        /// <summary>
        /// Deklartive rollenbasierte Sicherheit (Gruppe Administrators).
        /// </summary>
        [PrincipalPermission(SecurityAction.Demand, Role = GwlbLearningWcf.Administrators)]
        private static void RoleBaseSecurityDeclarativeAdministrators()
        {
            Write(MethodInfo.GetCurrentMethod(), GwlbLearningWcf.Administrators);
        }

        /// <summary>
        /// Deklartive rollenbasierte Sicherheit (Gruppe Member).
        /// </summary>
        [PrincipalPermission(SecurityAction.Demand, Role = GwlbLearningWcf.Members)]
        private static void RoleBaseSecurityDeclarativeMember()
        {
            Write(MethodInfo.GetCurrentMethod(), GwlbLearningWcf.Members);
        }

        /// <summary>
        /// Imperative rollenbasierte Sicherheit.
        /// </summary>
        /// <param name="strGroupName">Name der Gruppe, die geprüft werden soll.</param>
        private static void RoleBaseSecurityImperative(string strGroupName)
        {
            PrincipalPermission principalPerm = new PrincipalPermission(null, strGroupName);
            principalPerm.Demand();
            Write(MethodInfo.GetCurrentMethod(), strGroupName);
        }

        /// <summary>
        /// Schreibt eine Mitteilung auf den Bildschirm.
        /// </summary>
        /// <param name="objMethod">Name der Methode.</param>
        /// <param name="strGroup">Name der Gruppe.</param>
        private static void Write(MethodBase objMethod, string strGroup)
        {
            Console.WriteLine("Erfolgreicher Aufruf der Methode {0} mit Gruppe {1}.", objMethod.Name, strGroup);
        }

        /// <summary>
        /// Schreibt eine Mitteilung für eine nicht erfolgreiche Ausführung auf den Bildschirm.
        /// </summary>
        /// <param name="strMethodName">Name der Methode, die nicht ausgeführt werden konnte.</param>
        /// <param name="excObject">Ausnahme die gefangen wurde.</param>
        private static void WriteNotSuccessfull(string strMethodName, Exception excObject)
        {
            Console.WriteLine("NICHT erfolgreicher Aufruf der Methode '{0}'", strMethodName);
            Console.WriteLine(excObject.Message);
        }
    }
}
