﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WeroSoft.Samples.AssertLibrary
{
    /// <summary>
    /// Stellt Methoden für die Prüfung von Inhalten in Unit Tests zur Verfügung.
    /// </summary>
    public static class CwlbAssert
    {
        /// <summary>
        /// Prüft, ob ein short-Wert ungerade ist (mit Rest durch zwei teilbar).
        /// </summary>
        /// <param name="sValue">Wert der geprüft werden soll.</param>
        /// <param name="strMessage">Mitteilung für die Ausgabe im Fehlerfall.</param>
        /// <param name="aobjParams">Parameter für die Ausgabe im Fehlerfall.</param>
        public static void IsOdd(short sValue, string strMessage = null, params object[] aobjParams)
        {
            IsOdd((long)sValue, strMessage, aobjParams);
        }

        /// <summary>
        /// Prüft, ob ein int-Wert ungerade ist (mit Rest durch zwei teilbar).
        /// </summary>
        /// <param name="iValue">Wert der geprüft werden soll.</param>
        /// <param name="strMessage">Mitteilung für die Ausgabe im Fehlerfall.</param>
        /// <param name="aobjParams">Parameter für die Ausgabe im Fehlerfall.</param>
        public static void IsOdd(int iValue, string strMessage = null, params object[] aobjParams)
        {
            IsOdd((long)iValue, strMessage, aobjParams);
        }

        /// <summary>
        /// Prüft, ob ein long-Wert ungerade ist (mit Rest durch zwei teilbar).
        /// </summary>
        /// <param name="lValue">Wert der geprüft werden soll.</param>
        /// <param name="strMessage">Mitteilung für die Ausgabe im Fehlerfall.</param>
        /// <param name="aobjParams">Parameter für die Ausgabe im Fehlerfall.</param>
        public static void IsOdd(long lValue, string strMessage = null, params object[] aobjParams)
        {
            if (lValue % 2 == 0)
            {
                if (string.IsNullOrEmpty(strMessage))
                {
                    throw new AssertFailedException("Der Wert '{0}' definiert eine durch 2 teilbare Zahl.");
                }
                else
                {
                    throw new AssertFailedException(string.Format(strMessage, aobjParams));
                }
            }
        }

        /// <summary>
        /// Prüft, ob ein short-Wert gerade ist (ohne Rest durch zwei teilbar).
        /// </summary>
        /// <param name="sValue">Wert der geprüft werden soll.</param>
        /// <param name="strMessage">Mitteilung für die Ausgabe im Fehlerfall.</param>
        /// <param name="aobjParams">Parameter für die Ausgabe im Fehlerfall.</param>
        public static void IsEven(short sValue, string strMessage = null, params object[] aobjParams)
        {
            IsEven((long)sValue, strMessage, aobjParams);
        }

        /// <summary>
        /// Prüft, ob ein int-Wert gerade ist (ohne Rest durch zwei teilbar).
        /// </summary>
        /// <param name="sValue">Wert der geprüft werden soll.</param>
        /// <param name="strMessage">Mitteilung für die Ausgabe im Fehlerfall.</param>
        /// <param name="aobjParams">Parameter für die Ausgabe im Fehlerfall.</param>
        public static void IsEven(int iValue, string strMessage = null, params object[] aobjParams)
        {
            IsEven((long)iValue, strMessage, aobjParams);
        }

        /// <summary>
        /// Prüft, ob ein long-Wert gerade ist (ohne Rest durch zwei teilbar).
        /// </summary>
        /// <param name="sValue">Wert der geprüft werden soll.</param>
        /// <param name="strMessage">Mitteilung für die Ausgabe im Fehlerfall.</param>
        /// <param name="aobjParams">Parameter für die Ausgabe im Fehlerfall.</param>
        public static void IsEven(long lValue, string strMessage = null, params object[] aobjParams)
        {
            if (lValue % 2 != 0)
            {
                if (string.IsNullOrEmpty(strMessage))
                {
                    throw new AssertFailedException("Der Wert '{0}' definiert nicht eine durch 2 teilbare Zahl.");
                }
                else
                {
                    throw new AssertFailedException(string.Format(strMessage, aobjParams));
                }
            }
        }
    }
}
