﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WeroSoft.Samples.LearningUnitTest.Test
{
    /// <summary>
    /// Enthält den Code für die Initialisierung und die Bereinigung der Testassembly.
    /// </summary>
    [TestClass]
    public static class CwlbAssemblyLevelHandling
    {
        /// <summary>
        /// Definiert einen Dateinamen für eine Logdatei.
        /// </summary>
        public const string LogFileName = "TestExecutionLog.log";

        /// <summary>
        /// Definiert die Testkategorie Checkin.
        /// </summary>
        public const string CategoryCheckinTest = "CheckinTest";

        /// <summary>
        /// Definiert die Testkategorie Iteration.
        /// </summary>
        public const string CategoryIterationTest = "IterationTest";

        /// <summary>
        /// Definiert die Testkategorie Product.
        /// </summary>
        public const string CategoryProductTest = "ProductTest";

        /// <summary>
        /// Definiert eine hohe Priorität für den Test.
        /// </summary>
        public const int High = 3;

        /// <summary>
        /// Definiert eine mittlere Priorität für den Test.
        /// </summary>
        public const int Mid = 2;

        /// <summary>
        /// Definiert eine tiefe Priorität für den Test.
        /// </summary>
        public const int Low = 1;

        /// <summary>
        /// Initialisierung der Tests auf Stufe Assembly.
        /// </summary>
        /// <param name="objContext">Testkontext.</param>
        [AssemblyInitialize]
        public static void AssemblyInit(TestContext objContext)
        {
            Debug.WriteLine("AssemblyInit " + objContext.TestName);
        }

        /// <summary>
        /// Bereinigung auf Stufe Assembly.
        /// </summary>
        [AssemblyCleanup]
        public static void AssemblyCleanup()
        {
            Debug.WriteLine("AssemblyCleanup");
        }

        /// <summary>
        /// Schreibt eine Ausgabe in eine Datei.
        /// </summary>
        /// <param name="objContext">Zu verwendender Testkontext.</param>
        /// <param name="strFormat">Format der Ausgabe.</param>
        /// <param name="aobjParams">Daten der Ausgabe.</param>
        public static void WriteLog(TestContext objContext, string strFormat, params object[] aobjParams)
        {
            string strFileName = Path.Combine(objContext.TestRunResultsDirectory, LogFileName);
            objContext.AddResultFile(strFileName);
            File.AppendAllText(strFileName, string.Format("{0} {1}{2}",
                DateTime.Now.ToString("HH:mm:ss.fff"), 
                string.Format(strFormat, aobjParams),
                Environment.NewLine));
        }
    }
}
