﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using WeroSoft.Samples.AssertLibrary;

namespace WeroSoft.Samples.LearningUnitTest.Test
{
    /// <summary>
    /// Tests die Prüfmethode Assert.IsValidDataContract.
    /// </summary>
    [TestClass]
    public class CwlbDataContractTest
    {
        /// <summary>
        /// Tested einen ungültigen Vertragstyp.
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(AssertFailedException))]
        public void InvalidTypeTest()
        {
            Type typeInvalid = typeof(IConvertible);
            CwlbDataContractAssert.IsValidDataContract(typeInvalid, null);
        }

        /// <summary>
        /// Prüft eine Klasse als Datenklasse 
        /// </summary>
        [TestMethod]
        public void ImplizitValidDataContractTest()
        {
            CwlbDataContractAssert.IsValidDataContract(typeof(CwlbImplicitValidContract), null);
            CwlbDataContractAssert.IsValidDataContract(typeof(EwlbImplicitValidContract), null);
            CwlbDataContractAssert.IsValidDataContract(typeof(SwlbImplicitValidContract), null);
        }

        /// <summary>
        /// Testet die falsche Verwendung eines DataMember Attributs.
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(AssertFailedException))]
        public void ImplizitInvalidDataContractTest()
        {
            CwlbDataContractAssert.IsValidDataContract(typeof(CwlbImplicitInvalidContract), null);
        }

        /// <summary>
        /// Prüft eine explizit deklarierte Klasse als Datenklasse 
        /// </summary>
        [TestMethod]
        public void ExplicitValidDataContractTest()
        {
            CwlbDataContractAssert.IsValidDataContract(typeof(CwlbExplicitValidContract), null);
        }

        [TestMethod]
        public void SerializationTest()
        {
            CwlbExplicitValidContract objData = new CwlbExplicitValidContract();
            objData.TestField = "TestField";
            objData.TestStringProperty = "TestProperty";
            objData.TestDoubleProperty = Math.PI * Math.E;
            objData.TestDateTimeProperty = DateTime.Now;

            CwlbExplicitValidContract objDeserialized = CwlbDataContractAssert.IsDataContractSerializable<CwlbExplicitValidContract>(objData);

            Assert.AreEqual<string>(objData.TestField, objDeserialized.TestField, "Der Inhalt des Testfeldes ist nicht korrekt.");
            Assert.AreNotSame(objData.TestField, objDeserialized.TestField, "Die Referenzen der Inhalte des Testfeldes sind gleich.");
            Assert.AreEqual<double>(objData.TestDoubleProperty, objDeserialized.TestDoubleProperty, "Der Inhalt der double Eigenschaft ist nicht korrekt.");
            Assert.AreNotSame(objData.TestDoubleProperty, objDeserialized.TestDoubleProperty, "Die Referenzen der Inhalte des Testfeldes sind gleich.");
        }
    }
}
