﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Text.RegularExpressions;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WeroSoft.Samples.LearningUnitTest.Test
{
    /// <summary>
    /// Testet die Klasse CwlbDemo.
    /// </summary>
    [TestClass]
    public class CwlbDemoTest
    {
        /// <summary>
        /// Liefert oder definiert den Testkontext.
        /// </summary>
        public TestContext TestContext { get; set; }

        /// <summary>
        /// Testen der Methode MyProductiveMethod.
        /// </summary>
        [TestMethod]
        public void MyProductiveMethodTest()
        {
            // ------------------------------------------------------
            // Positiv Tests
            // ------------------------------------------------------
            CwlbDemo objDemo = new CwlbDemo();
            int iResult = objDemo.DemoMethod(0, 0, 0, "{0} * {1} * {2} = {3}");
            Assert.AreEqual<int>(0, iResult, "Das Resultat sollte den Wert 0 aufweisen");
            iResult = objDemo.DemoMethod(30, 0, 0, "{0} * {1} * {2} = {3}");
            Assert.AreEqual<int>(0, iResult, "Das Resultat sollte den Wert 0 aufweisen");
            iResult = objDemo.DemoMethod(60, 0, 0, "{0} * {1} * {2} = {3}");
            Assert.AreEqual<int>(0, iResult, "Das Resultat sollte den Wert 0 aufweisen");
            iResult = objDemo.DemoMethod(0, -30, 0, "{0} * {1} * {2} = {3}");
            Assert.AreEqual<int>(0, iResult, "Das Resultat sollte den Wert 0 aufweisen");
            iResult = objDemo.DemoMethod(0, +30, 0, "{0} * {1} * {2} = {3}");
            Assert.AreEqual<int>(0, iResult, "Das Resultat sollte den Wert 0 aufweisen");
            iResult = objDemo.DemoMethod(0, 0, int.MinValue, "{0} * {1} * {2} = {3}");
            Assert.AreEqual<int>(0, iResult, "Das Resultat sollte den Wert 0 aufweisen");
            iResult = objDemo.DemoMethod(0, 0, int.MaxValue, "{0} * {1} * {2} = {3}");
            Assert.AreEqual<int>(0, iResult, "Das Resultat sollte den Wert 0 aufweisen");
        }

        /// <summary>
        /// Demonstriert die Verwendung des Attributes DeploymentItemAttribute
        /// </summary>
        [DeploymentItem("..\\..\\TestDeployment\\TestDataFile.txt")]
        [TestMethod]
        public void TestDeploymentTest()
        {
            if (!File.Exists("TestDataFile.txt"))
            {
                Assert.Fail("Die Datei mit den Testdaten wurde nicht korrekt verteilt.");
            }

            string[] astrLines = File.ReadAllLines("TestDataFile.txt");
            Assert.AreEqual<int>(1, astrLines.Length, "Die Anzahl der Zeilen der Testdaten ist nicht wie erwartet.");
        }


        /// <summary>
        /// Demonstriert die Ausführung eines datengetriebenen Tests.
        /// </summary>
        [DeploymentItem("..\\..\\TestDeployment\\DataDrivenTestDemo.xml")]
        [DataSource("Microsoft.VisualStudio.TestTools.DataSource.XML", "DataDrivenTestDemo.xml", "Calculation", DataAccessMethod.Sequential)]
        [TestMethod]
        public void XmlDataDrivenDemoTest()
        {
            int iParam1 = Convert.ToInt32(TestContext.DataRow["Param1"]);
            int iParam2 = Convert.ToInt32(TestContext.DataRow["Param2"]);
            int iParam3 = Convert.ToInt32(TestContext.DataRow["Param3"]);
            int iExpected = Convert.ToInt32(TestContext.DataRow["Result"]);

            int iResult = CwlbDemo.Calculate(iParam1, iParam2, iParam3);
            Assert.AreEqual<int>(iExpected, iResult, "");
            TestContext.WriteLine("Executed Test: {0} * {1} * {2} = {3}", iParam1, iParam2, iParam3, iResult);
        }

        /// <summary>
        /// Demonstriert die Ausführung eines datengetriebenen Tests.
        /// </summary>
        /// <remarks>
        /// Die Konfiguration dieses Tests erfolgt über die Anwendungskonfiguration.
        /// </remarks>
        [DeploymentItem("..\\..\\TestDeployment\\XmlDemoUsingConfiguration.xml")]
        [DataSource("XmlDemoFile")]
        [TestMethod]
        public void XmlDataDrivenDemoUsingConfigTest()
        {
            int iParam1 = Convert.ToInt32(TestContext.DataRow["Param1"]);
            int iParam2 = Convert.ToInt32(TestContext.DataRow["Param2"]);
            int iParam3 = Convert.ToInt32(TestContext.DataRow["Param3"]);
            int iExpected = Convert.ToInt32(TestContext.DataRow["Result"]);

            int iResult = CwlbDemo.Calculate(iParam1, iParam2, iParam3);
            Assert.AreEqual<int>(iExpected, iResult, "");
            TestContext.WriteLine("Executed Test: {0} * {1} * {2} = {3}", iParam1, iParam2, iParam3, iResult);
        }

        /// <summary>
        /// Demonstriert die Ausführung eines datengetriebenen Tests.
        /// </summary>
        /// <remarks>
        /// Die Konfiguration dieses Tests erfolgt über die Anwendungskonfiguration.
        /// </remarks>
        [DeploymentItem("..\\..\\TestDeployment\\ExcelDemoUsingConfiguration.xls")]
        [DataSource("ExcelDemoFile")]
        [TestMethod]
        public void ExcelDataDrivenDemoUsingConfigTest()
        {
            int iParam1 = Convert.ToInt32(TestContext.DataRow["Param1"]);
            int iParam2 = Convert.ToInt32(TestContext.DataRow["Param2"]);
            int iParam3 = Convert.ToInt32(TestContext.DataRow["Param3"]);
            int iExpected = Convert.ToInt32(TestContext.DataRow["Result"]);

            int iResult = CwlbDemo.Calculate(iParam1, iParam2, iParam3);
            Assert.AreEqual<int>(iExpected, iResult, "");
            TestContext.WriteLine("Executed Test: {0} * {1} * {2} = {3}", iParam1, iParam2, iParam3, iResult);
        }

        /// <summary>
        /// Zeigt die Verwendung der Assertklassen.
        /// </summary>
        [TestMethod]
        public void AssertSimpleTest()
        {
            // Verwendete Daten
            string strResult = "TEST";
            string strResultOther = "TEST OTHER";
            string strExpected = "TEST";
            string strNull = null;
            string strResultLower = strResult.ToLower();
            double dResult = 19.9901;
            double dExpected = 20.0;
            double dRange = 0.01;
            TimeSpan tsMeasured = TimeSpan.FromMilliseconds(123456);
            TimeSpan tsExpected = TimeSpan.FromMilliseconds(123000);
            TimeSpan tsRange = TimeSpan.FromMilliseconds(1000);
            bool bResult = true;

            // Klasse Assert
            Assert.AreEqual(strExpected, strResult, "Die beiden Objekte sind gleich referenziert.");
            Assert.AreEqual(dExpected, dResult, dRange, "Der berechnete Wert liegt außerhalb der akzeptierbaren Grenzen");
            Assert.AreEqual(tsExpected.Ticks, tsMeasured.Ticks, tsRange.Ticks,"Die gemessene Zeit liegt ausserhalb der akzeptierbaren Grenzen");
            Assert.AreEqual<string>(strExpected, strResult, "Die Zeichenfolge ist nicht identisch.");
            Assert.AreEqual(strExpected, strResultLower, true, "Die Zeichenfolgen sind nicht gleich.");
            Assert.AreNotEqual<string>(strExpected, strResultOther, "Die Zeichenfolgen sind identisch.");
            Assert.AreSame(strExpected, strResult, "Es handelt sich nicht um die gleichen Objekte.");
            Assert.AreNotSame(strExpected, strResultOther, "Es handelt sich um die gleichen Objekte.");
            Assert.IsNotNull(strResult, "Das gelieferte Resultat darf nicht null sein.");
            Assert.IsNull(strNull, "Das Resultat muss eine Null-Referenz sein.");
            Assert.IsInstanceOfType(strResult, typeof(string), "Der Typ des Resultats ist nicht wie erwartet.");
            Assert.IsInstanceOfType(strResult, typeof(IEnumerable<char>), "Der Typ des Resultats ist nicht wie erwartet");
            Assert.IsTrue(bResult, "Das Resultat ist nicht wahr.");

            // Einfache Ergänzungen (Beispiele)
            Assert.IsTrue(char.IsUpper(strResult[0]), "Das erste Zeichen ist nicht ein Großbuchstabe");
            Assert.IsTrue(strResult.StartsWith("TE", StringComparison.InvariantCultureIgnoreCase), "Das Resultat beginnt nicht mit den Buchstaben 'TE'.");

            // Klasse StringAssert
            StringAssert.StartsWith(strResult, "T", "Die Zeichenfolge beginnt nicht mit einem 'T'.");
            StringAssert.Contains(strResult, "ES", "Die Zeichenfolge enthält die Buchstaben 'ES'.");
            StringAssert.EndsWith(strResult, "T", "Die Zeichenfolge endet nicht mit einem 'T'.");
            StringAssert.Matches(strResult, new Regex(@"^\w"), "Die Zeichenfolge beginnt mit einem Buchstaben.");
            StringAssert.DoesNotMatch(strResult, new Regex(@"^\W"), "Die Zeichenfolge beginnt nicht mit einem Buchstaben");

            // Verwendete Daten
            ICollection cobjResult = new List<string> {"ABC", "DEF", "FGH", "IJK"};
            ICollection cobjExpected = new List<string> { "ABC", "DEF", "FGH", "IJK" };
            ICollection cobjTotal = new List<string> { "ABC", "FGH", "IJK", "UVW", "XYZ", "DEF" };

            // Klasse CollectionAssert
            CollectionAssert.AllItemsAreInstancesOfType(cobjResult, typeof(string),
                "Die Auflistung entält auch Objekte, die nicht dem Typ string entsprechen");
            CollectionAssert.AllItemsAreInstancesOfType(cobjResult, typeof(IComparable),
                "Die Auflistung entält auch Objekte, die nicht dem Typ IComparable entsprechen");
            CollectionAssert.AllItemsAreNotNull(cobjResult, "Die Auflistung enthält Null-Referenzen.");
            CollectionAssert.AllItemsAreUnique(cobjResult, "Die Auflistung enthält Duplikate");
            CollectionAssert.AreEqual(cobjExpected, cobjResult, "Die Listen sind nicht gleich.");
            CollectionAssert.AreEquivalent(cobjExpected, cobjResult, "Die Listen sind nicht äquivalent.");
            CollectionAssert.Contains(cobjResult, "ABC", "Das Element 'ABC' kommt nicht in der Auflistung vor.");
            CollectionAssert.DoesNotContain(cobjResult, "UVW", "Das Element 'UVW' kommt in der Auflistung vor.");
            CollectionAssert.IsSubsetOf(cobjResult, cobjTotal, "Die Auflistung ist keine Untermenge von cobjTotal");
        }

        /// <summary>
        /// Demonstriert einen deklarativen Negativtest.
        /// </summary>
        [ExpectedException(typeof(DirectoryNotFoundException))]
        [TestMethod]
        public void DeclarativeNegativeDemoTest()
        {
            // Prüft die Erstellung einer Datei
            File.AppendAllText(@"C:\1F0E0746-4CF9-4D4F-84AC-FEB1BE08F0CB\Test.txt", "TEST");
        }

        /// <summary>
        /// Demonstriert einen deklarativen Negativtest mit einer Basisklasse.
        /// </summary>
        [ExpectedException(typeof(IOException), AllowDerivedTypes=true)]
        [TestMethod]
        public void DeclarativeNegativeBaseClassDemoTest()
        {
            // Prüft die Erstellung einer Datei
            File.AppendAllText(@"C:\1F0E0746-4CF9-4D4F-84AC-FEB1BE08F0CB\Test.txt", "TEST");
        }

        /// <summary>
        /// Demonstriert einen deklarativen Negativtest.
        /// </summary>
        [TestMethod]
        public void ImperativeNegativeDemoTest()
        {
            try
            {
                // Prüft die Erstellung einer Datei
                File.AppendAllText(@"C:\1F0E0746-4CF9-4D4F-84AC-FEB1BE08F0CB\Test.txt", "TEST");
                // Notbremse ziehen
                Assert.Fail("Eine interne Prüfung wurde nicht korrekt durchgeführt.");
            }
            catch (DirectoryNotFoundException)
            {
                // Das ist der korrekte Pfad für den Fehlerfall. Keine Aktion auslösen.
            }
            catch (Exception)
            {
                // Das ist die falsche Ausnahme. Sie können diese Fehlerbehandlung auch 
                // weglassen, da sowieso ein Fehler vorhanden ist und der Test nun als
                // nicht erfolgreich gilt. Durch die Behandlung des Fehlers können Sie 
                // aber auch noch Einfluss auf die Fehlermeldung nehmen.
                throw;
            }
        }
    }
}
