﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WeroSoft.Samples.LearningUnitTest.Test
{
    /// <summary>
    /// Demonstriert die Verwendung der Attribute für Unit Tests
    /// </summary>
    [TestClass]
    public class CwlbTestClassA
    {
        /// <summary>
        /// Liefert oder definiert den Testcontext.
        /// </summary>
        public TestContext TestContext { get; set; }

        /// <summary>
        /// Initialisierung auf Stufe der Klasse.
        /// </summary>
        static CwlbTestClassA()
        {
            Debug.WriteLine("Class A: Statischer Konstruktor");
        }

        /// <summary>
        /// Initilisiert eine neue Instanz der Testklasse.
        /// </summary>
        public CwlbTestClassA()
        {
            Debug.WriteLine("Class A: Instanz-Konstruktor");
        }

        /// <summary>
        /// Methode für die Initialisierung der Testklasse.
        /// </summary>
        /// <param name="objContext">Testkontext.</param>
        [ClassInitialize]
        public static void ClassInit(TestContext objContext)
        {
            Debug.WriteLine("Class A: Init " + objContext.TestName);
        }

        /// <summary>
        /// Methode für die Initialisierung einer Testmethode.
        /// </summary>
        [TestInitialize]
        public void Initialize()
        {
            Debug.WriteLine("Class A: Init TestMethod");
        }

        /// <summary>
        /// Methode für die Bereinigung des Aufrufs einer Testmethode.
        /// </summary>
        [TestCleanup]
        public void Cleanup()
        {
            Debug.WriteLine("Class A: Cleanup TestMethod");
        }

        /// <summary>
        /// Methode für die Bereinigung einer Testklasse.
        /// </summary>
        [ClassCleanup()]
        public static void ClassCleanup()
        {
            Debug.WriteLine("Class A: Cleanup");
        }

        /// <summary>
        /// Testmethode A1 innerhalb der Testklasse.
        /// </summary>
        [TestMethod]
        public void TestMethodA1()
        {
            Debug.WriteLine("Class A: TestMethodeA1: " + TestContext.TestName );
        }

        /// <summary>
        /// Testmethode A2 innerhalb der Testklasse.
        /// </summary>
        [Owner("Rolf Wenger")]
        [Description("Demonstriert die Verwendung von einigen Attributen einer Testmethode")]
        [TestMethod]
        public void TestMethodA2()
        {
            Debug.WriteLine("Class A: TestMethodeA2: " + TestContext.TestName);
        }
    }
}
