﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace WeroSoft.Samples.LearningUnitTest.Test
{
    /// <summary>
    /// Demonstriert die Verwendung der Attribute für Unit Tests
    /// </summary>
    [TestClass]
    public class CwlbTestClassB
    {
        /// <summary>
        /// Liefert oder definiert den Testcontext.
        /// </summary>
        public TestContext TestContext { get; set; }

        /// <summary>
        /// Methode für die Initialisierung der Testklasse.
        /// </summary>
        /// <param name="objContext">Testkontext.</param>
        [ClassInitialize]
        public static void ClassInit(TestContext objContext)
        {
            Debug.WriteLine("Class B: Init " + objContext.TestName);
        }

        /// <summary>
        /// Methode für die Initialisierung einer Testmethode.
        /// </summary>
        [TestInitialize]
        public void Initialize()
        {
            Debug.WriteLine("Class B: Init TestMethod");
        }

        /// <summary>
        /// Methode für die Bereinigung des Aufrufs einer Testmethode.
        /// </summary>
        [TestCleanup]
        public void Cleanup()
        {
            Debug.WriteLine("Class B: Cleanup TestMethod");
        }

        /// <summary>
        /// Methode für die Bereinigung einer Testklasse.
        /// </summary>
        [ClassCleanup()]
        public static void ClassCleanup()
        {
            Debug.WriteLine("Class B: Cleanup");
        }

        /// <summary>
        /// Testmethode B1 innerhalb der Testklasse.
        /// </summary>
        [TestMethod]
        public void TestMethodB1()
        {
            Debug.WriteLine("Class B: TestMethodeB1: " + TestContext.TestName );
        }

        /// <summary>
        /// Testmethode B2 innerhalb der Testklasse.
        /// </summary>
        [Owner("Rolf Wenger")]
        [Description("Demonstriert die Verwendung von einigen Attributen einer Testmethode")]
        [TestMethod]
        public void TestMethodB2()
        {
            Debug.WriteLine("Class B: TestMethodeB2: " + TestContext.TestName);
        }
    }
}
