﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WeroSoft.Samples.LearningUnitTest
{
    /// <summary>
    /// Bibliotheksklasse für den Umgang mit einem Unit Test
    /// </summary>
    public class CwlbDemo
    {
        /// <summary>
        /// Berechnet das Produkt aus drei Werten und gibt das Resultat auf der Konsole aus.
        /// </summary>
        /// <param name="iParam1">Wert zwischen 30 bis und mit 60.</param>
        /// <param name="iParam2">Wert zwischen -30 bis und mit +30.</param>
        /// <param name="iParam3">Beliebiger int Wert.</param>
        /// <param name="strFormat">Format für die Ausgabe</param>
        /// <returns>Int Wert.</returns>
        public int DemoMethod(int iParam1, int iParam2, int iParam3, string strFormat)
        {
            // Prüfung der Parameter
            if (iParam1 < 30 && iParam1 > 60)
            {
                throw new ArgumentException("iParam1 soll zwischen 30 und 60 sein.");
            }

            if (iParam2 < -30 && iParam2 > +30)
            {
                throw new ArgumentException("iParam2 soll zwischen -30 und +30 sein.");
            }

            if (string.IsNullOrWhiteSpace(strFormat))
            {
                throw new ArgumentNullException("strFormat darf nicht null oder leer sein, oder nur Whitspaces enthalten.");
            }

            if (strFormat.Count(chFormat => chFormat == '{') != 4)
            {
                throw new ArgumentException("Das Format sollte 4 Platzhalter enthalten.");
            }

            // Verarbeitung
            int iResult = Calculate(iParam1, iParam2, iParam3);
            Console.WriteLine(strFormat, iParam1, iParam2, iParam3, iResult);
            return iResult;
        }

        /// <summary>
        /// Berechnet das Produkt aus drei Werten.
        /// </summary>
        /// <param name="iParam1">Beliebiger int Wert.</param>
        /// <param name="iParam2">Beliebiger int Wert.</param>
        /// <param name="iParam3">Beliebiger int Wert.</param>
        /// <returns>Int Wert.</returns>
        public static int Calculate(int iParam1, int iParam2, int iParam3)
        {
            return iParam1 * iParam2 * iParam3;
        }
    }
}
