﻿using WeroSoft.Samples;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;

namespace WeroSoft.Samples
{
    /// <summary>
    /// Testklasse für Klasse CwlbCsDemoData.
    /// </summary>
    [TestClass()]
    public class CwlbDemoDataTest
    {
        /// <summary>
        /// Liefert oder definiert den TestContext des UnitTest.
        /// </summary>
        public TestContext TestContext {get; set;}

        /// <summary>
        /// Tested die Erstellung eines Objekts.
        /// </summary>
        [TestMethod()]
        public void CwlbCsDemoDataConstructorTest()
        {
            // Zufallszahlengenerator
            Random objRandom = new Random();

            // Erstellen des Demodatenobjekts
            CwlbDemoData objTarget = CwlbDemoData.Create(objRandom);
            Assert.AreNotEqual<Guid>(Guid.Empty, objTarget.ID, "Die ID wurde nicht korrekt initialisiert.");
            Assert.IsNotNull(objTarget.Name, "Der Name des Objekts soll definiert sein.");
            Assert.IsNull(objTarget.BinaryData, "Es sollten keine binären Daten vorhanden sein.");
            Assert.AreEqual<int>(1, CwlbDemoData.InstanceCounter, "Der Instancecounter sollte 1 sein.");

            // Modifizieren des Objekts ohne Ergänzung von Daten
            objTarget = CwlbDemoData.Modify(objTarget);
            Assert.IsNull(objTarget.BinaryData, "Es sollten keine binären Daten vorhanden sein.");

            // Erstellen eines Datenobjekts mit binären Daten.
            objTarget = CwlbDemoData.Create(objRandom, 1024);
            Assert.IsNotNull(objTarget.BinaryData, "Es sollten binären Daten vorhanden sein.");
            Assert.AreEqual<int>(2, CwlbDemoData.InstanceCounter, "Der Instancecounter sollte 2 sein.");

            // Lesen des ersten und des letzten Bytes
            byte byteFirst = objTarget.BinaryData[0];
            byte byteLast = objTarget.BinaryData[1023];

            objTarget = CwlbDemoData.Modify(objTarget);
            Assert.IsNotNull(objTarget.BinaryData, "Es sollten binären Daten vorhanden sein.");
            Assert.AreEqual<int>(1024, objTarget.BinaryData.Length, "Die Länge der erzeugten Daten ist nicht korrekt.");

            // Vergleichen der Daten
            Assert.AreEqual<byte>(byteFirst, objTarget.BinaryData[1023], "Die Bytes scheinen nicht umgedreht worden zu sein.");
            Assert.AreEqual<byte>(byteLast, objTarget.BinaryData[0], "Die Bytes scheinen nicht umgedreht worden zu sein.");

            // Test der Serialisierung und Deserialiserung
            objTarget = SerializingTest(objTarget);
            Assert.AreEqual<int>(3, CwlbDemoData.InstanceCounter, "Der Instancecounter sollte 3 sein.");
        }

        /// <summary>
        /// Tested die Serialisierung und deserialisierung des Objekts.
        /// </summary>
        /// <param name="objTest">Zu serialisierendes Objekt.</param>
        /// <returns>Deserialisiertes Objekt</returns>
        public CwlbDemoData SerializingTest(CwlbDemoData objTest)
        {
            using (var stream = new MemoryStream())
            {
                var formatter = new BinaryFormatter();
                formatter.Serialize(stream, objTest);
                stream.Position = 0;
                return (CwlbDemoData)formatter.Deserialize(stream);
            }
        }
    }
}
